/*****************************************************************************
 * Copyright (C) 2019 VLC authors and VideoLAN
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * ( at your option ) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/
#ifndef PLAYLIST_COMMON_HPP
#define PLAYLIST_COMMON_HPP

#include <QObject>

extern "C" {
    typedef struct vlc_playlist vlc_playlist_t;
}

class Playlist
{
public:
    Playlist() = default;
    explicit Playlist(vlc_playlist_t * const playlist)
        : m_playlist(playlist)
    { };

    vlc_playlist_t * m_playlist = nullptr;
};

Q_DECLARE_METATYPE(Playlist)

#endif // PLAYLIST_COMMON_HPP
