#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_Widgets_LabelExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ActionButtonOverlay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ActionButtonPrimary_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_BannerTabButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_BannerCone_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_BusyIndicatorExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CaptionLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CheckButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CheckedDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ComboBoxExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_AcrylicBackground_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_FloatingNotification_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_AnimatedBackground_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDThemeButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDThemeButtonSet_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDWindowButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDWindowButtonSet_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDTitlebarTapNDrapHandler_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CSDMouseStealer_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CurrentIndicator_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DragItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DrawerExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ExpandGridView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ExpandGridItemView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_EmptyLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_EmptyLabelButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_EmptyLabelHint_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_GridItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_HorizontalResizeHandle_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_IconLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_IconButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_IconToolButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_IconTrackButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ImageToolButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ListViewExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TableViewExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TableViewDelegateExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TableRowDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TableHeaderDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DefaultShadow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ListLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_LocalTabBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_MediaCover_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_SubtitleLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_MenuCaption_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_MenuLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_NavigableCol_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_NavigableRow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_PlayCover_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_RoundButtonExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ScaledImage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TextAutoScroller_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_SearchBox_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_SliderExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_SortControl_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_SpinBoxExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_StackViewExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_PageLoader_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ButtonExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TableColumns_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TextFieldExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TextToolButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_TrackDelayButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_VideoProgressBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_VideoQualityLabels_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ListSubtitleLabel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_PointingTooltip_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_FrostedGlassEffect_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ToolTipExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DropShadowImage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DoubleShadow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_FadingEdge_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_FadingEdgeForItemView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_PopupIconToolButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_PartialEffect_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ViewHeader_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ProgressIndicator_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ImageExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_EnhancedImageExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ScrollBarExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_FastBlend_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_RadioButtonExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_RoundedRectangleShadow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_VoronoiSnow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_DualKawaseBlur_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_CheckBoxExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Widgets_ProgressBarExt_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/LabelExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_LabelExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ActionButtonOverlay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ActionButtonOverlay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ActionButtonPrimary.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ActionButtonPrimary_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/BannerTabButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_BannerTabButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/BannerCone.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_BannerCone_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/BusyIndicatorExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_BusyIndicatorExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CaptionLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CaptionLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CheckButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CheckButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CheckedDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CheckedDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ComboBoxExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ComboBoxExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/AcrylicBackground.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_AcrylicBackground_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/FloatingNotification.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_FloatingNotification_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/AnimatedBackground.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_AnimatedBackground_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDThemeButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDThemeButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDThemeButtonSet.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDThemeButtonSet_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDWindowButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDWindowButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDWindowButtonSet.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDWindowButtonSet_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDTitlebarTapNDrapHandler.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDTitlebarTapNDrapHandler_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CSDMouseStealer.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CSDMouseStealer_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CurrentIndicator.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CurrentIndicator_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DragItem.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DragItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DrawerExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DrawerExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ExpandGridView.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ExpandGridView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ExpandGridItemView.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ExpandGridItemView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/EmptyLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_EmptyLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/EmptyLabelButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_EmptyLabelButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/EmptyLabelHint.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_EmptyLabelHint_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/GridItem.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_GridItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/HorizontalResizeHandle.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_HorizontalResizeHandle_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/IconLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_IconLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/IconButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_IconButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/IconToolButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_IconToolButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/IconTrackButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_IconTrackButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ImageToolButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ImageToolButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ListViewExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ListViewExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TableViewExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TableViewExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TableViewDelegateExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TableViewDelegateExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TableRowDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TableRowDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TableHeaderDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TableHeaderDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DefaultShadow.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DefaultShadow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ListLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ListLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/LocalTabBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_LocalTabBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/MediaCover.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_MediaCover_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/SubtitleLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_SubtitleLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/MenuCaption.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_MenuCaption_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/MenuLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_MenuLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/NavigableCol.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_NavigableCol_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/NavigableRow.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_NavigableRow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/PlayCover.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_PlayCover_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/RoundButtonExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_RoundButtonExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ScaledImage.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ScaledImage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TextAutoScroller.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TextAutoScroller_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/SearchBox.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_SearchBox_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/SliderExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_SliderExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/SortControl.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_SortControl_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/SpinBoxExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_SpinBoxExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/StackViewExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_StackViewExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/PageLoader.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_PageLoader_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ButtonExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ButtonExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TableColumns.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TableColumns_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TextFieldExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TextFieldExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TextToolButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TextToolButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/TrackDelayButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_TrackDelayButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/VideoProgressBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_VideoProgressBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/VideoQualityLabels.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_VideoQualityLabels_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ListSubtitleLabel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ListSubtitleLabel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/PointingTooltip.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_PointingTooltip_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/FrostedGlassEffect.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_FrostedGlassEffect_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ToolTipExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ToolTipExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DropShadowImage.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DropShadowImage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DoubleShadow.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DoubleShadow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/FadingEdge.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_FadingEdge_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/FadingEdgeForItemView.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_FadingEdgeForItemView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/PopupIconToolButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_PopupIconToolButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/PartialEffect.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_PartialEffect_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ViewHeader.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ViewHeader_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ProgressIndicator.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ProgressIndicator_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ImageExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ImageExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/EnhancedImageExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_EnhancedImageExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ScrollBarExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ScrollBarExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/FastBlend.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_FastBlend_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/RadioButtonExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_RadioButtonExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/RoundedRectangleShadow.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_RoundedRectangleShadow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/VoronoiSnow.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_VoronoiSnow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/DualKawaseBlur.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_DualKawaseBlur_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/CheckBoxExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_CheckBoxExt_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Widgets/ProgressBarExt.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Widgets_ProgressBarExt_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_widgets_cachegen)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_widgets_cachegen))
int QT_MANGLE_NAMESPACE(qCleanupResources_widgets_cachegen)() {
    return 1;
}
