/* Generated by wayland-scanner 1.25.0 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_buffer_interface;
extern const struct wl_interface wl_output_interface;

static const struct wl_interface *screenshooter_types[] = {
	&wl_output_interface,
	&wl_buffer_interface,
};

static const struct wl_message screenshooter_requests[] = {
	{ "shoot", "oo", screenshooter_types + 0 },
};

static const struct wl_message screenshooter_events[] = {
	{ "done", "", screenshooter_types + 0 },
};

WL_PRIVATE const struct wl_interface screenshooter_interface = {
	"screenshooter", 1,
	1, screenshooter_requests,
	1, screenshooter_events,
};

