/* A Bison parser, made by GNU Bison 3.8.2.  */

/* Bison interface for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

#ifndef YY_JSON_DEMUX_JSON_GRAMMAR_H_INCLUDED
# define YY_JSON_DEMUX_JSON_GRAMMAR_H_INCLUDED
/* Debug traces.  */
#ifndef JSONDEBUG
# if defined YYDEBUG
#if YYDEBUG
#   define JSONDEBUG 1
#  else
#   define JSONDEBUG 0
#  endif
# else /* ! defined YYDEBUG */
#  define JSONDEBUG 0
# endif /* ! defined YYDEBUG */
#endif  /* ! defined JSONDEBUG */
#if JSONDEBUG
extern int jsondebug;
#endif

/* Token kinds.  */
#ifndef JSONTOKENTYPE
# define JSONTOKENTYPE
  enum jsontokentype
  {
    JSONEMPTY = -2,
    JSONEOF = 0,                   /* YYEOF  */
    JSONerror = 256,               /* error  */
    JSONUNDEF = 257,               /* "invalid token"  */
    VALUE_NULL = 258,              /* VALUE_NULL  */
    BOOLEAN = 259,                 /* BOOLEAN  */
    NUMBER = 260,                  /* NUMBER  */
    STRING = 261                   /* STRING  */
  };
  typedef enum jsontokentype jsontoken_kind_t;
#endif
/* Token kinds.  */
#define JSONEMPTY -2
#define JSONEOF 0
#define JSONerror 256
#define JSONUNDEF 257
#define VALUE_NULL 258
#define BOOLEAN 259
#define NUMBER 260
#define STRING 261

/* Value type.  */
#if ! defined JSONSTYPE && ! defined JSONSTYPE_IS_DECLARED
union JSONSTYPE
{
#line 122 "demux/json/grammar.y"

	bool boolean;
	double number;
	char *string;
	struct json_value value;
	struct json_array array;
	struct json_member member;
	struct json_object object;

#line 97 "demux/json/grammar.h"

};
typedef union JSONSTYPE JSONSTYPE;
# define JSONSTYPE_IS_TRIVIAL 1
# define JSONSTYPE_IS_DECLARED 1
#endif




int jsonparse (void *opaque, void *scanner, struct json_object *result);


#endif /* !YY_JSON_DEMUX_JSON_GRAMMAR_H_INCLUDED  */
