/****************************************************************************
** Meta object code from reading C++ file 'mainctx.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mainctx.hpp"
#include "dialogs/toolbar/controlbar_profile_model.hpp"
#include "util/csdbuttonmodel.hpp"
#include "playlist/playlist_controller.hpp"
#include "maininterface/mainctx_submodels.hpp"
#include "maininterface/videosurface.hpp"
#include "medialibrary/medialib.hpp"
#include "player/player_controller.hpp"
#include "util/color_scheme_model.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainctx.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7MainCtxE_t {};
} // unnamed namespace

template <> constexpr inline auto MainCtx::qt_create_metaobjectdata<qt_meta_tag_ZN7MainCtxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainCtx",
        "fullscreenInterfaceToggled",
        "",
        "setInterfaceVisibible",
        "setInterfaceFullScreen",
        "toggleWindowVisibility",
        "askToQuit",
        "askShow",
        "askBoss",
        "askRaise",
        "kc_pressed",
        "playlistDockedChanged",
        "playlistVisibleChanged",
        "playlistWidthFactorChanged",
        "playerPlaylistFactorChanged",
        "interfaceAlwaysOnTopChanged",
        "hasEmbededVideoChanged",
        "showRemainingTimeChanged",
        "gridViewChanged",
        "hasGridListModeChanged",
        "groupingChanged",
        "Grouping",
        "albumSectionsChanged",
        "lyricsModeChanged",
        "colorSchemeChanged",
        "useClientSideDecorationChanged",
        "hasToolbarMenuChanged",
        "requestInterfaceMaximized",
        "requestInterfaceNormal",
        "requestInterfaceMinimized",
        "intfScaleFactorChanged",
        "intfDevicePixelRatioChanged",
        "pinVideoControlsChanged",
        "pinOpacityChanged",
        "hasAcrylicSurfaceChanged",
        "acrylicActiveChanged",
        "smoothScrollChanged",
        "preferHotkeysChanged",
        "useGlobalShortcutsChanged",
        "maxVolumeChanged",
        "safeAreaChanged",
        "mouseHideTimeoutChanged",
        "navBoxToggled",
        "bgConeToggled",
        "windowSuportExtendedFrameChanged",
        "windowExtendedMarginChanged",
        "margin",
        "artistAlbumsWidthFactorChanged",
        "mainInterfaceModesChanged",
        "MainInterfaceModes",
        "usingTouchChanged",
        "toggleToolbarMenu",
        "toggleInterfaceFullScreen",
        "setPlaylistDocked",
        "setPlaylistVisible",
        "setPlaylistWidthFactor",
        "setPlayerPlaylistWidthFactor",
        "factor",
        "setInterfaceAlwaysOnTop",
        "setShowRemainingTime",
        "setGridView",
        "setGrouping",
        "setAlbumSections",
        "setLyricsMode",
        "incrementIntfUserScaleFactor",
        "increment",
        "setIntfUserScaleFactor",
        "setHasToolbarMenu",
        "setPinVideoControls",
        "setPinOpacity",
        "setbgCone",
        "updateIntfScaleFactor",
        "onWindowVisibilityChanged",
        "QWindow::Visibility",
        "setHasAcrylicSurface",
        "setMinimalView",
        "setPlayerView",
        "sendHotkey",
        "Qt::Key",
        "key",
        "Qt::KeyboardModifiers",
        "modifiers",
        "sendVLCHotkey",
        "vlcHotkey",
        "emitBoss",
        "emitRaise",
        "emitShow",
        "reloadPrefs",
        "getExtraInterfaces",
        "VLCVarChoiceModel*",
        "pasteFromClipboard",
        "onInputChanged",
        "getIntfUserScaleFactor",
        "backdropBlurRequested",
        "setCursor",
        "Qt::CursorShape",
        "cursor",
        "restoreCursor",
        "QQuickItem*",
        "item",
        "unsetCursor",
        "qtVersion",
        "qtVersionCheck",
        "major",
        "minor",
        "patch",
        "qtQuickControlRejectsHoverEvents",
        "urlListToMimeData",
        "QJSValue",
        "array",
        "setFiltersChildMouseEvents",
        "enable",
        "setItemFlag",
        "QQuickItem::Flag",
        "flag",
        "enabled",
        "effectiveDevicePixelRatio",
        "const QQuickWindow*",
        "window",
        "windowHasDepthBuffer",
        "const QWindow*",
        "setTimeout",
        "func",
        "delay",
        "QJSValueList",
        "params",
        "const QObject*",
        "context",
        "platformHandlesResizeWithCSD",
        "platformHandlesTitleBarButtonsWithCSD",
        "platformHandlesShadowsWithCSD",
        "settingValue",
        "QVariant",
        "defaultValue",
        "setSettingValue",
        "value",
        "setAttachedToolTip",
        "toolTip",
        "dp",
        "px",
        "scale",
        "useXmasCone",
        "folderMRL",
        "QUrl",
        "fileMRL",
        "displayMRL",
        "mrl",
        "playlistDocked",
        "playlistVisible",
        "playlistWidthFactor",
        "playerPlaylistWidthFactor",
        "artistAlbumsWidthFactor",
        "interfaceAlwaysOnTop",
        "hasEmbededVideo",
        "showRemainingTime",
        "extraInterfaces",
        "intfScaleFactor",
        "mediaLibraryAvailable",
        "mediaLibrary",
        "MediaLib*",
        "gridView",
        "hasGridListMode",
        "grouping",
        "colorScheme",
        "ColorSchemeModel*",
        "hasVLM",
        "clientSideDecoration",
        "hasFirstrun",
        "csdBorderSize",
        "hasToolbarMenu",
        "canShowVideoPIP",
        "pinVideoControls",
        "pinOpacity",
        "controlbarProfileModel",
        "ControlbarProfileModel*",
        "hasAcrylicSurface",
        "smoothScroll",
        "intfMainWindow",
        "QWindow*",
        "useGlobalShortcuts",
        "maxVolume",
        "safeArea",
        "videoSurfaceProvider",
        "VideoSurfaceProvider*",
        "mouseHideTimeout",
        "albumSections",
        "lyricsMode",
        "usingTouch",
        "csdButtonModel",
        "CSDButtonModel*",
        "mainInterfaceModes",
        "effectiveMainInterfaceMode",
        "MainInterfaceMode",
        "initialEffectiveMainInterfaceMode",
        "osName",
        "OsType",
        "osVersion",
        "minimalView",
        "playerView",
        "acrylicActive",
        "preferHotkeys",
        "bgCone",
        "windowSuportExtendedFrame",
        "windowExtendedMargin",
        "search",
        "SearchCtx*",
        "sort",
        "SortCtx*",
        "GROUPING_NONE",
        "GROUPING_NAME",
        "GROUPING_FOLDER",
        "Windows",
        "Unknown",
        "MAININTERFACE_MODE_INVALID",
        "MAININTERFACE_MODE_MAINDISPLAY",
        "MAININTERFACE_MODE_PLAYER",
        "MAININTERFACE_MODE_MINIMAL"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fullscreenInterfaceToggled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'setInterfaceVisibible'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'setInterfaceFullScreen'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'toggleWindowVisibility'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'askToQuit'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'askShow'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'askBoss'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'askRaise'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'kc_pressed'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'playlistDockedChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'playlistVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'playlistWidthFactorChanged'
        QtMocHelpers::SignalData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'playerPlaylistFactorChanged'
        QtMocHelpers::SignalData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'interfaceAlwaysOnTopChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'hasEmbededVideoChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'showRemainingTimeChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'gridViewChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'hasGridListModeChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'groupingChanged'
        QtMocHelpers::SignalData<void(enum Grouping)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 2 },
        }}),
        // Signal 'albumSectionsChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'lyricsModeChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'colorSchemeChanged'
        QtMocHelpers::SignalData<void(QString)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'useClientSideDecorationChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasToolbarMenuChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestInterfaceMaximized'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestInterfaceNormal'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestInterfaceMinimized'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'intfScaleFactorChanged'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'intfDevicePixelRatioChanged'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pinVideoControlsChanged'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pinOpacityChanged'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasAcrylicSurfaceChanged'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'acrylicActiveChanged'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'smoothScrollChanged'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferHotkeysChanged'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'useGlobalShortcutsChanged'
        QtMocHelpers::SignalData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'maxVolumeChanged'
        QtMocHelpers::SignalData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'safeAreaChanged'
        QtMocHelpers::SignalData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseHideTimeoutChanged'
        QtMocHelpers::SignalData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'navBoxToggled'
        QtMocHelpers::SignalData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bgConeToggled'
        QtMocHelpers::SignalData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowSuportExtendedFrameChanged'
        QtMocHelpers::SignalData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowExtendedMarginChanged'
        QtMocHelpers::SignalData<void(unsigned )>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 46 },
        }}),
        // Signal 'artistAlbumsWidthFactorChanged'
        QtMocHelpers::SignalData<void(double)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'mainInterfaceModesChanged'
        QtMocHelpers::SignalData<void(MainInterfaceModes)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 49, 2 },
        }}),
        // Signal 'usingTouchChanged'
        QtMocHelpers::SignalData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleToolbarMenu'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleInterfaceFullScreen'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPlaylistDocked'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPlaylistVisible'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPlaylistWidthFactor'
        QtMocHelpers::SlotData<void(double)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setPlayerPlaylistWidthFactor'
        QtMocHelpers::SlotData<void(double)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 57 },
        }}),
        // Slot 'setInterfaceAlwaysOnTop'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setShowRemainingTime'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setGridView'
        QtMocHelpers::SlotData<void(bool)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setGrouping'
        QtMocHelpers::SlotData<void(enum Grouping)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 2 },
        }}),
        // Slot 'setAlbumSections'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setLyricsMode'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'incrementIntfUserScaleFactor'
        QtMocHelpers::SlotData<void(bool)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 65 },
        }}),
        // Slot 'setIntfUserScaleFactor'
        QtMocHelpers::SlotData<void(double)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setHasToolbarMenu'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPinVideoControls'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPinOpacity'
        QtMocHelpers::SlotData<void(float)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 2 },
        }}),
        // Slot 'setbgCone'
        QtMocHelpers::SlotData<void(bool)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateIntfScaleFactor'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onWindowVisibilityChanged'
        QtMocHelpers::SlotData<void(QWindow::Visibility)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 73, 2 },
        }}),
        // Slot 'setHasAcrylicSurface'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setMinimalView'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setPlayerView'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'sendHotkey'
        QtMocHelpers::SlotData<void(Qt::Key, Qt::KeyboardModifiers)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 78, 79 }, { 0x80000000 | 80, 81 },
        }}),
        // Slot 'sendVLCHotkey'
        QtMocHelpers::SlotData<void(int)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 83 },
        }}),
        // Slot 'emitBoss'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitRaise'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitShow'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadPrefs'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getExtraInterfaces'
        QtMocHelpers::SlotData<VLCVarChoiceModel *()>(88, 2, QMC::AccessPublic, 0x80000000 | 89),
        // Slot 'pasteFromClipboard'
        QtMocHelpers::SlotData<bool()>(90, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'onInputChanged'
        QtMocHelpers::SlotData<void(bool)>(91, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Method 'getIntfUserScaleFactor'
        QtMocHelpers::MethodData<double() const>(92, 2, QMC::AccessPublic, QMetaType::Double),
        // Method 'backdropBlurRequested'
        QtMocHelpers::MethodData<bool() const>(93, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'setCursor'
        QtMocHelpers::MethodData<void(Qt::CursorShape)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 95, 96 },
        }}),
        // Method 'restoreCursor'
        QtMocHelpers::MethodData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setCursor'
        QtMocHelpers::MethodData<void(QQuickItem *, Qt::CursorShape)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 98, 99 }, { 0x80000000 | 95, 96 },
        }}),
        // Method 'unsetCursor'
        QtMocHelpers::MethodData<void(QQuickItem *)>(100, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 98, 99 },
        }}),
        // Method 'qtVersion'
        QtMocHelpers::MethodData<unsigned int()>(101, 2, QMC::AccessPublic, QMetaType::UInt),
        // Method 'qtVersionCheck'
        QtMocHelpers::MethodData<unsigned int(unsigned char, unsigned char, unsigned char)>(102, 2, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::UChar, 103 }, { QMetaType::UChar, 104 }, { QMetaType::UChar, 105 },
        }}),
        // Method 'qtQuickControlRejectsHoverEvents'
        QtMocHelpers::MethodData<bool()>(106, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'urlListToMimeData'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &)>(107, 2, QMC::AccessPublic, 0x80000000 | 108, {{
            { 0x80000000 | 108, 109 },
        }}),
        // Method 'setFiltersChildMouseEvents'
        QtMocHelpers::MethodData<void(QQuickItem *, bool)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 98, 99 }, { QMetaType::Bool, 111 },
        }}),
        // Method 'setItemFlag'
        QtMocHelpers::MethodData<void(QQuickItem *, QQuickItem::Flag, bool)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 98, 99 }, { 0x80000000 | 113, 114 }, { QMetaType::Bool, 115 },
        }}),
        // Method 'setItemFlag'
        QtMocHelpers::MethodData<void(QQuickItem *, QQuickItem::Flag)>(112, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 98, 99 }, { 0x80000000 | 113, 114 },
        }}),
        // Method 'effectiveDevicePixelRatio'
        QtMocHelpers::MethodData<qreal(const QQuickWindow *)>(116, 2, QMC::AccessPublic, QMetaType::QReal, {{
            { 0x80000000 | 117, 118 },
        }}),
        // Method 'windowHasDepthBuffer'
        QtMocHelpers::MethodData<bool(const QWindow *)>(119, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 120, 118 },
        }}),
        // Method 'setTimeout'
        QtMocHelpers::MethodData<void(const QJSValue &, const int, const QJSValueList &, const QObject *)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 108, 122 }, { QMetaType::Int, 123 }, { 0x80000000 | 124, 125 }, { 0x80000000 | 126, 127 },
        }}),
        // Method 'setTimeout'
        QtMocHelpers::MethodData<void(const QJSValue &, const int, const QJSValueList &)>(121, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 108, 122 }, { QMetaType::Int, 123 }, { 0x80000000 | 124, 125 },
        }}),
        // Method 'setTimeout'
        QtMocHelpers::MethodData<void(const QJSValue &, const int)>(121, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 108, 122 }, { QMetaType::Int, 123 },
        }}),
        // Method 'platformHandlesResizeWithCSD'
        QtMocHelpers::MethodData<bool() const>(128, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'platformHandlesTitleBarButtonsWithCSD'
        QtMocHelpers::MethodData<bool() const>(129, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'platformHandlesShadowsWithCSD'
        QtMocHelpers::MethodData<bool() const>(130, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'settingValue'
        QtMocHelpers::MethodData<QVariant(const QString &, const QVariant &) const>(131, 2, QMC::AccessPublic, 0x80000000 | 132, {{
            { QMetaType::QString, 79 }, { 0x80000000 | 132, 133 },
        }}),
        // Method 'setSettingValue'
        QtMocHelpers::MethodData<void(const QString &, const QVariant &)>(134, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 79 }, { 0x80000000 | 132, 135 },
        }}),
        // Method 'setAttachedToolTip'
        QtMocHelpers::MethodData<void(QObject *)>(136, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 137 },
        }}),
        // Method 'dp'
        QtMocHelpers::MethodData<double(const double, const double)>(138, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Double, 139 }, { QMetaType::Double, 140 },
        }}),
        // Method 'dp'
        QtMocHelpers::MethodData<double(const double) const>(138, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Double, 139 },
        }}),
        // Method 'useXmasCone'
        QtMocHelpers::MethodData<bool() const>(141, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'folderMRL'
        QtMocHelpers::MethodData<QUrl(const QString &) const>(142, 2, QMC::AccessPublic, 0x80000000 | 143, {{
            { QMetaType::QString, 144 },
        }}),
        // Method 'folderMRL'
        QtMocHelpers::MethodData<QUrl(const QUrl &) const>(142, 2, QMC::AccessPublic, 0x80000000 | 143, {{
            { 0x80000000 | 143, 144 },
        }}),
        // Method 'displayMRL'
        QtMocHelpers::MethodData<QString(const QUrl &) const>(145, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 143, 146 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'playlistDocked'
        QtMocHelpers::PropertyData<bool>(147, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'playlistVisible'
        QtMocHelpers::PropertyData<bool>(148, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'playlistWidthFactor'
        QtMocHelpers::PropertyData<double>(149, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'playerPlaylistWidthFactor'
        QtMocHelpers::PropertyData<double>(150, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'artistAlbumsWidthFactor'
        QtMocHelpers::PropertyData<double>(151, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 43),
        // property 'interfaceAlwaysOnTop'
        QtMocHelpers::PropertyData<bool>(152, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'hasEmbededVideo'
        QtMocHelpers::PropertyData<bool>(153, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 14),
        // property 'showRemainingTime'
        QtMocHelpers::PropertyData<bool>(154, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 15),
        // property 'extraInterfaces'
        QtMocHelpers::PropertyData<VLCVarChoiceModel*>(155, 0x80000000 | 89, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'intfScaleFactor'
        QtMocHelpers::PropertyData<double>(156, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Final, 27),
        // property 'mediaLibraryAvailable'
        QtMocHelpers::PropertyData<bool>(157, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'mediaLibrary'
        QtMocHelpers::PropertyData<MediaLib*>(158, 0x80000000 | 159, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'gridView'
        QtMocHelpers::PropertyData<bool>(160, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 16),
        // property 'hasGridListMode'
        QtMocHelpers::PropertyData<bool>(161, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Final, 17),
        // property 'grouping'
        QtMocHelpers::PropertyData<enum Grouping>(162, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 18),
        // property 'colorScheme'
        QtMocHelpers::PropertyData<ColorSchemeModel*>(163, 0x80000000 | 164, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'hasVLM'
        QtMocHelpers::PropertyData<bool>(165, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'clientSideDecoration'
        QtMocHelpers::PropertyData<bool>(166, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 22),
        // property 'hasFirstrun'
        QtMocHelpers::PropertyData<bool>(167, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'csdBorderSize'
        QtMocHelpers::PropertyData<int>(168, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 22),
        // property 'hasToolbarMenu'
        QtMocHelpers::PropertyData<bool>(169, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 23),
        // property 'canShowVideoPIP'
        QtMocHelpers::PropertyData<bool>(170, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'pinVideoControls'
        QtMocHelpers::PropertyData<bool>(171, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 29),
        // property 'pinOpacity'
        QtMocHelpers::PropertyData<float>(172, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 30),
        // property 'controlbarProfileModel'
        QtMocHelpers::PropertyData<ControlbarProfileModel*>(173, 0x80000000 | 174, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'hasAcrylicSurface'
        QtMocHelpers::PropertyData<bool>(175, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 31),
        // property 'smoothScroll'
        QtMocHelpers::PropertyData<bool>(176, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 33),
        // property 'intfMainWindow'
        QtMocHelpers::PropertyData<QWindow*>(177, 0x80000000 | 178, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'useGlobalShortcuts'
        QtMocHelpers::PropertyData<bool>(179, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 35),
        // property 'maxVolume'
        QtMocHelpers::PropertyData<int>(180, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 36),
        // property 'safeArea'
        QtMocHelpers::PropertyData<float>(181, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Final, 37),
        // property 'videoSurfaceProvider'
        QtMocHelpers::PropertyData<VideoSurfaceProvider*>(182, 0x80000000 | 183, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 14),
        // property 'mouseHideTimeout'
        QtMocHelpers::PropertyData<int>(184, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 38),
        // property 'albumSections'
        QtMocHelpers::PropertyData<bool>(185, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 19),
        // property 'lyricsMode'
        QtMocHelpers::PropertyData<bool>(186, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 20),
        // property 'usingTouch'
        QtMocHelpers::PropertyData<bool>(187, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 45),
        // property 'csdButtonModel'
        QtMocHelpers::PropertyData<CSDButtonModel*>(188, 0x80000000 | 189, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'mainInterfaceModes'
        QtMocHelpers::PropertyData<MainInterfaceModes>(190, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 44),
        // property 'effectiveMainInterfaceMode'
        QtMocHelpers::PropertyData<enum MainInterfaceMode>(191, 0x80000000 | 192, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 44),
        // property 'initialEffectiveMainInterfaceMode'
        QtMocHelpers::PropertyData<enum MainInterfaceMode>(193, 0x80000000 | 192, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'osName'
        QtMocHelpers::PropertyData<enum OsType>(194, 0x80000000 | 195, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'osVersion'
        QtMocHelpers::PropertyData<int>(196, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'minimalView'
        QtMocHelpers::PropertyData<bool>(197, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 44),
        // property 'playerView'
        QtMocHelpers::PropertyData<bool>(198, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 44),
        // property 'acrylicActive'
        QtMocHelpers::PropertyData<bool>(199, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 32),
        // property 'preferHotkeys'
        QtMocHelpers::PropertyData<bool>(200, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 34),
        // property 'bgCone'
        QtMocHelpers::PropertyData<bool>(201, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Final, 40),
        // property 'windowSuportExtendedFrame'
        QtMocHelpers::PropertyData<bool>(202, QMetaType::Bool, QMC::DefaultPropertyFlags, 41),
        // property 'windowExtendedMargin'
        QtMocHelpers::PropertyData<uint>(203, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 42),
        // property 'search'
        QtMocHelpers::PropertyData<SearchCtx*>(204, 0x80000000 | 205, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'sort'
        QtMocHelpers::PropertyData<SortCtx*>(206, 0x80000000 | 207, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Grouping'
        QtMocHelpers::EnumData<enum Grouping>(21, 21, QMC::EnumFlags{}).add({
            {  208, Grouping::GROUPING_NONE },
            {  209, Grouping::GROUPING_NAME },
            {  210, Grouping::GROUPING_FOLDER },
        }),
        // enum 'OsType'
        QtMocHelpers::EnumData<enum OsType>(195, 195, QMC::EnumFlags{}).add({
            {  211, OsType::Windows },
            {  212, OsType::Unknown },
        }),
        // enum 'MainInterfaceMode'
        QtMocHelpers::EnumData<enum MainInterfaceMode>(192, 192, QMC::EnumFlags{}).add({
            {  213, MainInterfaceMode::MAININTERFACE_MODE_INVALID },
            {  214, MainInterfaceMode::MAININTERFACE_MODE_MAINDISPLAY },
            {  215, MainInterfaceMode::MAININTERFACE_MODE_PLAYER },
            {  216, MainInterfaceMode::MAININTERFACE_MODE_MINIMAL },
        }),
    };
    return QtMocHelpers::metaObjectData<MainCtx, qt_meta_tag_ZN7MainCtxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainCtx::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7MainCtxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7MainCtxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7MainCtxE_t>.metaTypes,
    nullptr
} };

void MainCtx::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainCtx *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fullscreenInterfaceToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setInterfaceVisibible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setInterfaceFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->toggleWindowVisibility(); break;
        case 4: _t->askToQuit(); break;
        case 5: _t->askShow(); break;
        case 6: _t->askBoss(); break;
        case 7: _t->askRaise(); break;
        case 8: _t->kc_pressed(); break;
        case 9: _t->playlistDockedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->playlistVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->playlistWidthFactorChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->playerPlaylistFactorChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->interfaceAlwaysOnTopChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->hasEmbededVideoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->showRemainingTimeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->gridViewChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->hasGridListModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->groupingChanged((*reinterpret_cast<std::add_pointer_t<enum Grouping>>(_a[1]))); break;
        case 19: _t->albumSectionsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->lyricsModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->colorSchemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->useClientSideDecorationChanged(); break;
        case 23: _t->hasToolbarMenuChanged(); break;
        case 24: _t->requestInterfaceMaximized(); break;
        case 25: _t->requestInterfaceNormal(); break;
        case 26: _t->requestInterfaceMinimized(); break;
        case 27: _t->intfScaleFactorChanged(); break;
        case 28: _t->intfDevicePixelRatioChanged(); break;
        case 29: _t->pinVideoControlsChanged(); break;
        case 30: _t->pinOpacityChanged(); break;
        case 31: _t->hasAcrylicSurfaceChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->acrylicActiveChanged(); break;
        case 33: _t->smoothScrollChanged(); break;
        case 34: _t->preferHotkeysChanged(); break;
        case 35: _t->useGlobalShortcutsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->maxVolumeChanged(); break;
        case 37: _t->safeAreaChanged(); break;
        case 38: _t->mouseHideTimeoutChanged(); break;
        case 39: _t->navBoxToggled(); break;
        case 40: _t->bgConeToggled(); break;
        case 41: _t->windowSuportExtendedFrameChanged(); break;
        case 42: _t->windowExtendedMarginChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 43: _t->artistAlbumsWidthFactorChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 44: _t->mainInterfaceModesChanged((*reinterpret_cast<std::add_pointer_t<MainInterfaceModes>>(_a[1]))); break;
        case 45: _t->usingTouchChanged(); break;
        case 46: _t->toggleToolbarMenu(); break;
        case 47: _t->toggleInterfaceFullScreen(); break;
        case 48: _t->setPlaylistDocked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->setPlaylistVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->setPlaylistWidthFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 51: _t->setPlayerPlaylistWidthFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 52: _t->setInterfaceAlwaysOnTop((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->setShowRemainingTime((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->setGridView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: _t->setGrouping((*reinterpret_cast<std::add_pointer_t<enum Grouping>>(_a[1]))); break;
        case 56: _t->setAlbumSections((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: _t->setLyricsMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 58: _t->incrementIntfUserScaleFactor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 59: _t->setIntfUserScaleFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 60: _t->setHasToolbarMenu((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: _t->setPinVideoControls((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: _t->setPinOpacity((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 63: _t->setbgCone((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->updateIntfScaleFactor(); break;
        case 65: _t->onWindowVisibilityChanged((*reinterpret_cast<std::add_pointer_t<QWindow::Visibility>>(_a[1]))); break;
        case 66: _t->setHasAcrylicSurface((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: _t->setMinimalView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->setPlayerView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->sendHotkey((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2]))); break;
        case 70: _t->sendVLCHotkey((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 71: _t->emitBoss(); break;
        case 72: _t->emitRaise(); break;
        case 73: _t->emitShow(); break;
        case 74: _t->reloadPrefs(); break;
        case 75: { VLCVarChoiceModel* _r = _t->getExtraInterfaces();
            if (_a[0]) *reinterpret_cast<VLCVarChoiceModel**>(_a[0]) = std::move(_r); }  break;
        case 76: { bool _r = _t->pasteFromClipboard();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 77: _t->onInputChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: { double _r = _t->getIntfUserScaleFactor();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 79: { bool _r = _t->backdropBlurRequested();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 80: _t->setCursor((*reinterpret_cast<std::add_pointer_t<Qt::CursorShape>>(_a[1]))); break;
        case 81: _t->restoreCursor(); break;
        case 82: _t->setCursor((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::CursorShape>>(_a[2]))); break;
        case 83: _t->unsetCursor((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 84: { uint _r = _t->qtVersion();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 85: { uint _r = _t->qtVersionCheck((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 86: { bool _r = _t->qtQuickControlRejectsHoverEvents();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 87: { QJSValue _r = _t->urlListToMimeData((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 88: _t->setFiltersChildMouseEvents((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 89: _t->setItemFlag((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QQuickItem::Flag>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 90: _t->setItemFlag((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QQuickItem::Flag>>(_a[2]))); break;
        case 91: { qreal _r = _t->effectiveDevicePixelRatio((*reinterpret_cast<std::add_pointer_t<const QQuickWindow*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qreal*>(_a[0]) = std::move(_r); }  break;
        case 92: { bool _r = _t->windowHasDepthBuffer((*reinterpret_cast<std::add_pointer_t<const QWindow*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 93: _t->setTimeout((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValueList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[4]))); break;
        case 94: _t->setTimeout((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValueList>>(_a[3]))); break;
        case 95: _t->setTimeout((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 96: { bool _r = _t->platformHandlesResizeWithCSD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 97: { bool _r = _t->platformHandlesTitleBarButtonsWithCSD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 98: { bool _r = _t->platformHandlesShadowsWithCSD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 99: { QVariant _r = _t->settingValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 100: _t->setSettingValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 101: _t->setAttachedToolTip((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 102: { double _r = _t->dp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 103: { double _r = _t->dp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 104: { bool _r = _t->useXmasCone();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 105: { QUrl _r = _t->folderMRL((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 106: { QUrl _r = _t->folderMRL((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 107: { QString _r = _t->displayMRL((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 82:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 83:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 87:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 88:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 89:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 90:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 93:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 94:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 95:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::fullscreenInterfaceToggled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::setInterfaceVisibible, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::setInterfaceFullScreen, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::toggleWindowVisibility, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::askToQuit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::askShow, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::askBoss, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::askRaise, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::kc_pressed, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::playlistDockedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::playlistVisibleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(double )>(_a, &MainCtx::playlistWidthFactorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(double )>(_a, &MainCtx::playerPlaylistFactorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::interfaceAlwaysOnTopChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::hasEmbededVideoChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::showRemainingTimeChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::gridViewChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::hasGridListModeChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(Grouping )>(_a, &MainCtx::groupingChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::albumSectionsChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::lyricsModeChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(QString )>(_a, &MainCtx::colorSchemeChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::useClientSideDecorationChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::hasToolbarMenuChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::requestInterfaceMaximized, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::requestInterfaceNormal, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::requestInterfaceMinimized, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::intfScaleFactorChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::intfDevicePixelRatioChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::pinVideoControlsChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::pinOpacityChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::hasAcrylicSurfaceChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::acrylicActiveChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::smoothScrollChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::preferHotkeysChanged, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(bool )>(_a, &MainCtx::useGlobalShortcutsChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::maxVolumeChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::safeAreaChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::mouseHideTimeoutChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::navBoxToggled, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::bgConeToggled, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::windowSuportExtendedFrameChanged, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(unsigned  )>(_a, &MainCtx::windowExtendedMarginChanged, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(double )>(_a, &MainCtx::artistAlbumsWidthFactorChanged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)(MainInterfaceModes )>(_a, &MainCtx::mainInterfaceModesChanged, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainCtx::*)()>(_a, &MainCtx::usingTouchChanged, 45))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 27:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWindow* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isPlaylistDocked(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isPlaylistVisible(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getPlaylistWidthFactor(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getPlayerPlaylistWidthFactor(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->artistAlbumsWidthFactor(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isInterfaceAlwaysOnTop(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hasEmbededVideo(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isShowRemainingTime(); break;
        case 8: *reinterpret_cast<VLCVarChoiceModel**>(_v) = _t->getExtraInterfaces(); break;
        case 9: *reinterpret_cast<double*>(_v) = _t->getIntfScaleFactor(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->hasMediaLibrary(); break;
        case 11: *reinterpret_cast<MediaLib**>(_v) = _t->getMediaLibrary(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->hasGridView(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->m_hasGridListMode; break;
        case 14: *reinterpret_cast<enum Grouping*>(_v) = _t->grouping(); break;
        case 15: *reinterpret_cast<ColorSchemeModel**>(_v) = _t->getColorScheme(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->hasVLM(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->useClientSideDecoration(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->hasFirstrun(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->CSDBorderSize(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->hasToolbarMenu(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->canShowVideoPIP(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->pinVideoControls(); break;
        case 23: *reinterpret_cast<float*>(_v) = _t->pinOpacity(); break;
        case 24: *reinterpret_cast<ControlbarProfileModel**>(_v) = _t->controlbarProfileModel(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->hasAcrylicSurface(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->smoothScroll(); break;
        case 27: *reinterpret_cast<QWindow**>(_v) = _t->intfMainWindow(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->getUseGlobalShortcuts(); break;
        case 29: *reinterpret_cast<int*>(_v) = _t->maxVolume(); break;
        case 30: *reinterpret_cast<float*>(_v) = _t->safeArea(); break;
        case 31: *reinterpret_cast<VideoSurfaceProvider**>(_v) = _t->getVideoSurfaceProvider(); break;
        case 32: *reinterpret_cast<int*>(_v) = _t->mouseHideTimeout(); break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->albumSections(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->lyricsMode(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->usingTouch(); break;
        case 36: *reinterpret_cast<CSDButtonModel**>(_v) = _t->csdButtonModel(); break;
        case 37: *reinterpret_cast<MainInterfaceModes*>(_v) = _t->getMainInterfaceModes(); break;
        case 38: QtMocHelpers::assignFlags<enum MainInterfaceMode>(_v, _t->getEffectiveMainInterfaceMode()); break;
        case 39: QtMocHelpers::assignFlags<enum MainInterfaceMode>(_v, _t->getInitialEffectiveMainInterfaceMode()); break;
        case 40: *reinterpret_cast<enum OsType*>(_v) = _t->getOSName(); break;
        case 41: *reinterpret_cast<int*>(_v) = _t->getOSVersion(); break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->isMinimalView(); break;
        case 43: *reinterpret_cast<bool*>(_v) = _t->isPlayerView(); break;
        case 44: *reinterpret_cast<bool*>(_v) = _t->acrylicActive(); break;
        case 45: *reinterpret_cast<bool*>(_v) = _t->preferHotkeys(); break;
        case 46: *reinterpret_cast<bool*>(_v) = _t->isbgCone(); break;
        case 47: *reinterpret_cast<bool*>(_v) = _t->windowSuportExtendedFrame(); break;
        case 48: *reinterpret_cast<uint*>(_v) = _t->windowExtendedMargin(); break;
        case 49: *reinterpret_cast<SearchCtx**>(_v) = _t->m_search; break;
        case 50: *reinterpret_cast<SortCtx**>(_v) = _t->m_sort; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlaylistDocked(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setPlaylistVisible(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setPlaylistWidthFactor(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setPlayerPlaylistWidthFactor(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setArtistAlbumsWidthFactor(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setInterfaceAlwaysOnTop(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setShowRemainingTime(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setGridView(*reinterpret_cast<bool*>(_v)); break;
        case 13:
            if (QtMocHelpers::setProperty(_t->m_hasGridListMode, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->hasGridListModeChanged(_t->m_hasGridListMode);
            break;
        case 14: _t->setGrouping(*reinterpret_cast<enum Grouping*>(_v)); break;
        case 20: _t->setHasToolbarMenu(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setPinVideoControls(*reinterpret_cast<bool*>(_v)); break;
        case 23: _t->setPinOpacity(*reinterpret_cast<float*>(_v)); break;
        case 28: _t->setUseGlobalShortcuts(*reinterpret_cast<bool*>(_v)); break;
        case 31: _t->setVideoSurfaceProvider(*reinterpret_cast<VideoSurfaceProvider**>(_v)); break;
        case 33: _t->setAlbumSections(*reinterpret_cast<bool*>(_v)); break;
        case 34: _t->setLyricsMode(*reinterpret_cast<bool*>(_v)); break;
        case 42: _t->setMinimalView(*reinterpret_cast<bool*>(_v)); break;
        case 43: _t->setPlayerView(*reinterpret_cast<bool*>(_v)); break;
        case 44: _t->setAcrylicActive(*reinterpret_cast<bool*>(_v)); break;
        case 45: _t->setPreferHotkeys(*reinterpret_cast<bool*>(_v)); break;
        case 46: _t->setbgCone(*reinterpret_cast<bool*>(_v)); break;
        case 48: _t->setWindowExtendedMargin(*reinterpret_cast<uint*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MainCtx::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainCtx::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7MainCtxE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MainCtx::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 108)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 108;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 108)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 108;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    return _id;
}

// SIGNAL 0
void MainCtx::fullscreenInterfaceToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MainCtx::setInterfaceVisibible(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MainCtx::setInterfaceFullScreen(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MainCtx::toggleWindowVisibility()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MainCtx::askToQuit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MainCtx::askShow()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MainCtx::askBoss()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MainCtx::askRaise()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MainCtx::kc_pressed()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MainCtx::playlistDockedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void MainCtx::playlistVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void MainCtx::playlistWidthFactorChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void MainCtx::playerPlaylistFactorChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void MainCtx::interfaceAlwaysOnTopChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void MainCtx::hasEmbededVideoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void MainCtx::showRemainingTimeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void MainCtx::gridViewChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void MainCtx::hasGridListModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void MainCtx::groupingChanged(Grouping _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void MainCtx::albumSectionsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void MainCtx::lyricsModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void MainCtx::colorSchemeChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void MainCtx::useClientSideDecorationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void MainCtx::hasToolbarMenuChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void MainCtx::requestInterfaceMaximized()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void MainCtx::requestInterfaceNormal()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void MainCtx::requestInterfaceMinimized()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void MainCtx::intfScaleFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void MainCtx::intfDevicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void MainCtx::pinVideoControlsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void MainCtx::pinOpacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void MainCtx::hasAcrylicSurfaceChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void MainCtx::acrylicActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void MainCtx::smoothScrollChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void MainCtx::preferHotkeysChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void MainCtx::useGlobalShortcutsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void MainCtx::maxVolumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void MainCtx::safeAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void MainCtx::mouseHideTimeoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void MainCtx::navBoxToggled()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void MainCtx::bgConeToggled()
{
    QMetaObject::activate(this, &staticMetaObject, 40, nullptr);
}

// SIGNAL 41
void MainCtx::windowSuportExtendedFrameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 41, nullptr);
}

// SIGNAL 42
void MainCtx::windowExtendedMarginChanged(unsigned  _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void MainCtx::artistAlbumsWidthFactorChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void MainCtx::mainInterfaceModesChanged(MainInterfaceModes _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void MainCtx::usingTouchChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 45, nullptr);
}
QT_WARNING_POP
