#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_PlayerControls_HighResolutionTimeWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ArtworkInfoWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_AspectRatioWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_AtoBButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_BackButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_BookmarkButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ChapterNextButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ChapterPreviousButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_DvdMenuButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ExpandingSpacerWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ExtendedSettingsButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_FasterButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_FrameButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_FullscreenButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_InfoButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_LangButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_LoopButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_MenuButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_NavigationBoxButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_NavigationWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_NextButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_OpenButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_PlaybackSpeedButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_PlayButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_PlayerSwitchButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_PlaylistButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_PreviousButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ProgramButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_QuitButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_RandomButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_RendererButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_RecordButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_ReverseButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_SkipBackButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_SkipForwardButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_SlowerButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_SnapshotButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_SpacerWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_StopButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_TeletextButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_TeletextColorButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_TeletextWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_VolumeWidget_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_PlayerControls_Fallback_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/HighResolutionTimeWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_HighResolutionTimeWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ArtworkInfoWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ArtworkInfoWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/AspectRatioWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_AspectRatioWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/AtoBButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_AtoBButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/BackButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_BackButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/BookmarkButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_BookmarkButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ChapterNextButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ChapterNextButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ChapterPreviousButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ChapterPreviousButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/DvdMenuButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_DvdMenuButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ExpandingSpacerWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ExpandingSpacerWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ExtendedSettingsButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ExtendedSettingsButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/FasterButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_FasterButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/FrameButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_FrameButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/FullscreenButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_FullscreenButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/InfoButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_InfoButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/LangButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_LangButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/LoopButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_LoopButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/MenuButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_MenuButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/NavigationBoxButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_NavigationBoxButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/NavigationWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_NavigationWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/NextButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_NextButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/OpenButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_OpenButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/PlaybackSpeedButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_PlaybackSpeedButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/PlayButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_PlayButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/PlayerSwitchButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_PlayerSwitchButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/PlaylistButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_PlaylistButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/PreviousButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_PreviousButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ProgramButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ProgramButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/QuitButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_QuitButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/RandomButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_RandomButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/RendererButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_RendererButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/RecordButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_RecordButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/ReverseButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_ReverseButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/SkipBackButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_SkipBackButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/SkipForwardButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_SkipForwardButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/SlowerButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_SlowerButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/SnapshotButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_SnapshotButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/SpacerWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_SpacerWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/StopButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_StopButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/TeletextButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_TeletextButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/TeletextColorButton.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_TeletextColorButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/TeletextWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_TeletextWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/VolumeWidget.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_VolumeWidget_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/PlayerControls/Fallback.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_PlayerControls_Fallback_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_playercontrols_cachegen)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_playercontrols_cachegen))
int QT_MANGLE_NAMESPACE(qCleanupResources_playercontrols_cachegen)() {
    return 1;
}
