#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'sprefs_medialibrary.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef SPREFS_MEDIALIBRARY_H
#define SPREFS_MEDIALIBRARY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/native/mlfolderseditor.hpp"

QT_BEGIN_NAMESPACE

class Ui_SPrefsMediaLibrary
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *mlGroupBox;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *banButton;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *addButton;
    MLFoldersEditor *bannedEntryPoints;
    MLFoldersEditor *entryPoints;
    QLabel *label;
    QLabel *label_3;
    QPushButton *reloadButton;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SPrefsMediaLibrary)
    {
        if (SPrefsMediaLibrary->objectName().isEmpty())
            SPrefsMediaLibrary->setObjectName("SPrefsMediaLibrary");
        SPrefsMediaLibrary->resize(700, 535);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SPrefsMediaLibrary->sizePolicy().hasHeightForWidth());
        SPrefsMediaLibrary->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(SPrefsMediaLibrary);
        verticalLayout->setObjectName("verticalLayout");
        mlGroupBox = new QGroupBox(SPrefsMediaLibrary);
        mlGroupBox->setObjectName("mlGroupBox");
        gridLayout = new QGridLayout(mlGroupBox);
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(mlGroupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 3, 1, 2, 1);

        banButton = new QPushButton(mlGroupBox);
        banButton->setObjectName("banButton");
        banButton->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(banButton, 4, 3, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(627, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 1, 1, 1, 2);

        addButton = new QPushButton(mlGroupBox);
        addButton->setObjectName("addButton");
        addButton->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(addButton, 1, 3, 1, 1);

        bannedEntryPoints = new MLFoldersEditor(mlGroupBox);
        if (bannedEntryPoints->rowCount() < 1)
            bannedEntryPoints->setRowCount(1);
        bannedEntryPoints->setObjectName("bannedEntryPoints");
        bannedEntryPoints->setEditTriggers(QAbstractItemView::AllEditTriggers);
        bannedEntryPoints->setAlternatingRowColors(true);
        bannedEntryPoints->setShowGrid(false);
        bannedEntryPoints->setRowCount(1);
        bannedEntryPoints->horizontalHeader()->setProperty("showSortIndicator", QVariant(false));

        gridLayout->addWidget(bannedEntryPoints, 5, 0, 1, 4);

        entryPoints = new MLFoldersEditor(mlGroupBox);
        if (entryPoints->rowCount() < 1)
            entryPoints->setRowCount(1);
        entryPoints->setObjectName("entryPoints");
        entryPoints->setEditTriggers(QAbstractItemView::AllEditTriggers);
        entryPoints->setAlternatingRowColors(true);
        entryPoints->setShowGrid(false);
        entryPoints->setRowCount(1);
        entryPoints->horizontalHeader()->setProperty("showSortIndicator", QVariant(false));

        gridLayout->addWidget(entryPoints, 2, 0, 1, 4);

        label = new QLabel(mlGroupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 3, 0, 2, 1);

        label_3 = new QLabel(mlGroupBox);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        reloadButton = new QPushButton(mlGroupBox);
        reloadButton->setObjectName("reloadButton");
        reloadButton->setMaximumSize(QSize(18, 18));

        gridLayout->addWidget(reloadButton, 0, 3, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 0, 1, 1, 1);


        verticalLayout->addWidget(mlGroupBox);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QWidget::setTabOrder(entryPoints, addButton);
        QWidget::setTabOrder(addButton, bannedEntryPoints);
        QWidget::setTabOrder(bannedEntryPoints, banButton);

        retranslateUi(SPrefsMediaLibrary);

        QMetaObject::connectSlotsByName(SPrefsMediaLibrary);
    } // setupUi

    void retranslateUi(QWidget *SPrefsMediaLibrary)
    {
        SPrefsMediaLibrary->setWindowTitle(Q_("Form", nullptr));
        mlGroupBox->setTitle(Q_("Folders", nullptr));
        label_2->setText(Q_("Folders scanned by the Media Library:", nullptr));
#if QT_CONFIG(tooltip)
        banButton->setToolTip(Q_("Add banned folder", nullptr));
#endif // QT_CONFIG(tooltip)
        banButton->setText(Q_("+", nullptr));
#if QT_CONFIG(tooltip)
        addButton->setToolTip(Q_("Add scanned folder", nullptr));
#endif // QT_CONFIG(tooltip)
        addButton->setText(Q_("+", nullptr));
        label->setText(Q_("Banned folders:", nullptr));
        label_3->setText(Q_("Reload All Folders", nullptr));
#if QT_CONFIG(accessibility)
        reloadButton->setAccessibleName(Q_("Reload", nullptr));
#endif // QT_CONFIG(accessibility)
        reloadButton->setText(Q_("\342\237\263", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SPrefsMediaLibrary: public Ui_SPrefsMediaLibrary {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SPREFS_MEDIALIBRARY_H
