#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'vlm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef VLM_H
#define VLM_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Vlm
{
public:
    QGridLayout *gridLayout;
    QGroupBox *mediaConfBox;
    QGridLayout *gridLayout1;
    QComboBox *mediaType;
    QFrame *line;
    QLabel *nameLabel;
    QLineEdit *nameLedit;
    QCheckBox *enableCheck;
    QLabel *inputLabel;
    QLineEdit *inputLedit;
    QPushButton *inputButton;
    QLabel *outputLabel;
    QLineEdit *outputLedit;
    QPushButton *outputButton;
    QGroupBox *schedBox;
    QCheckBox *loopBCast;
    QSpacerItem *spacerItem;
    QPushButton *addButton;
    QPushButton *clearButton;
    QPushButton *saveButton;
    QSpacerItem *spacerItem1;
    QGroupBox *mediaBox;
    QGridLayout *gridLayout2;
    QListWidget *vlmListItem;
    QFrame *line_2;
    QScrollArea *vlmItemScroll;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *Vlm)
    {
        if (Vlm->objectName().isEmpty())
            Vlm->setObjectName("Vlm");
        Vlm->setWindowModality(Qt::NonModal);
        Vlm->resize(738, 693);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Vlm->sizePolicy().hasHeightForWidth());
        Vlm->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(Vlm);
        gridLayout->setObjectName("gridLayout");
        mediaConfBox = new QGroupBox(Vlm);
        mediaConfBox->setObjectName("mediaConfBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mediaConfBox->sizePolicy().hasHeightForWidth());
        mediaConfBox->setSizePolicy(sizePolicy1);
        gridLayout1 = new QGridLayout(mediaConfBox);
        gridLayout1->setObjectName("gridLayout1");
        mediaType = new QComboBox(mediaConfBox);
        mediaType->setObjectName("mediaType");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mediaType->sizePolicy().hasHeightForWidth());
        mediaType->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(mediaType, 0, 0, 1, 3);

        line = new QFrame(mediaConfBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout1->addWidget(line, 1, 0, 1, 3);

        nameLabel = new QLabel(mediaConfBox);
        nameLabel->setObjectName("nameLabel");

        gridLayout1->addWidget(nameLabel, 2, 0, 1, 1);

        nameLedit = new QLineEdit(mediaConfBox);
        nameLedit->setObjectName("nameLedit");

        gridLayout1->addWidget(nameLedit, 2, 1, 1, 1);

        enableCheck = new QCheckBox(mediaConfBox);
        enableCheck->setObjectName("enableCheck");
        enableCheck->setChecked(true);

        gridLayout1->addWidget(enableCheck, 2, 2, 1, 1);

        inputLabel = new QLabel(mediaConfBox);
        inputLabel->setObjectName("inputLabel");

        gridLayout1->addWidget(inputLabel, 3, 0, 1, 1);

        inputLedit = new QLineEdit(mediaConfBox);
        inputLedit->setObjectName("inputLedit");

        gridLayout1->addWidget(inputLedit, 3, 1, 1, 1);

        inputButton = new QPushButton(mediaConfBox);
        inputButton->setObjectName("inputButton");

        gridLayout1->addWidget(inputButton, 3, 2, 1, 1);

        outputLabel = new QLabel(mediaConfBox);
        outputLabel->setObjectName("outputLabel");

        gridLayout1->addWidget(outputLabel, 4, 0, 1, 1);

        outputLedit = new QLineEdit(mediaConfBox);
        outputLedit->setObjectName("outputLedit");

        gridLayout1->addWidget(outputLedit, 4, 1, 1, 1);

        outputButton = new QPushButton(mediaConfBox);
        outputButton->setObjectName("outputButton");

        gridLayout1->addWidget(outputButton, 4, 2, 1, 1);

        schedBox = new QGroupBox(mediaConfBox);
        schedBox->setObjectName("schedBox");

        gridLayout1->addWidget(schedBox, 5, 0, 1, 3);

        loopBCast = new QCheckBox(mediaConfBox);
        loopBCast->setObjectName("loopBCast");

        gridLayout1->addWidget(loopBCast, 6, 0, 1, 2);


        gridLayout->addWidget(mediaConfBox, 0, 0, 5, 3);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(spacerItem, 0, 3, 1, 1);

        addButton = new QPushButton(Vlm);
        addButton->setObjectName("addButton");

        gridLayout->addWidget(addButton, 1, 3, 1, 1);

        clearButton = new QPushButton(Vlm);
        clearButton->setObjectName("clearButton");

        gridLayout->addWidget(clearButton, 2, 3, 1, 1);

        saveButton = new QPushButton(Vlm);
        saveButton->setObjectName("saveButton");

        gridLayout->addWidget(saveButton, 3, 3, 1, 1);

        spacerItem1 = new QSpacerItem(80, 81, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem1, 4, 3, 1, 1);

        mediaBox = new QGroupBox(Vlm);
        mediaBox->setObjectName("mediaBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(mediaBox->sizePolicy().hasHeightForWidth());
        mediaBox->setSizePolicy(sizePolicy3);
        gridLayout2 = new QGridLayout(mediaBox);
        gridLayout2->setObjectName("gridLayout2");
        vlmListItem = new QListWidget(mediaBox);
        vlmListItem->setObjectName("vlmListItem");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(vlmListItem->sizePolicy().hasHeightForWidth());
        vlmListItem->setSizePolicy(sizePolicy4);
        vlmListItem->setMaximumSize(QSize(200, 16777215));

        gridLayout2->addWidget(vlmListItem, 0, 0, 1, 1);

        line_2 = new QFrame(mediaBox);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout2->addWidget(line_2, 0, 1, 1, 1);

        vlmItemScroll = new QScrollArea(mediaBox);
        vlmItemScroll->setObjectName("vlmItemScroll");
        sizePolicy4.setHeightForWidth(vlmItemScroll->sizePolicy().hasHeightForWidth());
        vlmItemScroll->setSizePolicy(sizePolicy4);
        vlmItemScroll->setMinimumSize(QSize(300, 0));

        gridLayout2->addWidget(vlmItemScroll, 0, 2, 1, 1);


        gridLayout->addWidget(mediaBox, 5, 0, 1, 4);

        buttonBox = new QDialogButtonBox(Vlm);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::NoButton);

        gridLayout->addWidget(buttonBox, 6, 2, 1, 2);

#if QT_CONFIG(shortcut)
        nameLabel->setBuddy(nameLedit);
        inputLabel->setBuddy(inputLedit);
        outputLabel->setBuddy(outputLedit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(mediaType, nameLedit);
        QWidget::setTabOrder(nameLedit, enableCheck);
        QWidget::setTabOrder(enableCheck, inputLedit);
        QWidget::setTabOrder(inputLedit, inputButton);
        QWidget::setTabOrder(inputButton, outputLedit);
        QWidget::setTabOrder(outputLedit, outputButton);
        QWidget::setTabOrder(outputButton, addButton);
        QWidget::setTabOrder(addButton, clearButton);
        QWidget::setTabOrder(clearButton, saveButton);
        QWidget::setTabOrder(saveButton, loopBCast);
        QWidget::setTabOrder(loopBCast, vlmListItem);
        QWidget::setTabOrder(vlmListItem, vlmItemScroll);
        QWidget::setTabOrder(vlmItemScroll, buttonBox);

        retranslateUi(Vlm);

        QMetaObject::connectSlotsByName(Vlm);
    } // setupUi

    void retranslateUi(QWidget *Vlm)
    {
        Vlm->setWindowTitle(Q_("VLM configurator", nullptr));
        mediaConfBox->setTitle(Q_("Media Manager Edition", nullptr));
        nameLabel->setText(Q_("Name:", nullptr));
        enableCheck->setText(Q_("Enable", nullptr));
        inputLabel->setText(Q_("Input:", nullptr));
        inputButton->setText(Q_("Select Input", nullptr));
        outputLabel->setText(Q_("Output:", nullptr));
        outputButton->setText(Q_("Select Output", nullptr));
        schedBox->setTitle(Q_("Time Control", nullptr));
        loopBCast->setText(Q_("Loop", nullptr));
        addButton->setText(Q_("Add", nullptr));
        clearButton->setText(Q_("Clear", nullptr));
        saveButton->setText(Q_("Save", nullptr));
        mediaBox->setTitle(Q_("Media Manager List", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Vlm: public Ui_Vlm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // VLM_H
