#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'open_disk.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef OPEN_DISK_H
#define OPEN_DISK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OpenDisk
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *diskGroupBox;
    QGridLayout *gridLayout;
    QRadioButton *dvdRadioButton;
    QSpacerItem *spacerItem;
    QRadioButton *dvdaRadioButton;
    QRadioButton *audioCDRadioButton;
    QHBoxLayout *horizontalLayout_5;
    QLabel *deviceLabel;
    QComboBox *deviceCombo;
    QToolButton *ejectButton;
    QPushButton *browseDiscButton;
    QSpacerItem *spacerItem1;
    QRadioButton *vcdRadioButton;
    QSpacerItem *spacerItem2;
    QFrame *line;
    QRadioButton *bdRadioButton;
    QCheckBox *dvdsimple;
    QSpacerItem *spacerItem3;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *diskOptionBox;
    QHBoxLayout *horizontalLayout;
    QLabel *titleLabel;
    QSpinBox *titleSpin;
    QSpacerItem *horizontalSpacer;
    QLabel *chapterLabel;
    QSpinBox *chapterSpin;
    QGroupBox *diskOptionBox_2;
    QGridLayout *gridLayout_2;
    QLabel *audioLabel;
    QSpinBox *audioSpin;
    QSpacerItem *horizontalSpacer_2;
    QLabel *subtitlesLabel;
    QSpinBox *subtitlesSpin;
    QSpacerItem *spacerItem4;

    void setupUi(QWidget *OpenDisk)
    {
        if (OpenDisk->objectName().isEmpty())
            OpenDisk->setObjectName("OpenDisk");
        OpenDisk->resize(715, 347);
        OpenDisk->setMinimumSize(QSize(500, 0));
        verticalLayout = new QVBoxLayout(OpenDisk);
        verticalLayout->setObjectName("verticalLayout");
        diskGroupBox = new QGroupBox(OpenDisk);
        diskGroupBox->setObjectName("diskGroupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(diskGroupBox->sizePolicy().hasHeightForWidth());
        diskGroupBox->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(diskGroupBox);
        gridLayout->setObjectName("gridLayout");
        dvdRadioButton = new QRadioButton(diskGroupBox);
        dvdRadioButton->setObjectName("dvdRadioButton");
        dvdRadioButton->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        dvdRadioButton->setChecked(true);

        gridLayout->addWidget(dvdRadioButton, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(16, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 6, 1, 1);

        dvdaRadioButton = new QRadioButton(diskGroupBox);
        dvdaRadioButton->setObjectName("dvdaRadioButton");

        gridLayout->addWidget(dvdaRadioButton, 0, 3, 1, 1);

        audioCDRadioButton = new QRadioButton(diskGroupBox);
        audioCDRadioButton->setObjectName("audioCDRadioButton");

        gridLayout->addWidget(audioCDRadioButton, 0, 7, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        deviceLabel = new QLabel(diskGroupBox);
        deviceLabel->setObjectName("deviceLabel");

        horizontalLayout_5->addWidget(deviceLabel);

        deviceCombo = new QComboBox(diskGroupBox);
        deviceCombo->setObjectName("deviceCombo");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(deviceCombo->sizePolicy().hasHeightForWidth());
        deviceCombo->setSizePolicy(sizePolicy1);
        deviceCombo->setEditable(true);

        horizontalLayout_5->addWidget(deviceCombo);

        ejectButton = new QToolButton(diskGroupBox);
        ejectButton->setObjectName("ejectButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(ejectButton->sizePolicy().hasHeightForWidth());
        ejectButton->setSizePolicy(sizePolicy2);

        horizontalLayout_5->addWidget(ejectButton);

        browseDiscButton = new QPushButton(diskGroupBox);
        browseDiscButton->setObjectName("browseDiscButton");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(browseDiscButton->sizePolicy().hasHeightForWidth());
        browseDiscButton->setSizePolicy(sizePolicy3);

        horizontalLayout_5->addWidget(browseDiscButton);


        gridLayout->addLayout(horizontalLayout_5, 4, 0, 1, 10);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 8, 1, 1);

        vcdRadioButton = new QRadioButton(diskGroupBox);
        vcdRadioButton->setObjectName("vcdRadioButton");

        gridLayout->addWidget(vcdRadioButton, 0, 9, 1, 1);

        spacerItem2 = new QSpacerItem(31, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem2, 0, 10, 1, 1);

        line = new QFrame(diskGroupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 3, 4, 1, 6);

        bdRadioButton = new QRadioButton(diskGroupBox);
        bdRadioButton->setObjectName("bdRadioButton");

        gridLayout->addWidget(bdRadioButton, 0, 5, 1, 1);

        dvdsimple = new QCheckBox(diskGroupBox);
        dvdsimple->setObjectName("dvdsimple");

        gridLayout->addWidget(dvdsimple, 2, 1, 1, 5);

        spacerItem3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem3, 0, 2, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 0, 4, 1, 1);


        verticalLayout->addWidget(diskGroupBox);

        diskOptionBox = new QGroupBox(OpenDisk);
        diskOptionBox->setObjectName("diskOptionBox");
        horizontalLayout = new QHBoxLayout(diskOptionBox);
        horizontalLayout->setObjectName("horizontalLayout");
        titleLabel = new QLabel(diskOptionBox);
        titleLabel->setObjectName("titleLabel");

        horizontalLayout->addWidget(titleLabel);

        titleSpin = new QSpinBox(diskOptionBox);
        titleSpin->setObjectName("titleSpin");
        sizePolicy3.setHeightForWidth(titleSpin->sizePolicy().hasHeightForWidth());
        titleSpin->setSizePolicy(sizePolicy3);
        titleSpin->setAutoFillBackground(false);
        titleSpin->setAlignment(Qt::AlignmentFlag::AlignRight);
        titleSpin->setValue(0);

        horizontalLayout->addWidget(titleSpin);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        chapterLabel = new QLabel(diskOptionBox);
        chapterLabel->setObjectName("chapterLabel");

        horizontalLayout->addWidget(chapterLabel);

        chapterSpin = new QSpinBox(diskOptionBox);
        chapterSpin->setObjectName("chapterSpin");
        sizePolicy3.setHeightForWidth(chapterSpin->sizePolicy().hasHeightForWidth());
        chapterSpin->setSizePolicy(sizePolicy3);
        chapterSpin->setAlignment(Qt::AlignmentFlag::AlignRight);

        horizontalLayout->addWidget(chapterSpin);


        verticalLayout->addWidget(diskOptionBox);

        diskOptionBox_2 = new QGroupBox(OpenDisk);
        diskOptionBox_2->setObjectName("diskOptionBox_2");
        gridLayout_2 = new QGridLayout(diskOptionBox_2);
        gridLayout_2->setObjectName("gridLayout_2");
        audioLabel = new QLabel(diskOptionBox_2);
        audioLabel->setObjectName("audioLabel");

        gridLayout_2->addWidget(audioLabel, 0, 0, 1, 1);

        audioSpin = new QSpinBox(diskOptionBox_2);
        audioSpin->setObjectName("audioSpin");
        sizePolicy3.setHeightForWidth(audioSpin->sizePolicy().hasHeightForWidth());
        audioSpin->setSizePolicy(sizePolicy3);
        audioSpin->setAlignment(Qt::AlignmentFlag::AlignRight);
        audioSpin->setMinimum(-1);
        audioSpin->setMaximum(10);
        audioSpin->setValue(-1);

        gridLayout_2->addWidget(audioSpin, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 0, 2, 1, 1);

        subtitlesLabel = new QLabel(diskOptionBox_2);
        subtitlesLabel->setObjectName("subtitlesLabel");

        gridLayout_2->addWidget(subtitlesLabel, 0, 3, 1, 1);

        subtitlesSpin = new QSpinBox(diskOptionBox_2);
        subtitlesSpin->setObjectName("subtitlesSpin");
        sizePolicy3.setHeightForWidth(subtitlesSpin->sizePolicy().hasHeightForWidth());
        subtitlesSpin->setSizePolicy(sizePolicy3);
        subtitlesSpin->setAutoFillBackground(false);
        subtitlesSpin->setAlignment(Qt::AlignmentFlag::AlignRight);
        subtitlesSpin->setMinimum(-1);
        subtitlesSpin->setMaximum(31);
        subtitlesSpin->setValue(-1);

        gridLayout_2->addWidget(subtitlesSpin, 0, 4, 1, 1);


        verticalLayout->addWidget(diskOptionBox_2);

        spacerItem4 = new QSpacerItem(181, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout->addItem(spacerItem4);

#if QT_CONFIG(shortcut)
        deviceLabel->setBuddy(deviceCombo);
        titleLabel->setBuddy(titleSpin);
        chapterLabel->setBuddy(chapterSpin);
        audioLabel->setBuddy(audioSpin);
        subtitlesLabel->setBuddy(subtitlesSpin);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(dvdRadioButton, dvdaRadioButton);
        QWidget::setTabOrder(dvdaRadioButton, bdRadioButton);
        QWidget::setTabOrder(bdRadioButton, audioCDRadioButton);
        QWidget::setTabOrder(audioCDRadioButton, vcdRadioButton);
        QWidget::setTabOrder(vcdRadioButton, dvdsimple);
        QWidget::setTabOrder(dvdsimple, deviceCombo);
        QWidget::setTabOrder(deviceCombo, ejectButton);
        QWidget::setTabOrder(ejectButton, browseDiscButton);
        QWidget::setTabOrder(browseDiscButton, titleSpin);
        QWidget::setTabOrder(titleSpin, chapterSpin);
        QWidget::setTabOrder(chapterSpin, audioSpin);
        QWidget::setTabOrder(audioSpin, subtitlesSpin);

        retranslateUi(OpenDisk);

        QMetaObject::connectSlotsByName(OpenDisk);
    } // setupUi

    void retranslateUi(QWidget *OpenDisk)
    {
        OpenDisk->setWindowTitle(Q_("Form", nullptr));
        diskGroupBox->setTitle(Q_("Disc Selection", nullptr));
        dvdRadioButton->setText(Q_("DVD", nullptr));
        dvdaRadioButton->setText(Q_("DVD-Audio", nullptr));
        audioCDRadioButton->setText(Q_("Audio CD", nullptr));
        deviceLabel->setText(Q_("Disc device", nullptr));
#if QT_CONFIG(tooltip)
        ejectButton->setToolTip(Q_("Eject the disc", nullptr));
#endif // QT_CONFIG(tooltip)
        browseDiscButton->setText(Q_("Browse...", nullptr));
        vcdRadioButton->setText(Q_("SVCD/VCD", nullptr));
        bdRadioButton->setText(Q_("Blu-ray", nullptr));
#if QT_CONFIG(tooltip)
        dvdsimple->setToolTip(Q_("Disable Disc Menus", nullptr));
#endif // QT_CONFIG(tooltip)
        dvdsimple->setText(Q_("No disc menus", nullptr));
        diskOptionBox->setTitle(Q_("Starting Position", nullptr));
        titleLabel->setText(Q_("Title", nullptr));
        chapterLabel->setText(Q_("Chapter", nullptr));
        diskOptionBox_2->setTitle(Q_("Audio and Subtitles", nullptr));
        audioLabel->setText(Q_("Audio track", nullptr));
        subtitlesLabel->setText(Q_("Subtitle track", nullptr));
        subtitlesSpin->setSuffix(QString());
    } // retranslateUi

};

namespace Ui {
    class OpenDisk: public Ui_OpenDisk {};
} // namespace Ui

QT_END_NAMESPACE

#endif // OPEN_DISK_H
