// /qt/qml/VLC/Widgets/FastBlend.qml
#include <QtQml/qqmlprivate.h>
#include <QtCore/qdatetime.h>
#include <QtCore/qobject.h>
#include <QtCore/qstring.h>
#include <QtCore/qstringlist.h>
#include <QtCore/qtimezone.h>
#include <QtCore/qurl.h>
#include <QtCore/qvariant.h>
#include <QtQml/qjsengine.h>
#include <QtQml/qjsprimitivevalue.h>
#include <QtQml/qjsvalue.h>
#include <QtQml/qqmlcomponent.h>
#include <QtQml/qqmlcontext.h>
#include <QtQml/qqmlengine.h>
#include <QtQml/qqmllist.h>
#include <type_traits>
namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_Widgets_FastBlend_qml {
extern const unsigned char qmlData alignas(16) [];
extern const unsigned char qmlData alignas(16) [] = {

0x71,0x76,0x34,0x63,0x64,0x61,0x74,0x61,
0x49,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xf8,0x8,0x0,0x0,0xc8,0x18,0xd9,0xe1,
0xe2,0x5c,0xd4,0x45,0x9c,0x20,0x75,0x8a,
0x2f,0x3a,0x5,0x40,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x23,0x0,0x0,0x0,
0x1b,0x0,0x0,0x0,0x60,0x3,0x0,0x0,
0x3,0x0,0x0,0x0,0xc8,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xd4,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xd4,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xd4,0x0,0x0,0x0,
0x1e,0x0,0x0,0x0,0xd4,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x4c,0x1,0x0,0x0,
0x2,0x0,0x0,0x0,0x50,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x60,0x1,0x0,0x0,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x80,0x7,0x0,0x0,
0x60,0x1,0x0,0x0,0xb8,0x1,0x0,0x0,
0x80,0x2,0x0,0x0,0x33,0x1,0x0,0x0,
0x40,0x0,0x0,0x0,0x50,0x0,0x0,0x0,
0x93,0x0,0x0,0x0,0x33,0x1,0x0,0x0,
0x40,0x0,0x0,0x0,0x70,0x0,0x0,0x0,
0x93,0x0,0x0,0x0,0x33,0x1,0x0,0x0,
0x40,0x0,0x0,0x0,0x80,0x0,0x0,0x0,
0x33,0x0,0x0,0x0,0xb0,0x0,0x0,0x0,
0x40,0x1,0x0,0x0,0x33,0x0,0x0,0x0,
0xb0,0x0,0x0,0x0,0x50,0x1,0x0,0x0,
0x33,0x0,0x0,0x0,0xb0,0x0,0x0,0x0,
0x60,0x1,0x0,0x0,0x93,0x0,0x0,0x0,
0x33,0x1,0x0,0x0,0x40,0x0,0x0,0x0,
0x60,0x0,0x0,0x0,0x33,0x1,0x0,0x0,
0x40,0x0,0x0,0x0,0x70,0x0,0x0,0x0,
0x33,0x1,0x0,0x0,0x40,0x0,0x0,0x0,
0x80,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x80,0xad,0x3f,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x9,0x0,0x0,0x0,
0xa,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x26,0x0,0x50,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x26,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x0,0x3c,0x1,
0x3c,0x2,0x18,0x6,0x2,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x68,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0xd,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x4,0x0,
0xff,0xff,0xff,0xff,0xd,0x0,0x0,0x0,
0x29,0x0,0x50,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x2a,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x1c,0x0,0x0,0x0,
0x2b,0x0,0x0,0x0,0x3,0x0,0x0,0x0,
0x1e,0x0,0x0,0x0,0x2d,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x51,0x0,0x0,0x0,
0x2e,0x0,0x0,0x0,0x4,0x0,0x0,0x0,
0x2e,0x3,0x18,0x7,0x2e,0x4,0x3c,0x5,
0x3c,0x6,0x6e,0x7,0x50,0x10,0x2e,0x7,
0x18,0x8,0x2e,0x8,0x3c,0x9,0x3c,0xa,
0x6e,0x8,0x50,0x2,0x6,0x2,0x2e,0xb,
0x3c,0xc,0x3c,0xd,0x18,0x7,0x10,0xb,
0x9c,0x7,0x18,0x8,0x2e,0xe,0x3c,0xf,
0x3c,0x10,0x18,0x9,0x10,0x10,0x9c,0x9,
0x80,0x8,0x18,0xa,0x2e,0x11,0x3c,0x12,
0x3c,0x13,0x18,0xb,0x10,0x5,0x9c,0xb,
0x80,0xa,0x18,0xc,0x10,0x20,0x9e,0xc,
0x2,0x16,0x6,0x2,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xa4,0x0,0x0,0x0,0x36,0x0,0x0,0x0,
0x12,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x9,0x0,
0xff,0xff,0xff,0xff,0x8,0x0,0x0,0x0,
0x35,0x0,0x50,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x36,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x37,0x0,0x0,0x0,0x2,0x0,0x0,0x0,
0x11,0x0,0x0,0x0,0x39,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x1b,0x0,0x0,0x0,
0x3b,0x0,0x0,0x0,0x2,0x0,0x0,0x0,
0x27,0x0,0x0,0x0,0x38,0x0,0x0,0x0,
0x3,0x0,0x0,0x0,0x2a,0x0,0x0,0x0,
0x3a,0x0,0x0,0x0,0x4,0x0,0x0,0x0,
0x2d,0x0,0x0,0x0,0x3c,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x30,0x0,0x0,0x0,
0x3e,0x0,0x0,0x0,0x6,0x0,0x0,0x0,
0x33,0x0,0x0,0x0,0x40,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x14,0x1,0x6,0x2e,
0x14,0x18,0x7,0x2e,0x15,0x3c,0x16,0x3c,
0x17,0x6c,0x7,0x4e,0x16,0x2e,0x18,0x3c,
0x19,0x3c,0x1a,0x6c,0x7,0x4e,0xf,0x2e,
0x1b,0x3c,0x1c,0x3c,0x1d,0x6c,0x7,0x4e,
0x8,0x4c,0x9,0x12,0x17,0x2,0x12,0x18,
0x2,0x12,0x19,0x2,0x12,0x1a,0x2,0x16,
0x6,0x2,0x0,0x0,0x0,0x0,0x0,0x0,
0xd0,0x3,0x0,0x0,0xd8,0x3,0x0,0x0,
0xf0,0x3,0x0,0x0,0x10,0x4,0x0,0x0,
0x20,0x4,0x0,0x0,0x30,0x4,0x0,0x0,
0x50,0x4,0x0,0x0,0x68,0x4,0x0,0x0,
0x80,0x4,0x0,0x0,0x98,0x4,0x0,0x0,
0xa8,0x4,0x0,0x0,0xd8,0x4,0x0,0x0,
0xe8,0x4,0x0,0x0,0x0,0x5,0x0,0x0,
0x38,0x5,0x0,0x0,0x50,0x5,0x0,0x0,
0x80,0x5,0x0,0x0,0x88,0x5,0x0,0x0,
0xb0,0x5,0x0,0x0,0xf0,0x5,0x0,0x0,
0x8,0x6,0x0,0x0,0x10,0x6,0x0,0x0,
0x18,0x6,0x0,0x0,0x20,0x6,0x0,0x0,
0x80,0x6,0x0,0x0,0xe0,0x6,0x0,0x0,
0x38,0x7,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x7,0x0,0x0,0x0,0x51,0x0,0x74,0x0,
0x51,0x0,0x75,0x0,0x69,0x0,0x63,0x0,
0x6b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xc,0x0,0x0,0x0,0x53,0x0,0x68,0x0,
0x61,0x0,0x64,0x0,0x65,0x0,0x72,0x0,
0x45,0x0,0x66,0x0,0x66,0x0,0x65,0x0,
0x63,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x72,0x0,0x6f,0x0,
0x6f,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x4d,0x0,0x6f,0x0,
0x64,0x0,0x65,0x0,0x0,0x0,0x0,0x0,
0xa,0x0,0x0,0x0,0x53,0x0,0x6f,0x0,
0x75,0x0,0x72,0x0,0x63,0x0,0x65,0x0,
0x4f,0x0,0x76,0x0,0x65,0x0,0x72,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x41,0x0,0x64,0x0,
0x64,0x0,0x69,0x0,0x74,0x0,0x69,0x0,
0x76,0x0,0x65,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x4d,0x0,0x75,0x0,
0x6c,0x0,0x74,0x0,0x69,0x0,0x70,0x0,
0x6c,0x0,0x79,0x0,0x0,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x53,0x0,0x63,0x0,
0x72,0x0,0x65,0x0,0x65,0x0,0x6e,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x6d,0x0,0x6f,0x0,
0x64,0x0,0x65,0x0,0x0,0x0,0x0,0x0,
0x13,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x6d,0x0,0x6f,0x0,0x64,0x0,
0x65,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x63,0x0,0x6f,0x0,
0x6c,0x0,0x6f,0x0,0x72,0x0,0x0,0x0,
0x9,0x0,0x0,0x0,0x67,0x0,0x72,0x0,
0x61,0x0,0x79,0x0,0x73,0x0,0x63,0x0,
0x61,0x0,0x6c,0x0,0x65,0x0,0x0,0x0,
0x18,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x67,0x0,0x72,0x0,0x61,0x0,
0x79,0x0,0x73,0x0,0x63,0x0,0x61,0x0,
0x6c,0x0,0x65,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x62,0x0,0x6c,0x0,
0x65,0x0,0x6e,0x0,0x64,0x0,0x69,0x0,
0x6e,0x0,0x67,0x0,0x0,0x0,0x0,0x0,
0x15,0x0,0x0,0x0,0x73,0x0,0x75,0x0,
0x70,0x0,0x70,0x0,0x6f,0x0,0x72,0x0,
0x74,0x0,0x73,0x0,0x41,0x0,0x74,0x0,
0x6c,0x0,0x61,0x0,0x73,0x0,0x54,0x0,
0x65,0x0,0x78,0x0,0x74,0x0,0x75,0x0,
0x72,0x0,0x65,0x0,0x73,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x7a,0x0,0x0,0x0,
0xe,0x0,0x0,0x0,0x66,0x0,0x72,0x0,
0x61,0x0,0x67,0x0,0x6d,0x0,0x65,0x0,
0x6e,0x0,0x74,0x0,0x53,0x0,0x68,0x0,
0x61,0x0,0x64,0x0,0x65,0x0,0x72,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1d,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x66,0x0,0x72,0x0,0x61,0x0,
0x67,0x0,0x6d,0x0,0x65,0x0,0x6e,0x0,
0x74,0x0,0x53,0x0,0x68,0x0,0x61,0x0,
0x64,0x0,0x65,0x0,0x72,0x0,0x0,0x0,
0x9,0x0,0x0,0x0,0x46,0x0,0x61,0x0,
0x73,0x0,0x74,0x0,0x42,0x0,0x6c,0x0,
0x65,0x0,0x6e,0x0,0x64,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x72,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x67,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x62,0x0,0x0,0x0,
0x2a,0x0,0x0,0x0,0x71,0x0,0x72,0x0,
0x63,0x0,0x3a,0x0,0x2f,0x0,0x2f,0x0,
0x2f,0x0,0x73,0x0,0x68,0x0,0x61,0x0,
0x64,0x0,0x65,0x0,0x72,0x0,0x73,0x0,
0x2f,0x0,0x46,0x0,0x61,0x0,0x73,0x0,
0x74,0x0,0x42,0x0,0x6c,0x0,0x65,0x0,
0x6e,0x0,0x64,0x0,0x5f,0x0,0x61,0x0,
0x64,0x0,0x64,0x0,0x69,0x0,0x74,0x0,
0x69,0x0,0x76,0x0,0x65,0x0,0x2e,0x0,
0x66,0x0,0x72,0x0,0x61,0x0,0x67,0x0,
0x2e,0x0,0x71,0x0,0x73,0x0,0x62,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2a,0x0,0x0,0x0,0x71,0x0,0x72,0x0,
0x63,0x0,0x3a,0x0,0x2f,0x0,0x2f,0x0,
0x2f,0x0,0x73,0x0,0x68,0x0,0x61,0x0,
0x64,0x0,0x65,0x0,0x72,0x0,0x73,0x0,
0x2f,0x0,0x46,0x0,0x61,0x0,0x73,0x0,
0x74,0x0,0x42,0x0,0x6c,0x0,0x65,0x0,
0x6e,0x0,0x64,0x0,0x5f,0x0,0x6d,0x0,
0x75,0x0,0x6c,0x0,0x74,0x0,0x69,0x0,
0x70,0x0,0x6c,0x0,0x79,0x0,0x2e,0x0,
0x66,0x0,0x72,0x0,0x61,0x0,0x67,0x0,
0x2e,0x0,0x71,0x0,0x73,0x0,0x62,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x28,0x0,0x0,0x0,0x71,0x0,0x72,0x0,
0x63,0x0,0x3a,0x0,0x2f,0x0,0x2f,0x0,
0x2f,0x0,0x73,0x0,0x68,0x0,0x61,0x0,
0x64,0x0,0x65,0x0,0x72,0x0,0x73,0x0,
0x2f,0x0,0x46,0x0,0x61,0x0,0x73,0x0,
0x74,0x0,0x42,0x0,0x6c,0x0,0x65,0x0,
0x6e,0x0,0x64,0x0,0x5f,0x0,0x73,0x0,
0x63,0x0,0x72,0x0,0x65,0x0,0x65,0x0,
0x6e,0x0,0x2e,0x0,0x66,0x0,0x72,0x0,
0x61,0x0,0x67,0x0,0x2e,0x0,0x71,0x0,
0x73,0x0,0x62,0x0,0x0,0x0,0x0,0x0,
0x21,0x0,0x0,0x0,0x71,0x0,0x72,0x0,
0x63,0x0,0x3a,0x0,0x2f,0x0,0x2f,0x0,
0x2f,0x0,0x73,0x0,0x68,0x0,0x61,0x0,
0x64,0x0,0x65,0x0,0x72,0x0,0x73,0x0,
0x2f,0x0,0x46,0x0,0x61,0x0,0x73,0x0,
0x74,0x0,0x42,0x0,0x6c,0x0,0x65,0x0,
0x6e,0x0,0x64,0x0,0x2e,0x0,0x66,0x0,
0x72,0x0,0x61,0x0,0x67,0x0,0x2e,0x0,
0x71,0x0,0x73,0x0,0x62,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x10,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x24,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x12,0x0,0x10,0x0,
0xff,0xff,0x0,0x0,0x28,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x3,0x0,0x0,0x0,
0x0,0x0,0xff,0xff,0xff,0xff,0xff,0xff,
0x0,0x0,0x3,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x78,0x0,0x0,0x0,
0x0,0x0,0x1,0x0,0x78,0x0,0x0,0x0,
0x7c,0x0,0x0,0x0,0x0,0x0,0x6,0x0,
0x7c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xc,0x1,0x0,0x0,0x18,0x0,0x10,0x0,
0x19,0x0,0x50,0x0,0xc,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xc,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x9,0x0,0x0,0x0,
0x2,0x0,0x0,0x20,0x26,0x0,0x50,0x0,
0xb,0x0,0x0,0x0,0xb,0x0,0x0,0x0,
0x28,0x0,0x50,0x0,0xc,0x0,0x0,0x0,
0x4,0x0,0x0,0xa0,0x29,0x0,0x50,0x0,
0xc,0x1,0x0,0x0,0x11,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x2,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x35,0x0,0x50,0x0,
0x35,0x0,0x50,0x1,0x10,0x0,0x0,0x0,
0x0,0x0,0x2,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x33,0x0,0x50,0x0,
0x33,0x0,0x80,0x0,0xf,0x0,0x0,0x0,
0x0,0x0,0x1,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x31,0x0,0x50,0x0,
0x31,0x0,0xc0,0x1,0xe,0x0,0x0,0x0,
0x0,0x0,0x1,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x30,0x0,0x50,0x0,
0x30,0x0,0xf0,0x0,0xc,0x0,0x0,0x0,
0x8,0x0,0x7,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x29,0x0,0xc0,0x1,
0x29,0x0,0x70,0x2,0x9,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x26,0x0,0x20,0x1,
0x26,0x0,0x80,0x1,0x4,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x1b,0x0,0x50,0x0,
0x5,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1d,0x0,0x90,0x0,0x6,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x1f,0x0,0x90,0x0,
0x7,0x0,0x0,0x0,0x2,0x0,0x0,0x0,
0x21,0x0,0x90,0x0,0x8,0x0,0x0,0x0,
0x3,0x0,0x0,0x0,0x23,0x0,0x90,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
};
extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[] = {
{ 0, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<int>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for mode at line 38, column 5
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
int v2_4;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
{
}
{
}
// generate_GetLookup
s.v2_4 = 0;
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<int *>(argv[0]) = s.v2_4;
}
return;
}
 },{ 1, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<double>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for grayscale at line 41, column 5
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_30, markStack);
    aotContext->mark(v2_32, markStack);
    aotContext->mark(v2_44, markStack);
    aotContext->mark(v2_46, markStack);
    aotContext->mark(v2_60, markStack);
    aotContext->mark(v2_62, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_30 = nullptr;
QObject *v2_44 = nullptr;
QObject *v2_60 = nullptr;
QVariant v2_32;
QVariant v2_46;
QVariant v2_62;
bool v2_10;
bool v2_24;
double v2_28;
double v2_34;
double v2_38;
double v2_40;
double v2_48;
double v2_52;
double v2_54;
double v2_56;
double v2_64;
double v2_68;
double v2_70;
double v2_72;
double v2_76;
double v2_78;
int v2_0;
int v2_14;
int v2_22;
int v2_8;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(3, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadScopeObjectPropertyLookup(3);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
{
}
{
}
// generate_GetLookup
s.v2_8 = 2;
{
}
// generate_CmpStrictNotEqual
s.v2_10 = s.v2_0 != s.v2_8;
{
}
// generate_JumpFalse
if (!s.v2_10) {
    goto label_0;
}
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(16);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(7, &s.v2_14)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(16);
#endif
aotContext->initLoadScopeObjectPropertyLookup(7);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
{
}
{
}
// generate_GetLookup
s.v2_22 = 3;
{
}
// generate_CmpStrictNotEqual
s.v2_24 = s.v2_14 != s.v2_22;
{
}
// generate_JumpFalse
if (!s.v2_24) {
    goto label_0;
}
{
}
// generate_LoadZero
s.v2_28 = double(0);
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_28;
}
return;
label_0:;
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(32);
#endif
while (!aotContext->loadContextIdLookup(11, &s.v2_30)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(32);
#endif
aotContext->initLoadContextIdLookup(11);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(34);
#endif
s.v2_32 = QVariant(aotContext->lookupResultMetaType(12));
while (!aotContext->getObjectLookup(12, s.v2_30, s.v2_32.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(34);
#endif
aotContext->initGetObjectLookup(12, s.v2_30);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
s.v2_32 = QVariant(aotContext->lookupResultMetaType(12));
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(36);
#endif
while (!aotContext->getValueLookup(13, s.v2_32.data(), &s.v2_34)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(36);
#endif
aotContext->initGetValueLookup(13, []() { static const auto t = QMetaType::fromName("QQuickColorValueType"); return t; }().metaObject());
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
// generate_LoadInt
s.v2_38 = double(11);
{
}
// generate_Mul
s.v2_40 = (s.v2_34 * s.v2_38);
{
}
// generate_StoreReg
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(46);
#endif
while (!aotContext->loadContextIdLookup(14, &s.v2_44)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(46);
#endif
aotContext->initLoadContextIdLookup(14);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(48);
#endif
s.v2_46 = QVariant(aotContext->lookupResultMetaType(15));
while (!aotContext->getObjectLookup(15, s.v2_44, s.v2_46.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(48);
#endif
aotContext->initGetObjectLookup(15, s.v2_44);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
s.v2_46 = QVariant(aotContext->lookupResultMetaType(15));
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(50);
#endif
while (!aotContext->getValueLookup(16, s.v2_46.data(), &s.v2_48)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(50);
#endif
aotContext->initGetValueLookup(16, []() { static const auto t = QMetaType::fromName("QQuickColorValueType"); return t; }().metaObject());
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
// generate_LoadInt
s.v2_52 = double(16);
{
}
// generate_Mul
s.v2_54 = (s.v2_48 * s.v2_52);
{
}
// generate_Add
s.v2_56 = (s.v2_40 + s.v2_54);
{
}
// generate_StoreReg
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(62);
#endif
while (!aotContext->loadContextIdLookup(17, &s.v2_60)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(62);
#endif
aotContext->initLoadContextIdLookup(17);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(64);
#endif
s.v2_62 = QVariant(aotContext->lookupResultMetaType(18));
while (!aotContext->getObjectLookup(18, s.v2_60, s.v2_62.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(64);
#endif
aotContext->initGetObjectLookup(18, s.v2_60);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
s.v2_62 = QVariant(aotContext->lookupResultMetaType(18));
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(66);
#endif
while (!aotContext->getValueLookup(19, s.v2_62.data(), &s.v2_64)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(66);
#endif
aotContext->initGetValueLookup(19, []() { static const auto t = QMetaType::fromName("QQuickColorValueType"); return t; }().metaObject());
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
// generate_LoadInt
s.v2_68 = double(5);
{
}
// generate_Mul
s.v2_70 = (s.v2_64 * s.v2_68);
{
}
// generate_Add
s.v2_72 = (s.v2_56 + s.v2_70);
{
}
// generate_StoreReg
{
}
// generate_LoadInt
s.v2_76 = double(32);
{
}
// generate_Div
QT_WARNING_PUSH
QT_WARNING_DISABLE_MSVC(4723) // potential divide by 0
s.v2_78 = (s.v2_72 / s.v2_76);
QT_WARNING_POP
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_78;
}
return;
}
 },{ 2, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<QUrl>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for fragmentShader at line 53, column 5
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QUrl v2_39;
QUrl v2_42;
QUrl v2_45;
QUrl v2_48;
bool v2_13;
bool v2_23;
bool v2_33;
int v2_11;
int v2_21;
int v2_31;
int v2_3;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(5);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(20, &s.v2_3)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(5);
#endif
aotContext->initLoadScopeObjectPropertyLookup(20);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<QUrl *>(argv[0]) = QUrl();
}
return;
}
}
{
}
// generate_StoreReg
{
}
{
}
{
}
// generate_GetLookup
s.v2_11 = 1;
{
}
// generate_CmpStrictEqual
s.v2_13 = s.v2_3 == s.v2_11;
{
}
// generate_JumpTrue
if (s.v2_13) {
    goto label_0;
}
{
}
{
}
{
}
// generate_GetLookup
s.v2_21 = 2;
{
}
// generate_CmpStrictEqual
s.v2_23 = s.v2_3 == s.v2_21;
{
}
// generate_JumpTrue
if (s.v2_23) {
    goto label_1;
}
{
}
{
}
{
}
// generate_GetLookup
s.v2_31 = 3;
{
}
// generate_CmpStrictEqual
s.v2_33 = s.v2_3 == s.v2_31;
{
}
// generate_JumpTrue
if (s.v2_33) {
    goto label_2;
}
{
}
// generate_Jump
{
    goto label_3;
}
label_0:;
// generate_LoadRuntimeString
s.v2_39 = QUrl(QStringLiteral("qrc:///shaders/FastBlend_additive.frag.qsb"));
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<QUrl *>(argv[0]) = std::move(s.v2_39);
}
return;
label_1:;
// generate_LoadRuntimeString
s.v2_42 = QUrl(QStringLiteral("qrc:///shaders/FastBlend_multiply.frag.qsb"));
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<QUrl *>(argv[0]) = std::move(s.v2_42);
}
return;
label_2:;
// generate_LoadRuntimeString
s.v2_45 = QUrl(QStringLiteral("qrc:///shaders/FastBlend_screen.frag.qsb"));
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<QUrl *>(argv[0]) = std::move(s.v2_45);
}
return;
label_3:;
// generate_LoadRuntimeString
s.v2_48 = QUrl(QStringLiteral("qrc:///shaders/FastBlend.frag.qsb"));
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<QUrl *>(argv[0]) = std::move(s.v2_48);
}
return;
}
 },{ 0, 0, nullptr, nullptr }};
}
}
