#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'sprefs_interface.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef SPREFS_INTERFACE_H
#define SPREFS_INTERFACE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SPrefsInterface
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *langBox;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *langCombo;
    QGroupBox *LooknfeelBox;
    QVBoxLayout *LooknfeelLayout;
    QWidget *LooknfeelSelection;
    QGridLayout *LooknfeelSelection_l;
    QVBoxLayout *verticalLayout_2;
    QPushButton *skinImage;
    QRadioButton *skins;
    QVBoxLayout *verticalLayout_8;
    QPushButton *modernImage;
    QRadioButton *modernButton;
    QVBoxLayout *verticalLayout_7;
    QPushButton *classicImage;
    QRadioButton *classicButton;
    QFrame *line;
    QToolButton *toolbarEditor;
    QStackedWidget *styleStackedWidget;
    QWidget *nativePage;
    QHBoxLayout *horizontalLayout_4;
    QWidget *native_zone;
    QGridLayout *gridLayout;
    QCheckBox *resizingBox;
    QComboBox *notificationCombo;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *titleBarCheckBox;
    QCheckBox *embedVideo;
    QComboBox *autoRaiseComboBox;
    QLabel *notificationComboLabel;
    QCheckBox *systrayBox;
    QLabel *stylesLabel;
    QSpinBox *intfScaleFactorSpinBox;
    QLabel *colorSchemeLabel;
    QCheckBox *minimalviewBox;
    QLabel *autoRaiseLabel;
    QComboBox *stylesCombo;
    QSlider *intfScaleFactorSlider;
    QCheckBox *pauseMinimizedBox;
    QLabel *scaleFactorLabel;
    QCheckBox *menuBarCheck;
    QCheckBox *fsController;
    QComboBox *colorSchemeComboBox;
    QCheckBox *pinVideoControlsCheckbox;
    QWidget *skinsPage;
    QHBoxLayout *horizontalLayout_5;
    QWidget *skins_zone;
    QGridLayout *gridLayout_7;
    QLabel *skinsLabel;
    QPushButton *skinBrowse;
    QLineEdit *fileSkin;
    QLabel *skinFileLabel;
    QGroupBox *PlaylistAndInstancesBox;
    QGridLayout *gridLayout_2;
    QComboBox *continuePlaybackComboBox;
    QCheckBox *treePlaylist;
    QCheckBox *OneInterfaceMode;
    QCheckBox *oneInstanceFromFile;
    QCheckBox *EnqueueOneInterfaceMode;
    QCheckBox *playPauseBox;
    QLabel *continuePlaybackLabel;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_3;
    QSpinBox *updatesDays;
    QCheckBox *saveRecentlyPlayed;
    QPushButton *clearRecent;
    QCheckBox *MetadataNetworkAccessMode;
    QSpacerItem *clearRecentSpacer;
    QCheckBox *updatesBox;
    QGroupBox *osGroupBox;
    QGridLayout *gridLayout_4;
    QLabel *assocLabel;
    QSpacerItem *horizontalSpacer_6;
    QPushButton *assoButton;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SPrefsInterface)
    {
        if (SPrefsInterface->objectName().isEmpty())
            SPrefsInterface->setObjectName("SPrefsInterface");
        SPrefsInterface->resize(1446, 1378);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SPrefsInterface->sizePolicy().hasHeightForWidth());
        SPrefsInterface->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(SPrefsInterface);
        verticalLayout->setObjectName("verticalLayout");
        langBox = new QGroupBox(SPrefsInterface);
        langBox->setObjectName("langBox");
        horizontalLayout = new QHBoxLayout(langBox);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(langBox);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        langCombo = new QComboBox(langBox);
        langCombo->setObjectName("langCombo");

        horizontalLayout->addWidget(langCombo);


        verticalLayout->addWidget(langBox);

        LooknfeelBox = new QGroupBox(SPrefsInterface);
        LooknfeelBox->setObjectName("LooknfeelBox");
        LooknfeelLayout = new QVBoxLayout(LooknfeelBox);
        LooknfeelLayout->setObjectName("LooknfeelLayout");
        LooknfeelLayout->setContentsMargins(-1, -1, -1, 0);
        LooknfeelSelection = new QWidget(LooknfeelBox);
        LooknfeelSelection->setObjectName("LooknfeelSelection");
        LooknfeelSelection_l = new QGridLayout(LooknfeelSelection);
        LooknfeelSelection_l->setObjectName("LooknfeelSelection_l");
        LooknfeelSelection_l->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        skinImage = new QPushButton(LooknfeelSelection);
        skinImage->setObjectName("skinImage");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/prefsmenu/sample_skins.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        skinImage->setIcon(icon);
        skinImage->setIconSize(QSize(244, 145));
        skinImage->setFlat(true);

        verticalLayout_2->addWidget(skinImage);

        skins = new QRadioButton(LooknfeelSelection);
        skins->setObjectName("skins");

        verticalLayout_2->addWidget(skins, 0, Qt::AlignHCenter);


        LooknfeelSelection_l->addLayout(verticalLayout_2, 0, 3, 1, 1);

        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName("verticalLayout_8");
        modernImage = new QPushButton(LooknfeelSelection);
        modernImage->setObjectName("modernImage");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/prefsmenu/sample_minimal.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        modernImage->setIcon(icon1);
        modernImage->setIconSize(QSize(244, 145));
        modernImage->setFlat(true);

        verticalLayout_8->addWidget(modernImage);

        modernButton = new QRadioButton(LooknfeelSelection);
        modernButton->setObjectName("modernButton");
        modernButton->setChecked(true);

        verticalLayout_8->addWidget(modernButton, 0, Qt::AlignHCenter);


        LooknfeelSelection_l->addLayout(verticalLayout_8, 0, 1, 1, 1);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        classicImage = new QPushButton(LooknfeelSelection);
        classicImage->setObjectName("classicImage");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/prefsmenu/sample_complete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        classicImage->setIcon(icon2);
        classicImage->setIconSize(QSize(244, 145));
        classicImage->setFlat(true);

        verticalLayout_7->addWidget(classicImage);

        classicButton = new QRadioButton(LooknfeelSelection);
        classicButton->setObjectName("classicButton");

        verticalLayout_7->addWidget(classicButton, 0, Qt::AlignHCenter);


        LooknfeelSelection_l->addLayout(verticalLayout_7, 0, 2, 1, 1);

        line = new QFrame(LooknfeelSelection);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        LooknfeelSelection_l->addWidget(line, 1, 1, 1, 2);


        LooknfeelLayout->addWidget(LooknfeelSelection);

        toolbarEditor = new QToolButton(LooknfeelBox);
        toolbarEditor->setObjectName("toolbarEditor");

        LooknfeelLayout->addWidget(toolbarEditor);

        styleStackedWidget = new QStackedWidget(LooknfeelBox);
        styleStackedWidget->setObjectName("styleStackedWidget");
        nativePage = new QWidget();
        nativePage->setObjectName("nativePage");
        horizontalLayout_4 = new QHBoxLayout(nativePage);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, -1, 0, -1);
        native_zone = new QWidget(nativePage);
        native_zone->setObjectName("native_zone");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(10);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(native_zone->sizePolicy().hasHeightForWidth());
        native_zone->setSizePolicy(sizePolicy1);
        gridLayout = new QGridLayout(native_zone);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(10, 0, -1, 0);
        resizingBox = new QCheckBox(native_zone);
        resizingBox->setObjectName("resizingBox");

        gridLayout->addWidget(resizingBox, 1, 2, 1, 2);

        notificationCombo = new QComboBox(native_zone);
        notificationCombo->setObjectName("notificationCombo");

        gridLayout->addWidget(notificationCombo, 6, 2, 1, 2);

        horizontalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_7, 1, 4, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 1, 1, 1, 1);

        titleBarCheckBox = new QCheckBox(native_zone);
        titleBarCheckBox->setObjectName("titleBarCheckBox");

        gridLayout->addWidget(titleBarCheckBox, 5, 2, 1, 1);

        embedVideo = new QCheckBox(native_zone);
        embedVideo->setObjectName("embedVideo");

        gridLayout->addWidget(embedVideo, 1, 0, 1, 1);

        autoRaiseComboBox = new QComboBox(native_zone);
        autoRaiseComboBox->setObjectName("autoRaiseComboBox");

        gridLayout->addWidget(autoRaiseComboBox, 12, 2, 1, 2);

        notificationComboLabel = new QLabel(native_zone);
        notificationComboLabel->setObjectName("notificationComboLabel");

        gridLayout->addWidget(notificationComboLabel, 6, 0, 1, 1);

        systrayBox = new QCheckBox(native_zone);
        systrayBox->setObjectName("systrayBox");

        gridLayout->addWidget(systrayBox, 4, 0, 1, 2);

        stylesLabel = new QLabel(native_zone);
        stylesLabel->setObjectName("stylesLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(stylesLabel->sizePolicy().hasHeightForWidth());
        stylesLabel->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(stylesLabel, 10, 0, 1, 1);

        intfScaleFactorSpinBox = new QSpinBox(native_zone);
        intfScaleFactorSpinBox->setObjectName("intfScaleFactorSpinBox");
        intfScaleFactorSpinBox->setSingleStep(5);

        gridLayout->addWidget(intfScaleFactorSpinBox, 14, 3, 1, 1);

        colorSchemeLabel = new QLabel(native_zone);
        colorSchemeLabel->setObjectName("colorSchemeLabel");

        gridLayout->addWidget(colorSchemeLabel, 13, 0, 1, 1);

        minimalviewBox = new QCheckBox(native_zone);
        minimalviewBox->setObjectName("minimalviewBox");

        gridLayout->addWidget(minimalviewBox, 3, 0, 1, 1);

        autoRaiseLabel = new QLabel(native_zone);
        autoRaiseLabel->setObjectName("autoRaiseLabel");

        gridLayout->addWidget(autoRaiseLabel, 12, 0, 1, 1);

        stylesCombo = new QComboBox(native_zone);
        stylesCombo->setObjectName("stylesCombo");

        gridLayout->addWidget(stylesCombo, 10, 2, 1, 2);

        intfScaleFactorSlider = new QSlider(native_zone);
        intfScaleFactorSlider->setObjectName("intfScaleFactorSlider");
        intfScaleFactorSlider->setTracking(false);
        intfScaleFactorSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(intfScaleFactorSlider, 14, 2, 1, 1);

        pauseMinimizedBox = new QCheckBox(native_zone);
        pauseMinimizedBox->setObjectName("pauseMinimizedBox");

        gridLayout->addWidget(pauseMinimizedBox, 3, 2, 1, 2);

        scaleFactorLabel = new QLabel(native_zone);
        scaleFactorLabel->setObjectName("scaleFactorLabel");

        gridLayout->addWidget(scaleFactorLabel, 14, 0, 1, 1);

        menuBarCheck = new QCheckBox(native_zone);
        menuBarCheck->setObjectName("menuBarCheck");

        gridLayout->addWidget(menuBarCheck, 4, 2, 1, 1);

        fsController = new QCheckBox(native_zone);
        fsController->setObjectName("fsController");

        gridLayout->addWidget(fsController, 0, 0, 1, 5);

        colorSchemeComboBox = new QComboBox(native_zone);
        colorSchemeComboBox->setObjectName("colorSchemeComboBox");

        gridLayout->addWidget(colorSchemeComboBox, 13, 2, 1, 2);

        pinVideoControlsCheckbox = new QCheckBox(native_zone);
        pinVideoControlsCheckbox->setObjectName("pinVideoControlsCheckbox");

        gridLayout->addWidget(pinVideoControlsCheckbox, 5, 0, 1, 2);


        horizontalLayout_4->addWidget(native_zone);

        styleStackedWidget->addWidget(nativePage);
        skinsPage = new QWidget();
        skinsPage->setObjectName("skinsPage");
        horizontalLayout_5 = new QHBoxLayout(skinsPage);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(0, -1, 0, -1);
        skins_zone = new QWidget(skinsPage);
        skins_zone->setObjectName("skins_zone");
        skins_zone->setEnabled(true);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(skins_zone->sizePolicy().hasHeightForWidth());
        skins_zone->setSizePolicy(sizePolicy3);
        gridLayout_7 = new QGridLayout(skins_zone);
        gridLayout_7->setObjectName("gridLayout_7");
        gridLayout_7->setContentsMargins(0, 0, 0, 0);
        skinsLabel = new QLabel(skins_zone);
        skinsLabel->setObjectName("skinsLabel");
        sizePolicy3.setHeightForWidth(skinsLabel->sizePolicy().hasHeightForWidth());
        skinsLabel->setSizePolicy(sizePolicy3);
        skinsLabel->setFrameShape(QFrame::NoFrame);
        skinsLabel->setText(QString::fromUtf8("-This is the skinsLabel place-"));
        skinsLabel->setTextFormat(Qt::RichText);
        skinsLabel->setScaledContents(false);
        skinsLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        skinsLabel->setWordWrap(true);
        skinsLabel->setMargin(5);
        skinsLabel->setOpenExternalLinks(true);

        gridLayout_7->addWidget(skinsLabel, 2, 1, 1, 3);

        skinBrowse = new QPushButton(skins_zone);
        skinBrowse->setObjectName("skinBrowse");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(skinBrowse->sizePolicy().hasHeightForWidth());
        skinBrowse->setSizePolicy(sizePolicy4);

        gridLayout_7->addWidget(skinBrowse, 1, 3, 1, 1);

        fileSkin = new QLineEdit(skins_zone);
        fileSkin->setObjectName("fileSkin");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(fileSkin->sizePolicy().hasHeightForWidth());
        fileSkin->setSizePolicy(sizePolicy5);

        gridLayout_7->addWidget(fileSkin, 1, 2, 1, 1);

        skinFileLabel = new QLabel(skins_zone);
        skinFileLabel->setObjectName("skinFileLabel");
        skinFileLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_7->addWidget(skinFileLabel, 1, 1, 1, 1);


        horizontalLayout_5->addWidget(skins_zone);

        styleStackedWidget->addWidget(skinsPage);

        LooknfeelLayout->addWidget(styleStackedWidget);


        verticalLayout->addWidget(LooknfeelBox);

        PlaylistAndInstancesBox = new QGroupBox(SPrefsInterface);
        PlaylistAndInstancesBox->setObjectName("PlaylistAndInstancesBox");
        gridLayout_2 = new QGridLayout(PlaylistAndInstancesBox);
        gridLayout_2->setObjectName("gridLayout_2");
        continuePlaybackComboBox = new QComboBox(PlaylistAndInstancesBox);
        continuePlaybackComboBox->setObjectName("continuePlaybackComboBox");
        continuePlaybackComboBox->setEnabled(true);

        gridLayout_2->addWidget(continuePlaybackComboBox, 4, 1, 1, 1);

        treePlaylist = new QCheckBox(PlaylistAndInstancesBox);
        treePlaylist->setObjectName("treePlaylist");

        gridLayout_2->addWidget(treePlaylist, 2, 0, 1, 1);

        OneInterfaceMode = new QCheckBox(PlaylistAndInstancesBox);
        OneInterfaceMode->setObjectName("OneInterfaceMode");

        gridLayout_2->addWidget(OneInterfaceMode, 0, 0, 1, 1);

        oneInstanceFromFile = new QCheckBox(PlaylistAndInstancesBox);
        oneInstanceFromFile->setObjectName("oneInstanceFromFile");

        gridLayout_2->addWidget(oneInstanceFromFile, 1, 0, 1, 2);

        EnqueueOneInterfaceMode = new QCheckBox(PlaylistAndInstancesBox);
        EnqueueOneInterfaceMode->setObjectName("EnqueueOneInterfaceMode");

        gridLayout_2->addWidget(EnqueueOneInterfaceMode, 0, 1, 1, 1);

        playPauseBox = new QCheckBox(PlaylistAndInstancesBox);
        playPauseBox->setObjectName("playPauseBox");

        gridLayout_2->addWidget(playPauseBox, 2, 1, 1, 1);

        continuePlaybackLabel = new QLabel(PlaylistAndInstancesBox);
        continuePlaybackLabel->setObjectName("continuePlaybackLabel");

        gridLayout_2->addWidget(continuePlaybackLabel, 4, 0, 1, 1);


        verticalLayout->addWidget(PlaylistAndInstancesBox);

        groupBox_2 = new QGroupBox(SPrefsInterface);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_3 = new QGridLayout(groupBox_2);
        gridLayout_3->setObjectName("gridLayout_3");
        updatesDays = new QSpinBox(groupBox_2);
        updatesDays->setObjectName("updatesDays");
        updatesDays->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(updatesDays, 0, 2, 1, 1);

        saveRecentlyPlayed = new QCheckBox(groupBox_2);
        saveRecentlyPlayed->setObjectName("saveRecentlyPlayed");

        gridLayout_3->addWidget(saveRecentlyPlayed, 1, 0, 1, 1);

        clearRecent = new QPushButton(groupBox_2);
        clearRecent->setObjectName("clearRecent");

        gridLayout_3->addWidget(clearRecent, 1, 2, 1, 1);

        MetadataNetworkAccessMode = new QCheckBox(groupBox_2);
        MetadataNetworkAccessMode->setObjectName("MetadataNetworkAccessMode");

        gridLayout_3->addWidget(MetadataNetworkAccessMode, 2, 0, 1, 1);

        clearRecentSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(clearRecentSpacer, 1, 1, 1, 1);

        updatesBox = new QCheckBox(groupBox_2);
        updatesBox->setObjectName("updatesBox");

        gridLayout_3->addWidget(updatesBox, 0, 0, 1, 1);


        verticalLayout->addWidget(groupBox_2);

        osGroupBox = new QGroupBox(SPrefsInterface);
        osGroupBox->setObjectName("osGroupBox");
        gridLayout_4 = new QGridLayout(osGroupBox);
        gridLayout_4->setObjectName("gridLayout_4");
        assocLabel = new QLabel(osGroupBox);
        assocLabel->setObjectName("assocLabel");
        sizePolicy2.setHeightForWidth(assocLabel->sizePolicy().hasHeightForWidth());
        assocLabel->setSizePolicy(sizePolicy2);

        gridLayout_4->addWidget(assocLabel, 2, 0, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_6, 2, 1, 1, 1);

        assoButton = new QPushButton(osGroupBox);
        assoButton->setObjectName("assoButton");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(assoButton->sizePolicy().hasHeightForWidth());
        assoButton->setSizePolicy(sizePolicy6);
        assoButton->setMinimumSize(QSize(220, 0));

        gridLayout_4->addWidget(assoButton, 2, 2, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(135, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_8, 2, 3, 1, 1);


        verticalLayout->addWidget(osGroupBox);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label->setBuddy(langCombo);
        notificationComboLabel->setBuddy(notificationCombo);
        stylesLabel->setBuddy(stylesCombo);
        colorSchemeLabel->setBuddy(colorSchemeComboBox);
        autoRaiseLabel->setBuddy(autoRaiseComboBox);
        scaleFactorLabel->setBuddy(intfScaleFactorSlider);
        skinFileLabel->setBuddy(fileSkin);
        continuePlaybackLabel->setBuddy(continuePlaybackComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(langCombo, fsController);
        QWidget::setTabOrder(fsController, embedVideo);
        QWidget::setTabOrder(embedVideo, resizingBox);
        QWidget::setTabOrder(resizingBox, minimalviewBox);
        QWidget::setTabOrder(minimalviewBox, pauseMinimizedBox);
        QWidget::setTabOrder(pauseMinimizedBox, systrayBox);
        QWidget::setTabOrder(systrayBox, menuBarCheck);
        QWidget::setTabOrder(menuBarCheck, pinVideoControlsCheckbox);
        QWidget::setTabOrder(pinVideoControlsCheckbox, titleBarCheckBox);
        QWidget::setTabOrder(titleBarCheckBox, notificationCombo);
        QWidget::setTabOrder(notificationCombo, stylesCombo);
        QWidget::setTabOrder(stylesCombo, autoRaiseComboBox);
        QWidget::setTabOrder(autoRaiseComboBox, colorSchemeComboBox);
        QWidget::setTabOrder(colorSchemeComboBox, intfScaleFactorSlider);
        QWidget::setTabOrder(intfScaleFactorSlider, intfScaleFactorSpinBox);
        QWidget::setTabOrder(intfScaleFactorSpinBox, fileSkin);
        QWidget::setTabOrder(fileSkin, skinBrowse);
        QWidget::setTabOrder(skinBrowse, OneInterfaceMode);
        QWidget::setTabOrder(OneInterfaceMode, EnqueueOneInterfaceMode);
        QWidget::setTabOrder(EnqueueOneInterfaceMode, oneInstanceFromFile);
        QWidget::setTabOrder(oneInstanceFromFile, treePlaylist);
        QWidget::setTabOrder(treePlaylist, playPauseBox);
        QWidget::setTabOrder(playPauseBox, continuePlaybackComboBox);
        QWidget::setTabOrder(continuePlaybackComboBox, updatesBox);
        QWidget::setTabOrder(updatesBox, updatesDays);
        QWidget::setTabOrder(updatesDays, saveRecentlyPlayed);
        QWidget::setTabOrder(saveRecentlyPlayed, clearRecent);
        QWidget::setTabOrder(clearRecent, MetadataNetworkAccessMode);
        QWidget::setTabOrder(MetadataNetworkAccessMode, assoButton);

        retranslateUi(SPrefsInterface);

        styleStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(SPrefsInterface);
    } // setupUi

    void retranslateUi(QWidget *SPrefsInterface)
    {
        SPrefsInterface->setWindowTitle(Q_("Form", nullptr));
        langBox->setTitle(Q_("Language", nullptr));
        label->setText(Q_("Menus language:", nullptr));
        LooknfeelBox->setTitle(Q_("Look and feel", nullptr));
#if QT_CONFIG(tooltip)
        skinImage->setToolTip(Q_("VLC will use custom skin.", nullptr));
#endif // QT_CONFIG(tooltip)
        skinImage->setText(QString());
#if QT_CONFIG(tooltip)
        skins->setToolTip(Q_("VLC will use custom skin.", nullptr));
#endif // QT_CONFIG(tooltip)
        skins->setText(Q_("Use custom skin", nullptr));
#if QT_CONFIG(tooltip)
        modernImage->setToolTip(Q_("VLC will use a modern layout with\n"
"no menubar or pinned controls but\n"
"with client-side decoration.", nullptr));
#endif // QT_CONFIG(tooltip)
        modernImage->setText(QString());
#if QT_CONFIG(tooltip)
        modernButton->setToolTip(Q_("VLC will use a modern layout with\n"
"no menubar or pinned controls but\n"
"with client-side decoration.", nullptr));
#endif // QT_CONFIG(tooltip)
        modernButton->setText(Q_("Modern", nullptr));
#if QT_CONFIG(tooltip)
        classicImage->setToolTip(Q_("VLC will use a classic layout with\n"
"a menubar and pinned controls but\n"
"with no client-side decoration.", nullptr));
#endif // QT_CONFIG(tooltip)
        classicImage->setText(QString());
#if QT_CONFIG(tooltip)
        classicButton->setToolTip(Q_("VLC will use a classic layout with\n"
"a menubar and pinned controls but\n"
"with no client-side decoration.", nullptr));
#endif // QT_CONFIG(tooltip)
        classicButton->setText(Q_("Classic", nullptr));
        toolbarEditor->setText(Q_("Customize Toolbar", nullptr));
        resizingBox->setText(Q_("Resize interface to video size", nullptr));
        titleBarCheckBox->setText(Q_("Enable window titlebar", nullptr));
        embedVideo->setText(Q_("Integrate video in interface", nullptr));
        notificationComboLabel->setText(Q_("Show media change popup:", nullptr));
        systrayBox->setText(Q_("Show systray icon", nullptr));
        stylesLabel->setText(Q_("Force window style:", nullptr));
        intfScaleFactorSpinBox->setSuffix(Q_(" %", nullptr));
        colorSchemeLabel->setText(Q_("Color scheme:", nullptr));
        minimalviewBox->setText(Q_("Start in minimal view mode", nullptr));
        autoRaiseLabel->setText(Q_("Auto raising the interface:", nullptr));
        pauseMinimizedBox->setText(Q_("Pause playback when minimized", nullptr));
        scaleFactorLabel->setText(Q_("Interface scale factor:", nullptr));
        menuBarCheck->setText(Q_("Show the menu bar", nullptr));
        fsController->setText(Q_("Show controls in full screen mode", nullptr));
#if QT_CONFIG(tooltip)
        pinVideoControlsCheckbox->setToolTip(Q_("Disables autohiding of controls when playing videos", nullptr));
#endif // QT_CONFIG(tooltip)
        pinVideoControlsCheckbox->setText(Q_("Pin video controls", nullptr));
        skinBrowse->setText(Q_("Choose...", nullptr));
        skinFileLabel->setText(Q_("Skin resource file:", nullptr));
        PlaylistAndInstancesBox->setTitle(Q_("Playlist and Instances", nullptr));
        treePlaylist->setText(Q_("Display playlist tree", nullptr));
        OneInterfaceMode->setText(Q_("Allow only one instance", nullptr));
        oneInstanceFromFile->setText(Q_("Use only one instance when started from file manager", nullptr));
        EnqueueOneInterfaceMode->setText(Q_("Enqueue items into playlist in one instance mode", nullptr));
        playPauseBox->setText(Q_("Pause on the last frame of a video", nullptr));
        continuePlaybackLabel->setText(Q_("Continue playback?", nullptr));
        groupBox_2->setTitle(Q_("Privacy / Network Interaction", nullptr));
        updatesDays->setSuffix(Q_(" days", nullptr));
        updatesDays->setPrefix(Q_("Every ", nullptr));
        saveRecentlyPlayed->setText(Q_("Save recently played items", nullptr));
        clearRecent->setText(Q_("Clear Recently Played Items", nullptr));
        MetadataNetworkAccessMode->setText(Q_("Allow metadata network access", nullptr));
        updatesBox->setText(Q_("Activate updates notifier", nullptr));
        osGroupBox->setTitle(Q_("Operating System Integration", nullptr));
        assocLabel->setText(Q_("File extensions association", nullptr));
        assoButton->setText(Q_("Set up associations...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SPrefsInterface: public Ui_SPrefsInterface {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SPREFS_INTERFACE_H
