/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x1,0xac,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xa2,0x5,0x15,0xd,0x0,0x62,0x8e,0x2b,0x18,0x60,0x79,0x3,
  0x53,0xb2,0x4f,0x71,0xe5,0xea,0xeb,0x1b,0x2d,0x3c,0xdf,0x6a,0x3b,0x9f,0x42,0xef,
  0xc,0x1,0x90,0xe0,0x22,0x80,0x4b,0x4f,0x78,0x46,0x6a,0xed,0xd7,0xe8,0x47,0x93,
  0x2f,0x69,0xca,0xeb,0x34,0x69,0xb7,0x78,0xd2,0xf8,0x95,0x3b,0x1b,0xb2,0x60,0xdb,
  0x49,0x13,0xee,0x59,0x72,0x94,0x22,0xf7,0xb2,0x4f,0x37,0xc9,0xd9,0xa2,0xce,0x3c,
  0x93,0x46,0x7b,0xb5,0xe4,0x3d,0x39,0xa2,0xd9,0x74,0xac,0x2b,0xf9,0x53,0xf,0x77,
  0xf1,0x4f,0x1e,0xf5,0xcf,0x66,0xbc,0xb6,0x65,0xad,0xbd,0x15,0x6c,0x72,0x7,0x2e,
  0x55,0x5e,0xcb,0x8f,0x2b,0xd6,0xdb,0x5d,0xda,0x58,0xac,0xbe,0x8e,0x55,0x6b,0x1c,
  0x85,0xec,0x5d,0x8d,0x4a,0x4b,0xda,0xc8,0xb6,0xe,0x16,0x9f,0xa1,0x11,0xbb,0x7,
  0x20,0x4,0x84,0x50,0xcc,0xc9,0x48,0x47,0x27,0x5b,0xa3,0xfd,0x3,0x18,0x86,0xc1,
  0xa5,0x37,0x58,0x6f,0x30,0x82,0xef,0xac,0xbf,0x3,0xc2,0xe7,0xb7,0xe2,0xc3,0x11,
  0x6b,0x48,0x48,0x9e,0x2a,0x68,0xbf,0x18,0x95,0x1,0x80,0x81,0xa8,0xe1,0x52,0x30,
  0x24,0x49,0x41,0xab,0xd9,0x33,0x20,0x44,0x48,0xad,0xda,0x36,0x51,0x8,0x49,0x19,
  0xa5,0x10,0x42,0x46,0x99,0xc6,0x96,0x3,0xd7,0x80,0x6a,0x12,0x62,0x6e,0x61,0x2e,
  0x42,0x26,0xff,0x59,0x4a,0x3f,0x99,0x35,0xd7,0xd8,0xb8,0x60,0x11,0xc3,0x53,0x3d,
  0x4c,0xa7,0x70,0xdb,0xf8,0x87,0x83,0x94,0x77,0x97,0xbb,0x43,0x12,0x57,0xe3,0x6,
  0x2a,0x74,0x24,0x13,0x35,0xf2,0x41,0xba,0xed,0xd3,0x58,0x3d,0xd3,0x55,0xae,0x90,
  0xf0,0x92,0x66,0xd1,0x74,0x6b,0x26,0x60,0x17,0x3,0xd7,0x21,0xad,0xf4,0xf9,0x73,
  0x46,0xa7,0xcd,0xd,0xa4,0x92,0xc1,0xd8,0xf4,0x5,0x6e,0x28,0x10,0x57,0x84,0x7d,
  0x73,0xb0,0x84,0xed,0x4a,0xa6,0xb4,0xf7,0xb5,0x63,0xd8,0x64,0xc7,0xf6,0x50,0x4,
  0x65,0x41,0x75,0x2,0x5d,0x21,0x84,0xd3,0x85,0x68,0xd,0x97,0x94,0x12,0x2e,0x82,
  0x6a,0xb3,0xcc,0x48,0xcf,0xf5,0xcb,0xa9,0x5,0x62,0x8d,0x9,0x57,0xcb,0x41,0x65,
  0xe5,0x61,0x8e,0xd1,0x76,0x98,0x1b,0xea,0x5d,0xb6,0x9d,0x78,0xad,0xae,0x46,0x7a,
  0xd6,0x7a,0xa3,0xa2,0x38,0x82,0x12,0x9b,0xc3,0xbe,0x9e,0x22,0xa0,0xfe,0x21,0x1e,
  0xcc,0x41,0x44,0xe2,0x2c,0xd5,0x14,0x62,0x1f,0x16,0x56,0xe0,0x93,0x8,0xec,0xb8,
  0xcb,0xda,0xb,0xf8,0x5,0xc8,0x8,0xcf,0x58,0x25,0x1e,0xaf,0xb6,0x90,0x86,0xdf,
  0xad,0x3f,0xd2,0x74,0x74,0x29,0xfb,0x80,0x0,0xb5,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // VLC
  0x0,0x3,
  0x0,0x0,0x5b,0x3,
  0x0,0x56,
  0x0,0x4c,0x0,0x43,
    // MediaLibrary
  0x0,0xc,
  0x8,0x8e,0xd6,0x59,
  0x0,0x4d,
  0x0,0x65,0x0,0x64,0x0,0x69,0x0,0x61,0x0,0x4c,0x0,0x69,0x0,0x62,0x0,0x72,0x0,0x61,0x0,0x72,0x0,0x79,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC/MediaLibrary
  0x0,0x0,0x0,0x22,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x5,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC/MediaLibrary/qmldir
  0x0,0x0,0x0,0x40,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_medialibrary_assets)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_medialibrary_assets)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_medialibrary_assets)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_medialibrary_assets)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_medialibrary_assets)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_medialibrary_assets)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
