// /qt/qml/VLC/Widgets/VideoProgressBar.qml
#include <QtQml/qqmlprivate.h>
#include <QtCore/qalgorithms.h>
#include <QtCore/qdatetime.h>
#include <QtCore/qobject.h>
#include <QtCore/qrandom.h>
#include <QtCore/qstring.h>
#include <QtCore/qstringlist.h>
#include <QtCore/qtimezone.h>
#include <QtCore/qurl.h>
#include <QtCore/qvariant.h>
#include <QtQml/qjsengine.h>
#include <QtQml/qjsprimitivevalue.h>
#include <QtQml/qjsvalue.h>
#include <QtQml/qqmlcomponent.h>
#include <QtQml/qqmlcontext.h>
#include <QtQml/qqmlengine.h>
#include <QtQml/qqmllist.h>
#include <cmath>
#include <limits>
#include <type_traits>
namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_Widgets_VideoProgressBar_qml {
extern const unsigned char qmlData alignas(16) [];
extern const unsigned char qmlData alignas(16) [] = {

0x71,0x76,0x34,0x63,0x64,0x61,0x74,0x61,
0x49,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2c,0x11,0x0,0x0,0x60,0x8e,0x4,0x6c,
0xf5,0x8a,0xed,0x67,0x3a,0xa6,0x52,0x5b,
0xf8,0x2f,0xbf,0x77,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x23,0x0,0x0,0x0,
0x30,0x0,0x0,0x0,0x20,0x6,0x0,0x0,
0xd,0x0,0x0,0x0,0xc8,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xfc,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xfc,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xfc,0x0,0x0,0x0,
0x23,0x0,0x0,0x0,0xfc,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x88,0x1,0x0,0x0,
0x3,0x0,0x0,0x0,0x90,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0xa8,0x1,0x0,0x0,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x40,0xc,0x0,0x0,
0xa8,0x1,0x0,0x0,0x8,0x2,0x0,0x0,
0x58,0x2,0x0,0x0,0xa8,0x2,0x0,0x0,
0x38,0x3,0x0,0x0,0x88,0x3,0x0,0x0,
0xd8,0x3,0x0,0x0,0x28,0x4,0x0,0x0,
0x78,0x4,0x0,0x0,0xc8,0x4,0x0,0x0,
0x18,0x5,0x0,0x0,0x70,0x5,0x0,0x0,
0xd0,0x5,0x0,0x0,0x53,0x2,0x0,0x0,
0x53,0x2,0x0,0x0,0x70,0x2,0x0,0x0,
0x64,0x2,0x0,0x0,0x53,0x0,0x0,0x0,
0xb3,0x0,0x0,0x0,0x80,0x2,0x0,0x0,
0x93,0x2,0x0,0x0,0x43,0x0,0x0,0x0,
0x90,0x0,0x0,0x0,0xb3,0x2,0x0,0x0,
0x90,0x1,0x0,0x0,0x93,0x0,0x0,0x0,
0xa4,0x2,0x0,0x0,0x43,0x0,0x0,0x0,
0x90,0x0,0x0,0x0,0xb3,0x2,0x0,0x0,
0x30,0x1,0x0,0x0,0xb3,0x2,0x0,0x0,
0x50,0x1,0x0,0x0,0xb3,0x2,0x0,0x0,
0x70,0x1,0x0,0x0,0xd3,0x1,0x0,0x0,
0xc0,0x2,0x0,0x0,0x43,0x0,0x0,0x0,
0x80,0x0,0x0,0x0,0xd3,0x0,0x0,0x0,
0xd0,0x2,0x0,0x0,0xe0,0x2,0x0,0x0,
0x43,0x0,0x0,0x0,0x80,0x0,0x0,0x0,
0x13,0x1,0x0,0x0,0xf0,0x2,0x0,0x0,
0x13,0x1,0x0,0x0,0xb0,0x1,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x40,0xf5,0x7f,
0x4,0x0,0x0,0x0,0x0,0xc0,0x3,0x0,
0x1,0x0,0x0,0x0,0x0,0xc0,0x3,0x0,
0x44,0x0,0x0,0x0,0x15,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0xc,0x0,0x0,0x0,
0x1b,0x0,0x50,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x1b,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x0,0x18,0x7,
0x14,0x1,0xa,0x2e,0x1,0x3c,0x2,0x18,
0xb,0xac,0x3,0x7,0x2,0xa,0x18,0x6,
0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x5,0x0,0x0,0x0,
0xa,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x20,0x0,0x50,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x20,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x4,0x18,0x6,
0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0xf,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x24,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x24,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x5,0x3c,0x6,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x68,0x0,0x0,0x0,0x24,0x0,0x0,0x0,
0x1a,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x4,0x0,
0xff,0xff,0xff,0xff,0xd,0x0,0x0,0x0,
0x2e,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x2e,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x10,0x0,0x0,0x0,
0x2f,0x0,0x0,0x0,0x1,0x0,0x0,0x0,
0x1c,0x0,0x0,0x0,0x2e,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x23,0x0,0x0,0x0,
0x2f,0x0,0x0,0x0,0x1,0x0,0x0,0x0,
0x2e,0x7,0x18,0x7,0x2e,0x8,0x3c,0x9,
0x18,0xc,0x10,0x2,0x9c,0xc,0x18,0xa,
0x2e,0xa,0x3c,0xb,0x18,0xc,0x2e,0xc,
0x80,0xc,0x18,0xb,0xac,0xd,0x7,0x2,
0xa,0x18,0x6,0x2,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0xa,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x33,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x33,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0xe,0x3c,0xf,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x14,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x2a,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x2a,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x10,0x3c,0x11,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x16,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x2b,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x2b,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x12,0x3c,0x13,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x18,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x2c,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x2c,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x14,0x3c,0x15,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x1f,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x36,0x0,0xd0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x36,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x16,0x3c,0x17,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x22,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x37,0x0,0xc0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x37,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x18,0x3c,0x19,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x9,0x0,0x0,0x0,
0x23,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x37,0x0,0x90,0x3,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x37,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x1a,0x3c,0x1b,
0x3c,0x1c,0x18,0x6,0x2,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x14,0x0,0x0,0x0,
0x22,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x9,0x0,0x0,0x0,
0x38,0x0,0xc0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x38,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x1d,0x3c,0x1e,
0x18,0x7,0x14,0x2,0x8,0x2e,0x1f,0x3c,
0x20,0x9e,0x8,0x80,0x7,0x18,0x6,0x2,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x44,0x0,0x0,0x0,0x7,0x0,0x0,0x0,
0x23,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,
0xff,0xff,0xff,0xff,0x7,0x0,0x0,0x0,
0x38,0x0,0x20,0x5,0x0,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x38,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x2e,0x21,0x3c,0x22,
0x18,0x6,0x2,0x0,0x0,0x0,0x0,0x0,
0xe0,0x6,0x0,0x0,0xe8,0x6,0x0,0x0,
0x0,0x7,0x0,0x0,0x18,0x7,0x0,0x0,
0x28,0x7,0x0,0x0,0x48,0x7,0x0,0x0,
0x70,0x7,0x0,0x0,0xb0,0x7,0x0,0x0,
0xc0,0x7,0x0,0x0,0xd0,0x7,0x0,0x0,
0xe8,0x7,0x0,0x0,0x18,0x8,0x0,0x0,
0x38,0x8,0x0,0x0,0x58,0x8,0x0,0x0,
0x68,0x8,0x0,0x0,0x80,0x8,0x0,0x0,
0xb8,0x8,0x0,0x0,0xd0,0x8,0x0,0x0,
0xe8,0x8,0x0,0x0,0x0,0x9,0x0,0x0,
0x10,0x9,0x0,0x0,0x40,0x9,0x0,0x0,
0x50,0x9,0x0,0x0,0x80,0x9,0x0,0x0,
0x98,0x9,0x0,0x0,0xc8,0x9,0x0,0x0,
0xe0,0x9,0x0,0x0,0x10,0xa,0x0,0x0,
0x20,0xa,0x0,0x0,0x30,0xa,0x0,0x0,
0x48,0xa,0x0,0x0,0x68,0xa,0x0,0x0,
0xa8,0xa,0x0,0x0,0xc8,0xa,0x0,0x0,
0xe0,0xa,0x0,0x0,0x18,0xb,0x0,0x0,
0x48,0xb,0x0,0x0,0x60,0xb,0x0,0x0,
0x78,0xb,0x0,0x0,0x88,0xb,0x0,0x0,
0x98,0xb,0x0,0x0,0xb0,0xb,0x0,0x0,
0xc0,0xb,0x0,0x0,0xd0,0xb,0x0,0x0,
0xe8,0xb,0x0,0x0,0x8,0xc,0x0,0x0,
0x18,0xc,0x0,0x0,0x30,0xc,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x7,0x0,0x0,0x0,0x51,0x0,0x74,0x0,
0x51,0x0,0x75,0x0,0x69,0x0,0x63,0x0,
0x6b,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x9,0x0,0x0,0x0,0x56,0x0,0x4c,0x0,
0x43,0x0,0x2e,0x0,0x53,0x0,0x74,0x0,
0x79,0x0,0x6c,0x0,0x65,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x49,0x0,0x74,0x0,
0x65,0x0,0x6d,0x0,0x0,0x0,0x0,0x0,
0xb,0x0,0x0,0x0,0x70,0x0,0x72,0x0,
0x6f,0x0,0x67,0x0,0x72,0x0,0x65,0x0,
0x73,0x0,0x73,0x0,0x42,0x0,0x61,0x0,
0x72,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xe,0x0,0x0,0x0,0x69,0x0,0x6d,0x0,
0x70,0x0,0x6c,0x0,0x69,0x0,0x63,0x0,
0x69,0x0,0x74,0x0,0x48,0x0,0x65,0x0,
0x69,0x0,0x67,0x0,0x68,0x0,0x74,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1d,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x69,0x0,0x6d,0x0,0x70,0x0,
0x6c,0x0,0x69,0x0,0x63,0x0,0x69,0x0,
0x74,0x0,0x48,0x0,0x65,0x0,0x69,0x0,
0x67,0x0,0x68,0x0,0x74,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x63,0x0,0x6c,0x0,
0x69,0x0,0x70,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x76,0x0,0x61,0x0,
0x6c,0x0,0x75,0x0,0x65,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x72,0x0,0x61,0x0,
0x64,0x0,0x69,0x0,0x75,0x0,0x73,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x15,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x72,0x0,0x61,0x0,0x64,0x0,
0x69,0x0,0x75,0x0,0x73,0x0,0x0,0x0,
0xc,0x0,0x0,0x0,0x43,0x0,0x6f,0x0,
0x6c,0x0,0x6f,0x0,0x72,0x0,0x43,0x0,
0x6f,0x0,0x6e,0x0,0x74,0x0,0x65,0x0,
0x78,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0xc,0x0,0x0,0x0,0x63,0x0,0x6f,0x0,
0x6c,0x0,0x6f,0x0,0x72,0x0,0x43,0x0,
0x6f,0x0,0x6e,0x0,0x74,0x0,0x65,0x0,
0x78,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x74,0x0,0x68,0x0,
0x65,0x0,0x6d,0x0,0x65,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x63,0x0,0x6f,0x0,
0x6c,0x0,0x6f,0x0,0x72,0x0,0x53,0x0,
0x65,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x17,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x63,0x0,0x6f,0x0,0x6c,0x0,
0x6f,0x0,0x72,0x0,0x53,0x0,0x65,0x0,
0x74,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x9,0x0,0x0,0x0,0x52,0x0,0x65,0x0,
0x63,0x0,0x74,0x0,0x61,0x0,0x6e,0x0,
0x67,0x0,0x6c,0x0,0x65,0x0,0x0,0x0,
0x9,0x0,0x0,0x0,0x72,0x0,0x65,0x0,
0x63,0x0,0x74,0x0,0x61,0x0,0x6e,0x0,
0x67,0x0,0x6c,0x0,0x65,0x0,0x0,0x0,
0x7,0x0,0x0,0x0,0x61,0x0,0x6e,0x0,
0x63,0x0,0x68,0x0,0x6f,0x0,0x72,0x0,
0x73,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x6c,0x0,0x65,0x0,
0x66,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x13,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x6c,0x0,0x65,0x0,0x66,0x0,
0x74,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x72,0x0,0x69,0x0,
0x67,0x0,0x68,0x0,0x74,0x0,0x0,0x0,
0x14,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x72,0x0,0x69,0x0,0x67,0x0,
0x68,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x62,0x0,0x6f,0x0,
0x74,0x0,0x74,0x0,0x6f,0x0,0x6d,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x15,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x62,0x0,0x6f,0x0,0x74,0x0,
0x74,0x0,0x6f,0x0,0x6d,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x68,0x0,0x65,0x0,
0x69,0x0,0x67,0x0,0x68,0x0,0x74,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x15,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x68,0x0,0x65,0x0,0x69,0x0,
0x67,0x0,0x68,0x0,0x74,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x63,0x0,0x6f,0x0,
0x6c,0x0,0x6f,0x0,0x72,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x77,0x0,0x68,0x0,
0x69,0x0,0x74,0x0,0x65,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x47,0x0,0x72,0x0,
0x61,0x0,0x64,0x0,0x69,0x0,0x65,0x0,
0x6e,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0xb,0x0,0x0,0x0,0x6f,0x0,0x72,0x0,
0x69,0x0,0x65,0x0,0x6e,0x0,0x74,0x0,
0x61,0x0,0x74,0x0,0x69,0x0,0x6f,0x0,
0x6e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1a,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x6f,0x0,0x72,0x0,0x69,0x0,
0x65,0x0,0x6e,0x0,0x74,0x0,0x61,0x0,
0x74,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xc,0x0,0x0,0x0,0x47,0x0,0x72,0x0,
0x61,0x0,0x64,0x0,0x69,0x0,0x65,0x0,
0x6e,0x0,0x74,0x0,0x53,0x0,0x74,0x0,
0x6f,0x0,0x70,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x70,0x0,0x6f,0x0,
0x73,0x0,0x69,0x0,0x74,0x0,0x69,0x0,
0x6f,0x0,0x6e,0x0,0x0,0x0,0x0,0x0,
0x17,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x70,0x0,0x6f,0x0,0x73,0x0,
0x69,0x0,0x74,0x0,0x69,0x0,0x6f,0x0,
0x6e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x14,0x0,0x0,0x0,0x65,0x0,0x78,0x0,
0x70,0x0,0x72,0x0,0x65,0x0,0x73,0x0,
0x73,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,
0x20,0x0,0x66,0x0,0x6f,0x0,0x72,0x0,
0x20,0x0,0x63,0x0,0x6f,0x0,0x6c,0x0,
0x6f,0x0,0x72,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x67,0x0,0x72,0x0,
0x61,0x0,0x64,0x0,0x69,0x0,0x65,0x0,
0x6e,0x0,0x74,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x56,0x0,0x4c,0x0,
0x43,0x0,0x53,0x0,0x74,0x0,0x79,0x0,
0x6c,0x0,0x65,0x0,0x0,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x64,0x0,0x70,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x73,0x0,0x63,0x0,
0x61,0x0,0x6c,0x0,0x65,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x53,0x0,0x6c,0x0,
0x69,0x0,0x64,0x0,0x65,0x0,0x72,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x4d,0x0,0x61,0x0,
0x74,0x0,0x68,0x0,0x0,0x0,0x0,0x0,
0x3,0x0,0x0,0x0,0x6d,0x0,0x61,0x0,
0x78,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x6,0x0,0x0,0x0,0x70,0x0,0x61,0x0,
0x72,0x0,0x65,0x0,0x6e,0x0,0x74,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xa,0x0,0x0,0x0,0x48,0x0,0x6f,0x0,
0x72,0x0,0x69,0x0,0x7a,0x0,0x6f,0x0,
0x6e,0x0,0x74,0x0,0x61,0x0,0x6c,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x66,0x0,0x67,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x7,0x0,0x0,0x0,0x70,0x0,0x72,0x0,
0x69,0x0,0x6d,0x0,0x61,0x0,0x72,0x0,
0x79,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x5,0x0,0x0,0x0,0x77,0x0,0x69,0x0,
0x64,0x0,0x74,0x0,0x68,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x10,0x0,0x0,0x0,
0x7,0x0,0x0,0x0,0x38,0x0,0x0,0x0,
0x1,0x0,0x0,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x12,0x0,0x10,0x0,
0xff,0xff,0x0,0x0,0x1,0x0,0x0,0x0,
0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x13,0x0,0x10,0x0,0xff,0xff,0x0,0x0,
0x54,0x0,0x0,0x0,0x5c,0x1,0x0,0x0,
0xcc,0x1,0x0,0x0,0x9c,0x2,0x0,0x0,
0x3c,0x3,0x0,0x0,0xdc,0x3,0x0,0x0,
0x64,0x4,0x0,0x0,0x3,0x0,0x0,0x0,
0x4,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x3,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x78,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x78,0x0,0x0,0x0,0x78,0x0,0x0,0x0,
0x0,0x0,0x6,0x0,0x78,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x8,0x1,0x0,0x0,
0x18,0x0,0x10,0x0,0x19,0x0,0x50,0x0,
0x8,0x1,0x0,0x0,0x0,0x0,0x0,0x0,
0x8,0x1,0x0,0x0,0x0,0x0,0x0,0x0,
0x8,0x0,0x0,0x0,0x4,0x0,0x0,0x20,
0x1f,0x0,0x50,0x0,0x9,0x0,0x0,0x0,
0x2,0x0,0x0,0x20,0x20,0x0,0x50,0x0,
0xc,0x0,0x0,0x0,0xb,0x0,0x0,0x80,
0x22,0x0,0x50,0x0,0xc,0x0,0x0,0x0,
0x8,0x0,0x8,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x22,0x0,0x40,0x2,
0x22,0x0,0x20,0x3,0x9,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x20,0x0,0x20,0x1,
0x20,0x0,0xa0,0x1,0x8,0x0,0x0,0x0,
0x0,0x0,0x2,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x1f,0x0,0x30,0x1,
0x1f,0x0,0xa0,0x1,0x7,0x0,0x0,0x0,
0x0,0x0,0x1,0x0,0x1,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x1d,0x0,0x50,0x0,
0x1d,0x0,0xb0,0x0,0x5,0x0,0x0,0x0,
0x0,0x0,0x7,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x1b,0x0,0x50,0x0,
0x1b,0x0,0x50,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x8,0x0,0x2,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x27,0x0,0x50,0x0,
0x27,0x0,0x50,0x0,0xb,0x0,0x0,0x0,
0xd,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x1,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x6c,0x0,0x0,0x0,
0x22,0x0,0x20,0x3,0x23,0x0,0x90,0x0,
0x6c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x6c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xe,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x2,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x24,0x0,0x90,0x0,0x24,0x0,0x30,0x1,
0x0,0x0,0x0,0x0,0x10,0x0,0x0,0x0,
0x11,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x5,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0xcc,0x0,0x0,0x0,
0x27,0x0,0x50,0x0,0x28,0x0,0x90,0x0,
0xcc,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0xcc,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x24,0x0,0x0,0x0,0x0,0x0,0x8,0x0,
0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x35,0x0,0x90,0x0,0x35,0x0,0x30,0x1,
0x9,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x33,0x0,0x90,0x0,0x33,0x0,0x10,0x1,
0x1b,0x0,0x0,0x0,0x0,0x0,0x3,0x0,
0x0,0x0,0x0,0x0,0x1c,0x0,0x0,0x0,
0x32,0x0,0x90,0x0,0x32,0x0,0x0,0x1,
0x19,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x3,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2e,0x0,0x90,0x0,0x2e,0x0,0x10,0x1,
0x12,0x0,0x0,0x0,0x0,0x0,0xa,0x0,
0x3,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2a,0x0,0x90,0x0,0x2a,0x0,0x10,0x1,
0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x3,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x9c,0x0,0x0,0x0,
0x2a,0x0,0x90,0x0,0x0,0x0,0x0,0x0,
0x9c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x9c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x17,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x7,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2c,0x0,0x10,0x1,0x2c,0x0,0x90,0x1,
0x15,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x6,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2b,0x0,0x10,0x1,0x2b,0x0,0x80,0x1,
0x13,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x5,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x2a,0x0,0x10,0x1,0x2a,0x0,0x70,0x1,
0x0,0x0,0x0,0x0,0x1d,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x3,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x9c,0x0,0x0,0x0,
0x35,0x0,0x30,0x1,0x0,0x0,0x0,0x0,
0x9c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x9c,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1e,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x8,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x36,0x0,0xd0,0x0,0x36,0x0,0xa0,0x1,
0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x0,
0x5,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x37,0x0,0xd0,0x0,0x37,0x0,0xd0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x0,
0x6,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0xd0,0x0,0x38,0x0,0xd0,0x0,
0x0,0x0,0x0,0x0,0x20,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x2,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x84,0x0,0x0,0x0,
0x37,0x0,0xd0,0x0,0x0,0x0,0x0,0x0,
0x84,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x84,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1b,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0xa,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x37,0x0,0x90,0x3,0x37,0x0,0x0,0x4,
0x21,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0x9,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x37,0x0,0xc0,0x1,0x37,0x0,0x60,0x2,
0x0,0x0,0x0,0x0,0x20,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x0,0x0,0xff,0xff,
0xff,0xff,0xff,0xff,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x54,0x0,0x0,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x2,0x0,0x54,0x0,0x0,0x0,
0x0,0x0,0x0,0x0,0x84,0x0,0x0,0x0,
0x38,0x0,0xd0,0x0,0x0,0x0,0x0,0x0,
0x84,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x84,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x1b,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0xc,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0x20,0x5,0x38,0x0,0x90,0x5,
0x21,0x0,0x0,0x0,0x0,0x0,0x7,0x0,
0xb,0x0,0x0,0x0,0x0,0x0,0x0,0x0,
0x38,0x0,0xc0,0x1,0x38,0x0,0x60,0x2,
0x0,0x0,0x0,0x0
};
extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[] = {
{ 1, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<int>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for radius at line 32, column 5
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
int v2_0;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
{
double retrieved;
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(4, &retrieved)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadScopeObjectPropertyLookup(4);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<int *>(argv[0]) = int();
}
return;
}
}
s.v2_0 = QJSNumberCoercion::toInteger(std::move(retrieved));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<int *>(argv[0]) = s.v2_0;
}
return;
}
 },{ 3, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<double>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for height at line 46, column 9
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_16, markStack);
    aotContext->mark(v2_4, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_16 = nullptr;
QObject *v2_4 = nullptr;
double v2_10;
double v2_12;
double v2_18;
double v2_22;
double v2_24;
double v2_28;
double v2_6;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
{
}
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(6);
#endif
while (!aotContext->loadContextIdLookup(8, &s.v2_4)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(6);
#endif
aotContext->initLoadContextIdLookup(8);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
{
int retrieved;
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(8);
#endif
while (!aotContext->getObjectLookup(9, s.v2_4, &retrieved)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(8);
#endif
aotContext->initGetObjectLookup(9, s.v2_4);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
s.v2_6 = double(std::move(retrieved));
}
{
}
// generate_StoreReg
{
}
// generate_LoadInt
s.v2_10 = double(2);
{
}
// generate_Mul
s.v2_12 = (s.v2_6 * s.v2_10);
{
}
// generate_StoreReg
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(18);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(10, &s.v2_16)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(18);
#endif
aotContext->initLoadScopeObjectPropertyLookup(10);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(20);
#endif
while (!aotContext->getObjectLookup(11, s.v2_16, &s.v2_18)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(20);
#endif
aotContext->initGetObjectLookup(11, s.v2_16);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(24);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(12, &s.v2_22)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(24);
#endif
aotContext->initLoadScopeObjectPropertyLookup(12);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_Add
s.v2_24 = (s.v2_18 + s.v2_22);
{
}
// generate_StoreReg
{
}
// generate_CallPropertyLookup
{
const double arg1 = s.v2_12;
const double arg2 = s.v2_24;
s.v2_28 = [&]() { 
auto  tmpMax = (qIsNull(arg2) && qIsNull(arg1) && std::copysign(1.0, arg2) == 1) ? arg2 : ((arg2 > arg1 || std::isnan(arg2)) ? arg2 : arg1);
return tmpMax;
}();
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_28;
}
return;
}
 },{ 4, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<double>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for radius at line 51, column 9
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
double v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadContextIdLookup(14, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadContextIdLookup(14);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
{
int retrieved;
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
while (!aotContext->getObjectLookup(15, s.v2_0, &retrieved)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(15, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
s.v2_2 = double(std::move(retrieved));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_2;
}
return;
}
 },{ 5, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for left at line 42, column 9
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
    aotContext->mark(v2_2, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
QVariant v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(16, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadScopeObjectPropertyLookup(16);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
s.v2_2 = QVariant(aotContext->lookupResultMetaType(17));
while (!aotContext->getObjectLookup(17, s.v2_0, s.v2_2.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(17, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
s.v2_2 = QVariant(aotContext->lookupResultMetaType(17));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    if (!s.v2_2.isValid())
        aotContext->setReturnValueUndefined();
    const QMetaType returnType = s.v2_2.metaType();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0], s.v2_2.data());
}
return;
}
 },{ 6, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for right at line 43, column 9
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
    aotContext->mark(v2_2, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
QVariant v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(18, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadScopeObjectPropertyLookup(18);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
s.v2_2 = QVariant(aotContext->lookupResultMetaType(19));
while (!aotContext->getObjectLookup(19, s.v2_0, s.v2_2.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(19, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
s.v2_2 = QVariant(aotContext->lookupResultMetaType(19));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    if (!s.v2_2.isValid())
        aotContext->setReturnValueUndefined();
    const QMetaType returnType = s.v2_2.metaType();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0], s.v2_2.data());
}
return;
}
 },{ 7, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for bottom at line 44, column 9
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
    aotContext->mark(v2_2, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
QVariant v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadScopeObjectPropertyLookup(20, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadScopeObjectPropertyLookup(20);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
s.v2_2 = QVariant(aotContext->lookupResultMetaType(21));
while (!aotContext->getObjectLookup(21, s.v2_0, s.v2_2.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(21, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickAnchorLine"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
s.v2_2 = QVariant(aotContext->lookupResultMetaType(21));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    if (!s.v2_2.isValid())
        aotContext->setReturnValueUndefined();
    const QMetaType returnType = s.v2_2.metaType();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0], s.v2_2.data());
}
return;
}
 },{ 8, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = []() { static const auto t = QMetaType::fromName("QQuickGradient::Orientation"); return t; }();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for orientation at line 54, column 13
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
int v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
while (!aotContext->getEnumLookup(23, &s.v2_2)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetEnumLookup(23, []() { static const auto t = QMetaType::fromName("QQuickGradient*"); return t; }().metaObject(), "Orientation", "Horizontal");
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QQuickGradient::Orientation"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    const QMetaType returnType = QMetaType::fromType<int>();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0], &s.v2_2);
}
return;
}
 },{ 9, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<double>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for position at line 55, column 28
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
double v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadContextIdLookup(24, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadContextIdLookup(24);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
while (!aotContext->getObjectLookup(25, s.v2_0, &s.v2_2)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(25, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_2;
}
return;
}
 },{ 11, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = QMetaType::fromType<double>();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for position at line 56, column 28
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
    aotContext->mark(v2_9, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
QObject *v2_9 = nullptr;
double v2_11;
double v2_13;
double v2_15;
double v2_2;
double v8_6;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadContextIdLookup(29, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadContextIdLookup(29);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
while (!aotContext->getObjectLookup(30, s.v2_0, &s.v2_2)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(30, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_StoreReg
{
}
// generate_MoveConst
s.v8_6 = double(1);
{
}
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(11);
#endif
while (!aotContext->loadContextIdLookup(31, &s.v2_9)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(11);
#endif
aotContext->initLoadContextIdLookup(31);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(13);
#endif
while (!aotContext->getObjectLookup(32, s.v2_9, &s.v2_11)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(13);
#endif
aotContext->initGetObjectLookup(32, s.v2_9);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    *static_cast<double *>(argv[0]) = double();
}
return;
}
}
{
}
// generate_Div
QT_WARNING_PUSH
QT_WARNING_DISABLE_MSVC(4723) // potential divide by 0
s.v2_13 = (s.v8_6 / s.v2_11);
QT_WARNING_POP
{
}
// generate_Add
s.v2_15 = (s.v2_2 + s.v2_13);
{
}
{
}
// generate_Ret
if (argv[0]) {
    *static_cast<double *>(argv[0]) = s.v2_15;
}
return;
}
 },{ 12, 0, [](QV4::ExecutableCompilationUnit *contextUnit, QMetaType *argTypes) {
    struct { QV4::ExecutableCompilationUnit *compilationUnit; } c { contextUnit };
    const auto *aotContext = &c;
    Q_UNUSED(aotContext);
    argTypes[0] = []() { static const auto t = QMetaType::fromName("QColor"); return t; }();
}, 
    [](const QQmlPrivate::AOTCompiledContext *aotContext, void **argv) {
Q_UNUSED(aotContext)
Q_UNUSED(argv)
// expression for color at line 56, column 82
struct Storage : QQmlPrivate::AOTTrackedLocalsStorage {
Storage(const QQmlPrivate::AOTCompiledContext *ctxt, void **a)   : aotContext(ctxt), argv(a) {}
void markObjects(QV4::MarkStack *markStack) const final {    Q_UNUSED(markStack);
    aotContext->mark(v2_0, markStack);
    aotContext->mark(v2_2, markStack);
}
const QQmlPrivate::AOTCompiledContext *aotContext;
void **argv;
QObject *v2_0 = nullptr;
QVariant v2_2;
};
Storage s(aotContext, argv);
aotContext->setLocals(&s);
// generate_LoadQmlContextPropertyLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
while (!aotContext->loadContextIdLookup(33, &s.v2_0)) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(2);
#endif
aotContext->initLoadContextIdLookup(33);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QColor"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
}
{
}
// generate_GetLookup
#ifndef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
s.v2_2 = QVariant(aotContext->lookupResultMetaType(34));
while (!aotContext->getObjectLookup(34, s.v2_0, s.v2_2.data())) {
#ifdef QT_NO_DEBUG
aotContext->setInstructionPointer(4);
#endif
aotContext->initGetObjectLookup(34, s.v2_0);
if (aotContext->engine->hasError()) {
aotContext->setReturnValueUndefined();
if (argv[0]) {
    const QMetaType returnType = []() { static const auto t = QMetaType::fromName("QColor"); return t; }();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0]);
 }
return;
}
s.v2_2 = QVariant(aotContext->lookupResultMetaType(34));
}
{
}
{
}
// generate_Ret
if (argv[0]) {
    if (!s.v2_2.isValid())
        aotContext->setReturnValueUndefined();
    const QMetaType returnType = s.v2_2.metaType();
    returnType.destruct(argv[0]);
    returnType.construct(argv[0], s.v2_2.data());
}
return;
}
 },{ 0, 0, nullptr, nullptr }};
}
}
