/*****************************************************************************
 * orient.h: image reorientation video conversion
 *****************************************************************************
 * Copyright (C) 2010 Laurent Aimar
 * Copyright (C) 2012 Rémi Denis-Courmont
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#include <stdint.h>

typedef void (*plane_transform_cb)(void *, ptrdiff_t, const void *, ptrdiff_t,
                                   int, int);

#define PLANE_TRANSFORM_MAX_ORDER 3

struct plane_transforms {
    plane_transform_cb hflip[PLANE_TRANSFORM_MAX_ORDER + 1];
    plane_transform_cb transpose[PLANE_TRANSFORM_MAX_ORDER + 1];
};

