/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x1,0xdb,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xc4,0x5,0x8d,0xe,0x0,0x16,0x93,0x39,0x1f,0x30,0xaf,0x73,
  0x1b,0x42,0x49,0x41,0x2b,0xbb,0xbf,0xf9,0x4f,0x68,0x79,0xc0,0xe,0x2b,0xec,0x55,
  0xa1,0xeb,0xdf,0x8c,0x78,0x97,0x12,0x38,0x68,0x40,0x39,0x1,0x33,0x0,0x32,0x0,
  0x2e,0x0,0xf0,0x72,0x1a,0x80,0x13,0x68,0xfa,0x54,0x73,0x92,0x43,0x7,0x89,0xeb,
  0x1a,0x7b,0x3b,0xff,0xa8,0x28,0x77,0x60,0x9e,0xe7,0x79,0xf0,0x7a,0xfa,0x7a,0x4,
  0xd,0x9f,0x72,0x96,0xcc,0x7c,0xcb,0x1d,0x37,0xe2,0x5f,0xf4,0x35,0x9c,0x28,0x11,
  0x69,0xfe,0xb0,0xb6,0x30,0x12,0x15,0x5d,0x8b,0x2a,0x2d,0x5,0x2c,0x5e,0xd1,0x32,
  0xc2,0xc4,0xd6,0x65,0xe7,0x46,0x5b,0xbe,0xe0,0xb7,0xd4,0x2a,0x62,0x85,0xd6,0xae,
  0x35,0x29,0x14,0xb7,0x1b,0x83,0xa0,0x6d,0xd7,0xfa,0xe8,0xe9,0x72,0x2e,0x25,0x22,
  0x2c,0xde,0x4a,0xf3,0x80,0x79,0x1,0xee,0x55,0x53,0xbb,0x8a,0xc4,0x6f,0xbf,0x94,
  0xac,0x49,0x27,0xf1,0x92,0x61,0x59,0x28,0xae,0x6b,0xa5,0x5c,0xa6,0xb,0xb9,0x4b,
  0x45,0x4d,0xc4,0x9f,0xa7,0xf9,0x1e,0x76,0xa8,0xcc,0xef,0xa9,0x86,0xee,0x4b,0xed,
  0x2d,0xbe,0xf9,0x93,0xca,0xcc,0x83,0x97,0xf3,0xde,0x21,0xf6,0x36,0xc2,0xfb,0x36,
  0xea,0xbf,0x43,0xba,0xb7,0x9a,0x52,0x23,0xbf,0xb2,0x4f,0x2b,0xbc,0x6,0xe9,0x2c,
  0xe9,0x93,0x9f,0x35,0xde,0x89,0xcf,0x70,0x5b,0xd4,0x47,0x3b,0xcc,0x1,0x95,0xa9,
  0xed,0x19,0x80,0x85,0xa8,0xa1,0x8a,0x48,0xd1,0x48,0x93,0x82,0x42,0x36,0xc6,0x50,
  0x44,0x94,0x9c,0xa4,0xd,0x21,0x14,0x85,0x28,0x23,0x30,0xb2,0x9a,0xb1,0x4d,0xa,
  0x8d,0x1,0xb0,0x2,0xca,0xb7,0x4f,0x46,0x23,0x51,0x22,0x9f,0x26,0x89,0x59,0x70,
  0x33,0xce,0xfc,0x22,0xbb,0x47,0x1a,0x53,0x14,0x6b,0x89,0x51,0x9e,0x69,0x29,0xe9,
  0xcb,0x43,0xdd,0x85,0x60,0x29,0x69,0x85,0x67,0x97,0x97,0x88,0x8a,0x3f,0xcf,0x3e,
  0x94,0x56,0x27,0x63,0x81,0xb5,0x44,0x4e,0xad,0x52,0xb3,0x65,0x8c,0x8c,0x8a,0xe9,
  0xc5,0xd4,0xe,0x58,0x61,0xb3,0x9c,0xd4,0x1a,0x8,0x61,0x49,0x91,0x9f,0x41,0x51,
  0x11,0x1c,0x7,0x30,0xac,0x6d,0xd2,0xd3,0xa2,0x46,0xb1,0x45,0x53,0xa4,0x21,0x1c,
  0x86,0xe7,0x67,0x6d,0xa,0xe4,0xdb,0xf1,0x14,0x72,0x10,0x1e,0x42,0x2,0xc5,0xa1,
  0x7d,0x63,0x80,0xf,0x72,0x42,0xbb,0xab,0x74,0x8c,0x22,0x46,0x45,0x20,0xc0,0x80,
  0x1,0x81,0x14,0x58,0xa2,0x53,0x59,0x9f,0xd,0x8a,0x99,0x48,0x3d,0xf6,0x53,0x11,
  0x4b,0x87,0xe6,0x3a,0x99,0x17,0xb8,0xd1,0x90,0x24,0x31,0x82,0x77,0xa2,0xc3,0x5f,
  0x86,0xe,0x52,0x5,0x9f,0xdf,0xa1,0x8b,0x27,0xd9,0xab,0xec,0x63,0x23,0x6d,0x98,
  0x32,0x5,0x3c,0x6a,0x95,0x40,0x13,0x0,0x51,0xeb,0xb,0xa,0xf5,0x28,0x91,0xd9,
  0xc4,0x7f,0xa0,0xaa,0x82,0x46,0x19,0x0,0x55,0x1,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // VLC
  0x0,0x3,
  0x0,0x0,0x5b,0x3,
  0x0,0x56,
  0x0,0x4c,0x0,0x43,
    // PlayerControls
  0x0,0xe,
  0x3,0x44,0xf,0x33,
  0x0,0x50,
  0x0,0x6c,0x0,0x61,0x0,0x79,0x0,0x65,0x0,0x72,0x0,0x43,0x0,0x6f,0x0,0x6e,0x0,0x74,0x0,0x72,0x0,0x6f,0x0,0x6c,0x0,0x73,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC/PlayerControls
  0x0,0x0,0x0,0x22,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x5,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,
  // :/qt/qml/VLC/PlayerControls/qmldir
  0x0,0x0,0x0,0x44,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9f,0x42,0x56,0x5d,0x0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_playercontrols_assets)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_playercontrols_assets)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_playercontrols_assets)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_playercontrols_assets)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_playercontrols_assets)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_playercontrols_assets)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
