#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'messages_panel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef MESSAGES_PANEL_H
#define MESSAGES_PANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/native/customwidgets.hpp"

QT_BEGIN_NAMESPACE

class Ui_messagesPanelWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *mainTab;
    QWidget *tab;
    QGridLayout *msgLayout;
    QPlainTextEdit *messages;
    QLabel *label;
    QVLCDebugLevelSpinBox *verbosityBox;
    QLabel *label_2;
    QLineEdit *filterEdit;
    QPushButton *saveLogButton;
    QWidget *tab_3;
    QGridLayout *gridLayout_2;
    QPlainTextEdit *errors;
    QWidget *tab_2;
    QGridLayout *gridLayout;
    QTreeWidget *modulesTree;
    QDialogButtonBox *bottomButtonsBox;

    void setupUi(QWidget *messagesPanelWidget)
    {
        if (messagesPanelWidget->objectName().isEmpty())
            messagesPanelWidget->setObjectName("messagesPanelWidget");
        messagesPanelWidget->resize(570, 440);
        verticalLayout_2 = new QVBoxLayout(messagesPanelWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mainTab = new QTabWidget(messagesPanelWidget);
        mainTab->setObjectName("mainTab");
        tab = new QWidget();
        tab->setObjectName("tab");
        msgLayout = new QGridLayout(tab);
        msgLayout->setObjectName("msgLayout");
        messages = new QPlainTextEdit(tab);
        messages->setObjectName("messages");
        messages->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        messages->setReadOnly(true);

        msgLayout->addWidget(messages, 0, 0, 1, 6);

        label = new QLabel(tab);
        label->setObjectName("label");

        msgLayout->addWidget(label, 1, 0, 1, 1);

        verbosityBox = new QVLCDebugLevelSpinBox(tab);
        verbosityBox->setObjectName("verbosityBox");
        verbosityBox->setWrapping(true);
        verbosityBox->setMinimum(0);
        verbosityBox->setMaximum(2);
        verbosityBox->setValue(0);

        msgLayout->addWidget(verbosityBox, 1, 1, 1, 1);

        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        msgLayout->addWidget(label_2, 1, 2, 1, 1);

        filterEdit = new QLineEdit(tab);
        filterEdit->setObjectName("filterEdit");
        filterEdit->setMinimumSize(QSize(80, 0));

        msgLayout->addWidget(filterEdit, 1, 3, 1, 1);

        saveLogButton = new QPushButton(tab);
        saveLogButton->setObjectName("saveLogButton");

        msgLayout->addWidget(saveLogButton, 1, 4, 1, 1);

        mainTab->addTab(tab, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        gridLayout_2 = new QGridLayout(tab_3);
        gridLayout_2->setObjectName("gridLayout_2");
        errors = new QPlainTextEdit(tab_3);
        errors->setObjectName("errors");
        errors->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        errors->setReadOnly(true);

        gridLayout_2->addWidget(errors, 0, 0, 1, 1);

        mainTab->addTab(tab_3, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        gridLayout = new QGridLayout(tab_2);
        gridLayout->setObjectName("gridLayout");
        modulesTree = new QTreeWidget(tab_2);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        modulesTree->setHeaderItem(__qtreewidgetitem);
        modulesTree->setObjectName("modulesTree");

        gridLayout->addWidget(modulesTree, 0, 0, 1, 1);

        mainTab->addTab(tab_2, QString());

        verticalLayout_2->addWidget(mainTab);

        bottomButtonsBox = new QDialogButtonBox(messagesPanelWidget);
        bottomButtonsBox->setObjectName("bottomButtonsBox");
        bottomButtonsBox->setStandardButtons(QDialogButtonBox::NoButton);

        verticalLayout_2->addWidget(bottomButtonsBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(verbosityBox);
        label_2->setBuddy(filterEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(messagesPanelWidget);

        mainTab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(messagesPanelWidget);
    } // setupUi

    void retranslateUi(QWidget *messagesPanelWidget)
    {
        messagesPanelWidget->setWindowTitle(Q_("Messages", nullptr));
        label->setText(Q_("&Verbosity:", nullptr));
        label_2->setText(Q_("&Filter:", nullptr));
        saveLogButton->setText(Q_("&Save as...", nullptr));
        mainTab->setTabText(mainTab->indexOf(tab), Q_("Messages", nullptr));
        mainTab->setTabText(mainTab->indexOf(tab_3), Q_("Errors", nullptr));
        mainTab->setTabText(mainTab->indexOf(tab_2), Q_("Modules Tree", nullptr));
    } // retranslateUi

};

namespace Ui {
    class messagesPanelWidget: public Ui_messagesPanelWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MESSAGES_PANEL_H
