#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'sprefs_subtitles.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef SPREFS_SUBTITLES_H
#define SPREFS_SUBTITLES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SPrefsSubtitles
{
public:
    QVBoxLayout *mainLayout;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout;
    QCheckBox *OSDBox;
    QCheckBox *OSDTitleBox;
    QComboBox *OSDTitlePos;
    QLabel *OSDTitlePosLabel;
    QCheckBox *spuActiveBox;
    QWidget *spuZone;
    QVBoxLayout *spuZoneLayout;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout1;
    QLabel *subLangLabel;
    QComboBox *encoding;
    QLineEdit *preferredLanguage;
    QLabel *encodLabel;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *fontLabel;
    QLabel *fontSizeLabel;
    QComboBox *fontSize;
    QSpacerItem *fontSizeSpacer;
    QLabel *fontColorLabel;
    QPushButton *fontColor;
    QLabel *fontEffectLabel;
    QComboBox *effect;
    QLabel *outlineColorLabel;
    QPushButton *outlineColor;
    QCheckBox *shadowCheck;
    QLabel *subsPosLabel;
    QSpinBox *subsPosition;
    QFontComboBox *font;
    QCheckBox *backgroundCheck;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout;
    QWidget *secondarySpuZone;
    QGridLayout *gridLayout2;
    QComboBox *secondarySubsAlignment;
    QLabel *secondarySubsAlignmentLabel;
    QSpacerItem *secondarySubsAlignmentSpacer;
    QLabel *secondarySubsPosLabel;
    QSpinBox *secondarySubsPosition;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SPrefsSubtitles)
    {
        if (SPrefsSubtitles->objectName().isEmpty())
            SPrefsSubtitles->setObjectName("SPrefsSubtitles");
        SPrefsSubtitles->resize(741, 416);
        mainLayout = new QVBoxLayout(SPrefsSubtitles);
        mainLayout->setObjectName("mainLayout");
        groupBox_3 = new QGroupBox(SPrefsSubtitles);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout = new QGridLayout(groupBox_3);
        gridLayout->setObjectName("gridLayout");
        OSDBox = new QCheckBox(groupBox_3);
        OSDBox->setObjectName("OSDBox");

        gridLayout->addWidget(OSDBox, 0, 0, 1, 1);

        OSDTitleBox = new QCheckBox(groupBox_3);
        OSDTitleBox->setObjectName("OSDTitleBox");

        gridLayout->addWidget(OSDTitleBox, 1, 0, 1, 1);

        OSDTitlePos = new QComboBox(groupBox_3);
        OSDTitlePos->setObjectName("OSDTitlePos");

        gridLayout->addWidget(OSDTitlePos, 1, 2, 1, 1);

        OSDTitlePosLabel = new QLabel(groupBox_3);
        OSDTitlePosLabel->setObjectName("OSDTitlePosLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(OSDTitlePosLabel->sizePolicy().hasHeightForWidth());
        OSDTitlePosLabel->setSizePolicy(sizePolicy);
        OSDTitlePosLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(OSDTitlePosLabel, 1, 1, 1, 1);


        mainLayout->addWidget(groupBox_3);

        spuActiveBox = new QCheckBox(SPrefsSubtitles);
        spuActiveBox->setObjectName("spuActiveBox");

        mainLayout->addWidget(spuActiveBox);

        spuZone = new QWidget(SPrefsSubtitles);
        spuZone->setObjectName("spuZone");
        spuZoneLayout = new QVBoxLayout(spuZone);
        spuZoneLayout->setObjectName("spuZoneLayout");
        spuZoneLayout->setContentsMargins(0, 0, 0, 0);
        groupBox_2 = new QGroupBox(spuZone);
        groupBox_2->setObjectName("groupBox_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox_2->sizePolicy().hasHeightForWidth());
        groupBox_2->setSizePolicy(sizePolicy1);
        gridLayout1 = new QGridLayout(groupBox_2);
        gridLayout1->setObjectName("gridLayout1");
        subLangLabel = new QLabel(groupBox_2);
        subLangLabel->setObjectName("subLangLabel");
        subLangLabel->setMinimumSize(QSize(250, 0));

        gridLayout1->addWidget(subLangLabel, 0, 0, 1, 1);

        encoding = new QComboBox(groupBox_2);
        encoding->setObjectName("encoding");
        encoding->setMaxVisibleItems(12);

        gridLayout1->addWidget(encoding, 1, 1, 1, 1);

        preferredLanguage = new QLineEdit(groupBox_2);
        preferredLanguage->setObjectName("preferredLanguage");

        gridLayout1->addWidget(preferredLanguage, 0, 1, 1, 1);

        encodLabel = new QLabel(groupBox_2);
        encodLabel->setObjectName("encodLabel");
        encodLabel->setMinimumSize(QSize(250, 0));

        gridLayout1->addWidget(encodLabel, 1, 0, 1, 1);


        spuZoneLayout->addWidget(groupBox_2);

        groupBox = new QGroupBox(spuZone);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        fontLabel = new QLabel(groupBox);
        fontLabel->setObjectName("fontLabel");
        fontLabel->setMinimumSize(QSize(250, 0));

        gridLayout_2->addWidget(fontLabel, 0, 0, 1, 1);

        fontSizeLabel = new QLabel(groupBox);
        fontSizeLabel->setObjectName("fontSizeLabel");
        fontSizeLabel->setMinimumSize(QSize(250, 0));

        gridLayout_2->addWidget(fontSizeLabel, 1, 0, 1, 1);

        fontSize = new QComboBox(groupBox);
        fontSize->setObjectName("fontSize");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(fontSize->sizePolicy().hasHeightForWidth());
        fontSize->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(fontSize, 1, 1, 1, 1);

        fontSizeSpacer = new QSpacerItem(10, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(fontSizeSpacer, 1, 2, 1, 1);

        fontColorLabel = new QLabel(groupBox);
        fontColorLabel->setObjectName("fontColorLabel");

        gridLayout_2->addWidget(fontColorLabel, 1, 3, 1, 1);

        fontColor = new QPushButton(groupBox);
        fontColor->setObjectName("fontColor");

        gridLayout_2->addWidget(fontColor, 1, 4, 1, 1);

        fontEffectLabel = new QLabel(groupBox);
        fontEffectLabel->setObjectName("fontEffectLabel");
        fontEffectLabel->setMinimumSize(QSize(250, 0));

        gridLayout_2->addWidget(fontEffectLabel, 2, 0, 1, 1);

        effect = new QComboBox(groupBox);
        effect->setObjectName("effect");
        sizePolicy2.setHeightForWidth(effect->sizePolicy().hasHeightForWidth());
        effect->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(effect, 2, 1, 1, 1);

        outlineColorLabel = new QLabel(groupBox);
        outlineColorLabel->setObjectName("outlineColorLabel");

        gridLayout_2->addWidget(outlineColorLabel, 2, 3, 1, 1);

        outlineColor = new QPushButton(groupBox);
        outlineColor->setObjectName("outlineColor");

        gridLayout_2->addWidget(outlineColor, 2, 4, 1, 1);

        shadowCheck = new QCheckBox(groupBox);
        shadowCheck->setObjectName("shadowCheck");

        gridLayout_2->addWidget(shadowCheck, 3, 0, 1, 1);

        subsPosLabel = new QLabel(groupBox);
        subsPosLabel->setObjectName("subsPosLabel");
        subsPosLabel->setMinimumSize(QSize(250, 0));

        gridLayout_2->addWidget(subsPosLabel, 4, 0, 1, 1);

        subsPosition = new QSpinBox(groupBox);
        subsPosition->setObjectName("subsPosition");
        subsPosition->setMaximumSize(QSize(16777215, 16777215));
        subsPosition->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        subsPosition->setMinimum(-1024);
        subsPosition->setMaximum(1024);

        gridLayout_2->addWidget(subsPosition, 4, 1, 1, 1);

        font = new QFontComboBox(groupBox);
        font->setObjectName("font");

        gridLayout_2->addWidget(font, 0, 1, 1, 4);

        backgroundCheck = new QCheckBox(groupBox);
        backgroundCheck->setObjectName("backgroundCheck");

        gridLayout_2->addWidget(backgroundCheck, 3, 1, 1, 1);


        spuZoneLayout->addWidget(groupBox);

        groupBox_5 = new QGroupBox(spuZone);
        groupBox_5->setObjectName("groupBox_5");
        sizePolicy1.setHeightForWidth(groupBox_5->sizePolicy().hasHeightForWidth());
        groupBox_5->setSizePolicy(sizePolicy1);
        verticalLayout = new QVBoxLayout(groupBox_5);
        verticalLayout->setObjectName("verticalLayout");
        secondarySpuZone = new QWidget(groupBox_5);
        secondarySpuZone->setObjectName("secondarySpuZone");
        gridLayout2 = new QGridLayout(secondarySpuZone);
        gridLayout2->setObjectName("gridLayout2");
        secondarySubsAlignment = new QComboBox(secondarySpuZone);
        secondarySubsAlignment->setObjectName("secondarySubsAlignment");

        gridLayout2->addWidget(secondarySubsAlignment, 0, 1, 1, 1);

        secondarySubsAlignmentLabel = new QLabel(secondarySpuZone);
        secondarySubsAlignmentLabel->setObjectName("secondarySubsAlignmentLabel");
        sizePolicy.setHeightForWidth(secondarySubsAlignmentLabel->sizePolicy().hasHeightForWidth());
        secondarySubsAlignmentLabel->setSizePolicy(sizePolicy);
        secondarySubsAlignmentLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout2->addWidget(secondarySubsAlignmentLabel, 0, 0, 1, 1);

        secondarySubsAlignmentSpacer = new QSpacerItem(50, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(secondarySubsAlignmentSpacer, 0, 2, 1, 1);

        secondarySubsPosLabel = new QLabel(secondarySpuZone);
        secondarySubsPosLabel->setObjectName("secondarySubsPosLabel");

        gridLayout2->addWidget(secondarySubsPosLabel, 0, 3, 1, 1);

        secondarySubsPosition = new QSpinBox(secondarySpuZone);
        secondarySubsPosition->setObjectName("secondarySubsPosition");
        secondarySubsPosition->setMaximumSize(QSize(16777215, 16777215));
        secondarySubsPosition->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        secondarySubsPosition->setMinimum(-1024);
        secondarySubsPosition->setMaximum(1024);

        gridLayout2->addWidget(secondarySubsPosition, 0, 4, 1, 1);


        verticalLayout->addWidget(secondarySpuZone);


        spuZoneLayout->addWidget(groupBox_5);


        mainLayout->addWidget(spuZone);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mainLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        OSDTitlePosLabel->setBuddy(OSDTitlePos);
        subLangLabel->setBuddy(preferredLanguage);
        encodLabel->setBuddy(encoding);
        fontLabel->setBuddy(font);
        fontSizeLabel->setBuddy(fontSize);
        fontColorLabel->setBuddy(fontColor);
        fontEffectLabel->setBuddy(effect);
        outlineColorLabel->setBuddy(outlineColor);
        subsPosLabel->setBuddy(subsPosition);
        secondarySubsAlignmentLabel->setBuddy(secondarySubsAlignment);
        secondarySubsPosLabel->setBuddy(secondarySubsPosition);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(OSDBox, OSDTitleBox);
        QWidget::setTabOrder(OSDTitleBox, OSDTitlePos);
        QWidget::setTabOrder(OSDTitlePos, spuActiveBox);
        QWidget::setTabOrder(spuActiveBox, preferredLanguage);
        QWidget::setTabOrder(preferredLanguage, encoding);
        QWidget::setTabOrder(encoding, font);
        QWidget::setTabOrder(font, fontSize);
        QWidget::setTabOrder(fontSize, fontColor);
        QWidget::setTabOrder(fontColor, effect);
        QWidget::setTabOrder(effect, outlineColor);
        QWidget::setTabOrder(outlineColor, shadowCheck);
        QWidget::setTabOrder(shadowCheck, backgroundCheck);
        QWidget::setTabOrder(backgroundCheck, subsPosition);

        retranslateUi(SPrefsSubtitles);

        QMetaObject::connectSlotsByName(SPrefsSubtitles);
    } // setupUi

    void retranslateUi(QWidget *SPrefsSubtitles)
    {
        SPrefsSubtitles->setWindowTitle(Q_("Form", nullptr));
        groupBox_3->setTitle(Q_("On Screen Display", nullptr));
        OSDBox->setText(Q_("Enable On Screen Display (OSD)", nullptr));
        OSDTitleBox->setText(Q_("Show media title on video start", nullptr));
        OSDTitlePosLabel->setText(Q_("Position", nullptr));
        spuActiveBox->setText(Q_("Enable subtitles", nullptr));
        groupBox_2->setTitle(Q_("Subtitle Language", nullptr));
        subLangLabel->setText(Q_("Preferred subtitle language", nullptr));
        encodLabel->setText(Q_("Default encoding", nullptr));
        groupBox->setTitle(Q_("Subtitle effects", nullptr));
        fontLabel->setText(Q_("Font", nullptr));
        fontSizeLabel->setText(Q_("Font size", nullptr));
        fontColorLabel->setText(Q_("Font color", nullptr));
        fontEffectLabel->setText(Q_("Outline thickness", nullptr));
        outlineColorLabel->setText(Q_("Outline color", nullptr));
        shadowCheck->setText(Q_("Add a shadow", nullptr));
        subsPosLabel->setText(Q_("Force subtitle position", nullptr));
        subsPosition->setSuffix(Q_(" px", nullptr));
        backgroundCheck->setText(Q_("Add a background", nullptr));
        groupBox_5->setTitle(Q_("Dual Subtitles", nullptr));
        secondarySubsAlignmentLabel->setText(Q_("Align", nullptr));
        secondarySubsPosLabel->setText(Q_("Offset", nullptr));
        secondarySubsPosition->setSuffix(Q_(" px", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SPrefsSubtitles: public Ui_SPrefsSubtitles {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SPREFS_SUBTITLES_H
