#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_MediaLibrary_ArtistTopBanner_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_AudioGridItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_HomeDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_HomePage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MediaView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoGroupDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicAlbums_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicAlbumsDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicAlbumsGridExpandDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicArtist_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicArtistDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicArtistsAlbums_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicAllArtists_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicArtistsDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicGenres_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicGenresDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicPlaylistsDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicTrackListDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicTracksDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_UrlListDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoAll_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoAllDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoAllSubDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_PlaylistMediaList_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_PlaylistMedia_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_PlaylistMediaDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoPlaylistsDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoGridItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoInfoExpandPanel_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoListDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_VideoGridDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MusicAlbumSectionDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MLGlobalShortcuts_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_Bookmarks_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MLContextMenu_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MLDragItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_MLTableColumns_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_MediaLibrary_ScanProgressBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/ArtistTopBanner.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_ArtistTopBanner_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/AudioGridItem.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_AudioGridItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/HomeDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_HomeDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/HomePage.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_HomePage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MediaView.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MediaView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoGroupDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoGroupDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicAlbums.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicAlbums_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicAlbumsDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicAlbumsDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicAlbumsGridExpandDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicAlbumsGridExpandDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicArtist.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicArtist_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicArtistDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicArtistDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicArtistsAlbums.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicArtistsAlbums_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicAllArtists.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicAllArtists_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicArtistsDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicArtistsDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicGenres.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicGenres_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicGenresDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicGenresDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicPlaylistsDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicPlaylistsDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicTrackListDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicTrackListDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicTracksDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicTracksDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/UrlListDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_UrlListDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoAll.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoAll_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoAllDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoAllDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoAllSubDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoAllSubDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/PlaylistMediaList.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_PlaylistMediaList_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/PlaylistMedia.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_PlaylistMedia_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/PlaylistMediaDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_PlaylistMediaDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoPlaylistsDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoPlaylistsDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoGridItem.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoGridItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoInfoExpandPanel.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoInfoExpandPanel_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoListDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoListDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/VideoGridDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_VideoGridDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MusicAlbumSectionDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MusicAlbumSectionDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MLGlobalShortcuts.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MLGlobalShortcuts_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/Bookmarks.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_Bookmarks_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MLContextMenu.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MLContextMenu_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MLDragItem.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MLDragItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/MLTableColumns.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_MLTableColumns_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/MediaLibrary/ScanProgressBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_MediaLibrary_ScanProgressBar_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_medialibrary_cachegen)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_medialibrary_cachegen))
int QT_MANGLE_NAMESPACE(qCleanupResources_medialibrary_cachegen)() {
    return 1;
}
