#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'profiles.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef PROFILES_H
#define PROFILES_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/native/customwidgets.hpp"

QT_BEGIN_NAMESPACE

class Ui_Profiles
{
public:
    QFormLayout *formLayout_4;
    QTabWidget *tabWidget;
    QWidget *muxertab;
    QVBoxLayout *verticalLayout;
    QWidget *muxer;
    QGridLayout *gridLayout_5;
    QRadioButton *FLACMux;
    QRadioButton *MOVMux;
    QRadioButton *OggMux;
    QRadioButton *MKVMux;
    QRadioButton *MJPEGMux;
    QRadioButton *PSMux;
    QRadioButton *FLVMux;
    QRadioButton *TSMux;
    QRadioButton *RAWMux;
    QRadioButton *WAVMux;
    QRadioButton *WEBMux;
    QRadioButton *MPEG1Mux;
    QRadioButton *AVIMux;
    QRadioButton *ASFMux;
    QRadioButton *MP3Mux;
    QGroupBox *muxcaps;
    QGridLayout *gridLayout_7;
    YesNoCheckBox *capsubs;
    YesNoCheckBox *capstream;
    YesNoCheckBox *capchaps;
    YesNoCheckBox *capaudio;
    YesNoCheckBox *capvideo;
    YesNoCheckBox *capmenu;
    QLabel *muxerwarning;
    QWidget *videoCodec;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *valueholder_video_enable;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *valueholder_video_copy;
    QTabWidget *transcodevideo;
    QWidget *tab_1;
    QFormLayout *formLayout_2;
    QLabel *vCodecLabel;
    QComboBox *valueholder_video_codec;
    QLabel *vBitrateLabel;
    QLabel *vFrameLabel;
    QDoubleSpinBox *valueholder_vcodec_framerate;
    QLabel *label_4;
    QLineEdit *valueholder_vcodec_custom;
    QLabel *label_3;
    QSpinBox *valueholder_vcodec_qp;
    QSpinBox *valueholder_vcodec_bitrate;
    QWidget *tab;
    QFormLayout *formLayout_6;
    QLabel *label;
    QLabel *vScaleLabel;
    QComboBox *valueholder_vcodec_scale;
    QLabel *Framesize;
    QHBoxLayout *horizontalLayout;
    QLabel *widthLabel;
    QSpinBox *valueholder_vcodec_width;
    QLabel *heightLabel;
    QSpinBox *valueholder_vcodec_height;
    QWidget *tab_2;
    QGridLayout *gridLayout;
    QListWidget *valueholder_video_filters;
    QWidget *audioCodec;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *valueholder_audio_enable;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *valueholder_audio_copy;
    QTabWidget *transcodeaudio;
    QWidget *tab_3;
    QFormLayout *formLayout;
    QLabel *aCodecLabel;
    QComboBox *valueholder_audio_codec;
    QLabel *aBitrateLabel;
    QLabel *aChannelsLabel;
    QSpinBox *valueholder_acodec_bitrate;
    QSpinBox *valueholder_acodec_channels;
    QLabel *aSampleLabel;
    QComboBox *valueholder_acodec_samplerate;
    QLabel *aGainLabel;
    QDoubleSpinBox *valueholder_acodec_gain;
    QWidget *tab_4;
    QGridLayout *gridLayout_2;
    QListWidget *valueholder_audio_filters;
    QSpacerItem *verticalSpacer;
    QWidget *subtitles;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *valueholder_subtitles_enable;
    QFormLayout *formLayout_5;
    QLabel *label_2;
    QComboBox *valueholder_subtitles_codec;
    QCheckBox *valueholder_subtitles_overlay;
    QDialogButtonBox *buttonBox;
    QLineEdit *profileLine;
    QLabel *label_5;
    QSpacerItem *verticalSpacer_2;
    QButtonGroup *buttonGroup;

    void setupUi(QWidget *Profiles)
    {
        if (Profiles->objectName().isEmpty())
            Profiles->setObjectName("Profiles");
        Profiles->resize(584, 471);
        formLayout_4 = new QFormLayout(Profiles);
        formLayout_4->setObjectName("formLayout_4");
        tabWidget = new QTabWidget(Profiles);
        tabWidget->setObjectName("tabWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        muxertab = new QWidget();
        muxertab->setObjectName("muxertab");
        verticalLayout = new QVBoxLayout(muxertab);
        verticalLayout->setObjectName("verticalLayout");
        muxer = new QWidget(muxertab);
        muxer->setObjectName("muxer");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(muxer->sizePolicy().hasHeightForWidth());
        muxer->setSizePolicy(sizePolicy1);
        gridLayout_5 = new QGridLayout(muxer);
        gridLayout_5->setObjectName("gridLayout_5");
        FLACMux = new QRadioButton(muxer);
        buttonGroup = new QButtonGroup(Profiles);
        buttonGroup->setObjectName("buttonGroup");
        buttonGroup->addButton(FLACMux);
        FLACMux->setObjectName("FLACMux");

        gridLayout_5->addWidget(FLACMux, 3, 1, 1, 1);

        MOVMux = new QRadioButton(muxer);
        buttonGroup->addButton(MOVMux);
        MOVMux->setObjectName("MOVMux");

        gridLayout_5->addWidget(MOVMux, 0, 3, 1, 1);

        OggMux = new QRadioButton(muxer);
        buttonGroup->addButton(OggMux);
        OggMux->setObjectName("OggMux");

        gridLayout_5->addWidget(OggMux, 0, 2, 1, 1);

        MKVMux = new QRadioButton(muxer);
        buttonGroup->addButton(MKVMux);
        MKVMux->setObjectName("MKVMux");

        gridLayout_5->addWidget(MKVMux, 2, 1, 1, 1);

        MJPEGMux = new QRadioButton(muxer);
        buttonGroup->addButton(MJPEGMux);
        MJPEGMux->setObjectName("MJPEGMux");

        gridLayout_5->addWidget(MJPEGMux, 1, 1, 1, 1);

        PSMux = new QRadioButton(muxer);
        buttonGroup->addButton(PSMux);
        PSMux->setObjectName("PSMux");

        gridLayout_5->addWidget(PSMux, 1, 0, 1, 1);

        FLVMux = new QRadioButton(muxer);
        buttonGroup->addButton(FLVMux);
        FLVMux->setObjectName("FLVMux");

        gridLayout_5->addWidget(FLVMux, 1, 3, 1, 1);

        TSMux = new QRadioButton(muxer);
        buttonGroup->addButton(TSMux);
        TSMux->setObjectName("TSMux");
        TSMux->setChecked(true);

        gridLayout_5->addWidget(TSMux, 0, 0, 1, 1);

        RAWMux = new QRadioButton(muxer);
        buttonGroup->addButton(RAWMux);
        RAWMux->setObjectName("RAWMux");

        gridLayout_5->addWidget(RAWMux, 2, 2, 1, 1);

        WAVMux = new QRadioButton(muxer);
        buttonGroup->addButton(WAVMux);
        WAVMux->setObjectName("WAVMux");

        gridLayout_5->addWidget(WAVMux, 1, 2, 1, 1);

        WEBMux = new QRadioButton(muxer);
        buttonGroup->addButton(WEBMux);
        WEBMux->setObjectName("WEBMux");

        gridLayout_5->addWidget(WEBMux, 0, 1, 1, 1);

        MPEG1Mux = new QRadioButton(muxer);
        buttonGroup->addButton(MPEG1Mux);
        MPEG1Mux->setObjectName("MPEG1Mux");

        gridLayout_5->addWidget(MPEG1Mux, 2, 0, 1, 1);

        AVIMux = new QRadioButton(muxer);
        buttonGroup->addButton(AVIMux);
        AVIMux->setObjectName("AVIMux");

        gridLayout_5->addWidget(AVIMux, 2, 3, 1, 1);

        ASFMux = new QRadioButton(muxer);
        buttonGroup->addButton(ASFMux);
        ASFMux->setObjectName("ASFMux");

        gridLayout_5->addWidget(ASFMux, 3, 0, 1, 1);

        MP3Mux = new QRadioButton(muxer);
        buttonGroup->addButton(MP3Mux);
        MP3Mux->setObjectName("MP3Mux");

        gridLayout_5->addWidget(MP3Mux, 3, 2, 1, 1);


        verticalLayout->addWidget(muxer);

        muxcaps = new QGroupBox(muxertab);
        muxcaps->setObjectName("muxcaps");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(muxcaps->sizePolicy().hasHeightForWidth());
        muxcaps->setSizePolicy(sizePolicy2);
        gridLayout_7 = new QGridLayout(muxcaps);
        gridLayout_7->setObjectName("gridLayout_7");
        capsubs = new YesNoCheckBox(muxcaps);
        capsubs->setObjectName("capsubs");
        capsubs->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capsubs, 0, 3, 1, 1);

        capstream = new YesNoCheckBox(muxcaps);
        capstream->setObjectName("capstream");
        capstream->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capstream, 0, 4, 1, 1);

        capchaps = new YesNoCheckBox(muxcaps);
        capchaps->setObjectName("capchaps");
        capchaps->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capchaps, 2, 4, 1, 1);

        capaudio = new YesNoCheckBox(muxcaps);
        capaudio->setObjectName("capaudio");
        capaudio->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capaudio, 2, 0, 1, 1);

        capvideo = new YesNoCheckBox(muxcaps);
        capvideo->setObjectName("capvideo");
        capvideo->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capvideo, 0, 0, 1, 1);

        capmenu = new YesNoCheckBox(muxcaps);
        capmenu->setObjectName("capmenu");
        capmenu->setFocusPolicy(Qt::NoFocus);

        gridLayout_7->addWidget(capmenu, 2, 3, 1, 1);


        verticalLayout->addWidget(muxcaps);

        muxerwarning = new QLabel(muxertab);
        muxerwarning->setObjectName("muxerwarning");
        sizePolicy2.setHeightForWidth(muxerwarning->sizePolicy().hasHeightForWidth());
        muxerwarning->setSizePolicy(sizePolicy2);
        muxerwarning->setStyleSheet(QString::fromUtf8(""));

        verticalLayout->addWidget(muxerwarning);

        tabWidget->addTab(muxertab, QString());
        videoCodec = new QWidget();
        videoCodec->setObjectName("videoCodec");
        videoCodec->setEnabled(true);
        verticalLayout_2 = new QVBoxLayout(videoCodec);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName("verticalLayout_2");
        valueholder_video_enable = new QGroupBox(videoCodec);
        valueholder_video_enable->setObjectName("valueholder_video_enable");
        valueholder_video_enable->setFlat(true);
        valueholder_video_enable->setCheckable(true);
        valueholder_video_enable->setChecked(true);
        verticalLayout_5 = new QVBoxLayout(valueholder_video_enable);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, -1, 0, -1);
        valueholder_video_copy = new QCheckBox(valueholder_video_enable);
        valueholder_video_copy->setObjectName("valueholder_video_copy");

        verticalLayout_5->addWidget(valueholder_video_copy);

        transcodevideo = new QTabWidget(valueholder_video_enable);
        transcodevideo->setObjectName("transcodevideo");
        tab_1 = new QWidget();
        tab_1->setObjectName("tab_1");
        formLayout_2 = new QFormLayout(tab_1);
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        vCodecLabel = new QLabel(tab_1);
        vCodecLabel->setObjectName("vCodecLabel");
        vCodecLabel->setEnabled(true);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, vCodecLabel);

        valueholder_video_codec = new QComboBox(tab_1);
        valueholder_video_codec->setObjectName("valueholder_video_codec");
        valueholder_video_codec->setEditable(false);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, valueholder_video_codec);

        vBitrateLabel = new QLabel(tab_1);
        vBitrateLabel->setObjectName("vBitrateLabel");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, vBitrateLabel);

        vFrameLabel = new QLabel(tab_1);
        vFrameLabel->setObjectName("vFrameLabel");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, vFrameLabel);

        valueholder_vcodec_framerate = new QDoubleSpinBox(tab_1);
        valueholder_vcodec_framerate->setObjectName("valueholder_vcodec_framerate");
        valueholder_vcodec_framerate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, valueholder_vcodec_framerate);

        label_4 = new QLabel(tab_1);
        label_4->setObjectName("label_4");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, label_4);

        valueholder_vcodec_custom = new QLineEdit(tab_1);
        valueholder_vcodec_custom->setObjectName("valueholder_vcodec_custom");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::FieldRole, valueholder_vcodec_custom);

        label_3 = new QLabel(tab_1);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy3);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, label_3);

        valueholder_vcodec_qp = new QSpinBox(tab_1);
        valueholder_vcodec_qp->setObjectName("valueholder_vcodec_qp");
        valueholder_vcodec_qp->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, valueholder_vcodec_qp);

        valueholder_vcodec_bitrate = new QSpinBox(tab_1);
        valueholder_vcodec_bitrate->setObjectName("valueholder_vcodec_bitrate");
        valueholder_vcodec_bitrate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        valueholder_vcodec_bitrate->setAccelerated(true);
        valueholder_vcodec_bitrate->setMinimum(0);
        valueholder_vcodec_bitrate->setMaximum(32768);
        valueholder_vcodec_bitrate->setSingleStep(16);
        valueholder_vcodec_bitrate->setValue(800);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, valueholder_vcodec_bitrate);

        transcodevideo->addTab(tab_1, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        formLayout_6 = new QFormLayout(tab);
        formLayout_6->setObjectName("formLayout_6");
        formLayout_6->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        label = new QLabel(tab);
        label->setObjectName("label");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy4);
        label->setWordWrap(true);

        formLayout_6->setWidget(0, QFormLayout::ItemRole::SpanningRole, label);

        vScaleLabel = new QLabel(tab);
        vScaleLabel->setObjectName("vScaleLabel");

        formLayout_6->setWidget(2, QFormLayout::ItemRole::LabelRole, vScaleLabel);

        valueholder_vcodec_scale = new QComboBox(tab);
        valueholder_vcodec_scale->setObjectName("valueholder_vcodec_scale");
        valueholder_vcodec_scale->setEditable(true);

        formLayout_6->setWidget(2, QFormLayout::ItemRole::FieldRole, valueholder_vcodec_scale);

        Framesize = new QLabel(tab);
        Framesize->setObjectName("Framesize");

        formLayout_6->setWidget(3, QFormLayout::ItemRole::LabelRole, Framesize);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        widthLabel = new QLabel(tab);
        widthLabel->setObjectName("widthLabel");
        widthLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(widthLabel);

        valueholder_vcodec_width = new QSpinBox(tab);
        valueholder_vcodec_width->setObjectName("valueholder_vcodec_width");
        valueholder_vcodec_width->setMaximum(9999);
        valueholder_vcodec_width->setSingleStep(8);

        horizontalLayout->addWidget(valueholder_vcodec_width);

        heightLabel = new QLabel(tab);
        heightLabel->setObjectName("heightLabel");
        heightLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(heightLabel);

        valueholder_vcodec_height = new QSpinBox(tab);
        valueholder_vcodec_height->setObjectName("valueholder_vcodec_height");
        valueholder_vcodec_height->setMaximum(9999);
        valueholder_vcodec_height->setSingleStep(8);

        horizontalLayout->addWidget(valueholder_vcodec_height);


        formLayout_6->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        transcodevideo->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        gridLayout = new QGridLayout(tab_2);
        gridLayout->setObjectName("gridLayout");
        valueholder_video_filters = new QListWidget(tab_2);
        valueholder_video_filters->setObjectName("valueholder_video_filters");

        gridLayout->addWidget(valueholder_video_filters, 0, 0, 2, 1);

        transcodevideo->addTab(tab_2, QString());

        verticalLayout_5->addWidget(transcodevideo);


        verticalLayout_2->addWidget(valueholder_video_enable);

        tabWidget->addTab(videoCodec, QString());
        audioCodec = new QWidget();
        audioCodec->setObjectName("audioCodec");
        verticalLayout_3 = new QVBoxLayout(audioCodec);
        verticalLayout_3->setObjectName("verticalLayout_3");
        valueholder_audio_enable = new QGroupBox(audioCodec);
        valueholder_audio_enable->setObjectName("valueholder_audio_enable");
        valueholder_audio_enable->setFlat(true);
        valueholder_audio_enable->setCheckable(true);
        verticalLayout_6 = new QVBoxLayout(valueholder_audio_enable);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, -1, 0, -1);
        valueholder_audio_copy = new QCheckBox(valueholder_audio_enable);
        valueholder_audio_copy->setObjectName("valueholder_audio_copy");

        verticalLayout_6->addWidget(valueholder_audio_copy);

        transcodeaudio = new QTabWidget(valueholder_audio_enable);
        transcodeaudio->setObjectName("transcodeaudio");
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        formLayout = new QFormLayout(tab_3);
        formLayout->setObjectName("formLayout");
        aCodecLabel = new QLabel(tab_3);
        aCodecLabel->setObjectName("aCodecLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, aCodecLabel);

        valueholder_audio_codec = new QComboBox(tab_3);
        valueholder_audio_codec->setObjectName("valueholder_audio_codec");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, valueholder_audio_codec);

        aBitrateLabel = new QLabel(tab_3);
        aBitrateLabel->setObjectName("aBitrateLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, aBitrateLabel);

        aChannelsLabel = new QLabel(tab_3);
        aChannelsLabel->setObjectName("aChannelsLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, aChannelsLabel);

        valueholder_acodec_bitrate = new QSpinBox(tab_3);
        valueholder_acodec_bitrate->setObjectName("valueholder_acodec_bitrate");
        valueholder_acodec_bitrate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        valueholder_acodec_bitrate->setMinimum(8);
        valueholder_acodec_bitrate->setMaximum(512);
        valueholder_acodec_bitrate->setValue(128);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, valueholder_acodec_bitrate);

        valueholder_acodec_channels = new QSpinBox(tab_3);
        valueholder_acodec_channels->setObjectName("valueholder_acodec_channels");
        valueholder_acodec_channels->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        valueholder_acodec_channels->setMinimum(1);
        valueholder_acodec_channels->setMaximum(10);
        valueholder_acodec_channels->setValue(2);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, valueholder_acodec_channels);

        aSampleLabel = new QLabel(tab_3);
        aSampleLabel->setObjectName("aSampleLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, aSampleLabel);

        valueholder_acodec_samplerate = new QComboBox(tab_3);
        valueholder_acodec_samplerate->setObjectName("valueholder_acodec_samplerate");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, valueholder_acodec_samplerate);

        aGainLabel = new QLabel(tab_3);
        aGainLabel->setObjectName("aGainLabel");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, aGainLabel);

        valueholder_acodec_gain = new QDoubleSpinBox(tab_3);
        valueholder_acodec_gain->setObjectName("valueholder_acodec_gain");
        valueholder_acodec_gain->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        valueholder_acodec_gain->setMinimum(0.000000000000000);
        valueholder_acodec_gain->setMaximum(400.000000000000000);
        valueholder_acodec_gain->setSingleStep(10.000000000000000);
        valueholder_acodec_gain->setValue(0.000000000000000);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, valueholder_acodec_gain);

        transcodeaudio->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        gridLayout_2 = new QGridLayout(tab_4);
        gridLayout_2->setObjectName("gridLayout_2");
        valueholder_audio_filters = new QListWidget(tab_4);
        valueholder_audio_filters->setObjectName("valueholder_audio_filters");

        gridLayout_2->addWidget(valueholder_audio_filters, 0, 0, 2, 1);

        transcodeaudio->addTab(tab_4, QString());

        verticalLayout_6->addWidget(transcodeaudio);


        verticalLayout_3->addWidget(valueholder_audio_enable);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabWidget->addTab(audioCodec, QString());
        subtitles = new QWidget();
        subtitles->setObjectName("subtitles");
        verticalLayout_4 = new QVBoxLayout(subtitles);
        verticalLayout_4->setObjectName("verticalLayout_4");
        valueholder_subtitles_enable = new QGroupBox(subtitles);
        valueholder_subtitles_enable->setObjectName("valueholder_subtitles_enable");
        valueholder_subtitles_enable->setFlat(true);
        valueholder_subtitles_enable->setCheckable(true);
        formLayout_5 = new QFormLayout(valueholder_subtitles_enable);
        formLayout_5->setObjectName("formLayout_5");
        label_2 = new QLabel(valueholder_subtitles_enable);
        label_2->setObjectName("label_2");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::LabelRole, label_2);

        valueholder_subtitles_codec = new QComboBox(valueholder_subtitles_enable);
        valueholder_subtitles_codec->setObjectName("valueholder_subtitles_codec");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::FieldRole, valueholder_subtitles_codec);

        valueholder_subtitles_overlay = new QCheckBox(valueholder_subtitles_enable);
        valueholder_subtitles_overlay->setObjectName("valueholder_subtitles_overlay");

        formLayout_5->setWidget(1, QFormLayout::ItemRole::SpanningRole, valueholder_subtitles_overlay);


        verticalLayout_4->addWidget(valueholder_subtitles_enable);

        tabWidget->addTab(subtitles, QString());

        formLayout_4->setWidget(3, QFormLayout::ItemRole::SpanningRole, tabWidget);

        buttonBox = new QDialogButtonBox(Profiles);
        buttonBox->setObjectName("buttonBox");

        formLayout_4->setWidget(4, QFormLayout::ItemRole::FieldRole, buttonBox);

        profileLine = new QLineEdit(Profiles);
        profileLine->setObjectName("profileLine");

        formLayout_4->setWidget(1, QFormLayout::ItemRole::FieldRole, profileLine);

        label_5 = new QLabel(Profiles);
        label_5->setObjectName("label_5");

        formLayout_4->setWidget(1, QFormLayout::ItemRole::LabelRole, label_5);

        verticalSpacer_2 = new QSpacerItem(20, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        formLayout_4->setItem(2, QFormLayout::ItemRole::SpanningRole, verticalSpacer_2);

#if QT_CONFIG(shortcut)
        vCodecLabel->setBuddy(valueholder_video_codec);
        vBitrateLabel->setBuddy(valueholder_vcodec_bitrate);
        vFrameLabel->setBuddy(valueholder_vcodec_framerate);
        label_4->setBuddy(valueholder_vcodec_custom);
        label_3->setBuddy(valueholder_vcodec_qp);
        vScaleLabel->setBuddy(valueholder_vcodec_scale);
        widthLabel->setBuddy(valueholder_vcodec_width);
        heightLabel->setBuddy(valueholder_vcodec_height);
        aCodecLabel->setBuddy(valueholder_audio_codec);
        aBitrateLabel->setBuddy(valueholder_acodec_bitrate);
        aChannelsLabel->setBuddy(valueholder_acodec_channels);
        aSampleLabel->setBuddy(valueholder_acodec_samplerate);
        aGainLabel->setBuddy(valueholder_acodec_gain);
        label_2->setBuddy(valueholder_subtitles_codec);
        label_5->setBuddy(profileLine);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(profileLine, tabWidget);
        QWidget::setTabOrder(tabWidget, TSMux);
        QWidget::setTabOrder(TSMux, WEBMux);
        QWidget::setTabOrder(WEBMux, OggMux);
        QWidget::setTabOrder(OggMux, MOVMux);
        QWidget::setTabOrder(MOVMux, PSMux);
        QWidget::setTabOrder(PSMux, MJPEGMux);
        QWidget::setTabOrder(MJPEGMux, WAVMux);
        QWidget::setTabOrder(WAVMux, FLVMux);
        QWidget::setTabOrder(FLVMux, MPEG1Mux);
        QWidget::setTabOrder(MPEG1Mux, MKVMux);
        QWidget::setTabOrder(MKVMux, RAWMux);
        QWidget::setTabOrder(RAWMux, AVIMux);
        QWidget::setTabOrder(AVIMux, ASFMux);
        QWidget::setTabOrder(ASFMux, FLACMux);
        QWidget::setTabOrder(FLACMux, MP3Mux);
        QWidget::setTabOrder(MP3Mux, valueholder_video_enable);
        QWidget::setTabOrder(valueholder_video_enable, valueholder_video_copy);
        QWidget::setTabOrder(valueholder_video_copy, transcodevideo);
        QWidget::setTabOrder(transcodevideo, valueholder_video_codec);
        QWidget::setTabOrder(valueholder_video_codec, valueholder_vcodec_bitrate);
        QWidget::setTabOrder(valueholder_vcodec_bitrate, valueholder_vcodec_qp);
        QWidget::setTabOrder(valueholder_vcodec_qp, valueholder_vcodec_framerate);
        QWidget::setTabOrder(valueholder_vcodec_framerate, valueholder_vcodec_custom);
        QWidget::setTabOrder(valueholder_vcodec_custom, valueholder_vcodec_scale);
        QWidget::setTabOrder(valueholder_vcodec_scale, valueholder_vcodec_width);
        QWidget::setTabOrder(valueholder_vcodec_width, valueholder_vcodec_height);
        QWidget::setTabOrder(valueholder_vcodec_height, valueholder_video_filters);
        QWidget::setTabOrder(valueholder_video_filters, valueholder_audio_enable);
        QWidget::setTabOrder(valueholder_audio_enable, valueholder_audio_copy);
        QWidget::setTabOrder(valueholder_audio_copy, transcodeaudio);
        QWidget::setTabOrder(transcodeaudio, valueholder_audio_codec);
        QWidget::setTabOrder(valueholder_audio_codec, valueholder_acodec_bitrate);
        QWidget::setTabOrder(valueholder_acodec_bitrate, valueholder_acodec_channels);
        QWidget::setTabOrder(valueholder_acodec_channels, valueholder_acodec_samplerate);
        QWidget::setTabOrder(valueholder_acodec_samplerate, valueholder_audio_filters);
        QWidget::setTabOrder(valueholder_audio_filters, valueholder_subtitles_enable);
        QWidget::setTabOrder(valueholder_subtitles_enable, valueholder_subtitles_codec);
        QWidget::setTabOrder(valueholder_subtitles_codec, valueholder_subtitles_overlay);

        retranslateUi(Profiles);

        tabWidget->setCurrentIndex(0);
        transcodevideo->setCurrentIndex(0);
        transcodeaudio->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Profiles);
    } // setupUi

    void retranslateUi(QWidget *Profiles)
    {
        Profiles->setWindowTitle(Q_("Profile edition", nullptr));
        FLACMux->setText(Q_("FLAC", nullptr));
        MOVMux->setText(Q_("MP&4/MOV", nullptr));
        OggMux->setText(Q_("Ogg/Ogm", nullptr));
        MKVMux->setText(Q_("M&KV", nullptr));
        MJPEGMux->setText(Q_("M&JPEG", nullptr));
        PSMux->setText(Q_("MPEG-PS", nullptr));
        FLVMux->setText(Q_("F&LV", nullptr));
        TSMux->setText(Q_("&MPEG-TS", nullptr));
        RAWMux->setText(Q_("RAW", nullptr));
        WAVMux->setText(Q_("WAV", nullptr));
        WEBMux->setText(Q_("Webm", nullptr));
        MPEG1Mux->setText(Q_("MPEG &1", nullptr));
        AVIMux->setText(Q_("AVI", nullptr));
        ASFMux->setText(Q_("ASF/WMV", nullptr));
        MP3Mux->setText(Q_("MP&3", nullptr));
        muxcaps->setTitle(Q_("Features", nullptr));
        capsubs->setText(Q_("Subtitles", nullptr));
        capstream->setText(Q_("Streamable", nullptr));
        capchaps->setText(Q_("Chapters", nullptr));
        capaudio->setText(Q_("Audio", nullptr));
        capvideo->setText(Q_("Video", nullptr));
        capmenu->setText(Q_("Menus", nullptr));
        muxerwarning->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(muxertab), Q_("Encapsulation", nullptr));
        valueholder_video_enable->setTitle(Q_("Video", nullptr));
        valueholder_video_copy->setText(Q_("Keep original video track", nullptr));
        vCodecLabel->setText(Q_("Codec", nullptr));
        vBitrateLabel->setText(Q_("Bitrate", nullptr));
        vFrameLabel->setText(Q_("Fra&me Rate", nullptr));
        valueholder_vcodec_framerate->setSpecialValueText(Q_("Same as source", nullptr));
        valueholder_vcodec_framerate->setPrefix(QString());
        valueholder_vcodec_framerate->setSuffix(Q_(" fps", nullptr));
        label_4->setText(Q_("Custom options", nullptr));
        label_3->setText(Q_("&Quality", nullptr));
        valueholder_vcodec_qp->setSpecialValueText(Q_("Not Used", nullptr));
        valueholder_vcodec_bitrate->setSpecialValueText(Q_("Not Used", nullptr));
        valueholder_vcodec_bitrate->setSuffix(Q_(" kb/s", nullptr));
        transcodevideo->setTabText(transcodevideo->indexOf(tab_1), Q_("Encoding parameters", nullptr));
        label->setText(Q_("You just need to fill one of the three following parameters, VLC will autodetect the other using the original aspect ratio", nullptr));
        vScaleLabel->setText(Q_("Scale", nullptr));
        Framesize->setText(Q_("Frame size", nullptr));
        widthLabel->setText(Q_("Width", nullptr));
        valueholder_vcodec_width->setSpecialValueText(Q_("Auto", nullptr));
        valueholder_vcodec_width->setSuffix(Q_("px", nullptr));
        heightLabel->setText(Q_("Height", nullptr));
        valueholder_vcodec_height->setSpecialValueText(Q_("Auto", nullptr));
        valueholder_vcodec_height->setSuffix(Q_("px", nullptr));
        transcodevideo->setTabText(transcodevideo->indexOf(tab), Q_("Resolution", nullptr));
        transcodevideo->setTabText(transcodevideo->indexOf(tab_2), Q_("Filters", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(videoCodec), Q_("Video codec", nullptr));
        valueholder_audio_enable->setTitle(Q_("Audio", nullptr));
        valueholder_audio_copy->setText(Q_("Keep original audio track", nullptr));
        aCodecLabel->setText(Q_("Codec", nullptr));
        aBitrateLabel->setText(Q_("Bitrate", nullptr));
        aChannelsLabel->setText(Q_("Channels", nullptr));
        valueholder_acodec_bitrate->setSuffix(Q_(" kb/s", nullptr));
        aSampleLabel->setText(Q_("Sa&mple Rate", nullptr));
        aGainLabel->setText(Q_("Volume", nullptr));
#if QT_CONFIG(tooltip)
        valueholder_acodec_gain->setToolTip(Q_("Boost or reduce the audio volume of the output file.\n"
"0 = No change. 100% = original volume. 200% = double volume.", nullptr));
#endif // QT_CONFIG(tooltip)
        valueholder_acodec_gain->setSpecialValueText(Q_("No change", nullptr));
        valueholder_acodec_gain->setSuffix(Q_(" %", nullptr));
        transcodeaudio->setTabText(transcodeaudio->indexOf(tab_3), Q_("Encoding parameters", nullptr));
        transcodeaudio->setTabText(transcodeaudio->indexOf(tab_4), Q_("Filters", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(audioCodec), Q_("Audio codec", nullptr));
        valueholder_subtitles_enable->setTitle(Q_("Subtitles", nullptr));
        label_2->setText(Q_("Codec", nullptr));
        valueholder_subtitles_overlay->setText(Q_("Overlay subtitles on the video", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(subtitles), Q_("Subtitles", nullptr));
        label_5->setText(Q_("Profile &Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Profiles: public Ui_Profiles {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PROFILES_H
