/****************************************************************************
** Meta object code from reading C++ file 'kirigamiwheelhandler.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "kirigamiwheelhandler.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kirigamiwheelhandler.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t {};
} // unnamed namespace

template <> constexpr inline auto Kirigami::KirigamiWheelEvent::qt_create_metaobjectdata<qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kirigami::KirigamiWheelEvent",
        "QML.Element",
        "WheelEvent",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "",
        "x",
        "y",
        "angleDelta",
        "QPointF",
        "pixelDelta",
        "buttons",
        "modifiers",
        "inverted",
        "accepted"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'x'
        QtMocHelpers::PropertyData<qreal>(7, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'y'
        QtMocHelpers::PropertyData<qreal>(8, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'angleDelta'
        QtMocHelpers::PropertyData<QPointF>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'pixelDelta'
        QtMocHelpers::PropertyData<QPointF>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'buttons'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'modifiers'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'inverted'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'accepted'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<KirigamiWheelEvent, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Kirigami::KirigamiWheelEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t>.metaTypes,
    nullptr
} };

void Kirigami::KirigamiWheelEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KirigamiWheelEvent *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->x(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->y(); break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->angleDelta(); break;
        case 3: *reinterpret_cast<QPointF*>(_v) = _t->pixelDelta(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->buttons(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->modifiers(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->inverted(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isAccepted(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 7: _t->setAccepted(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Kirigami::KirigamiWheelEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kirigami::KirigamiWheelEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami18KirigamiWheelEventE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Kirigami::KirigamiWheelEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Kirigami::WheelFilterItem::qt_create_metaobjectdata<qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kirigami::WheelFilterItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WheelFilterItem, qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kirigami::WheelFilterItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>.metaTypes,
    nullptr
} };

void Kirigami::WheelFilterItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WheelFilterItem *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Kirigami::WheelFilterItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kirigami::WheelFilterItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami15WheelFilterItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Kirigami::WheelFilterItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8Kirigami12WheelHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto Kirigami::WheelHandler::qt_create_metaobjectdata<qt_meta_tag_ZN8Kirigami12WheelHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kirigami::WheelHandler",
        "QML.Element",
        "auto",
        "targetChanged",
        "",
        "verticalStepSizeChanged",
        "horizontalStepSizeChanged",
        "pageScrollModifiersChanged",
        "filterMouseEventsChanged",
        "keyNavigationEnabledChanged",
        "blockTargetWheelChanged",
        "scrollFlickableTargetChanged",
        "wheel",
        "KirigamiWheelEvent*",
        "_k_rebindScrollBars",
        "scrollUp",
        "stepSize",
        "scrollDown",
        "scrollLeft",
        "scrollRight",
        "target",
        "QQuickItem*",
        "verticalStepSize",
        "horizontalStepSize",
        "pageScrollModifiers",
        "Qt::KeyboardModifiers",
        "filterMouseEvents",
        "keyNavigationEnabled",
        "blockTargetWheel",
        "scrollFlickableTarget"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'targetChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'verticalStepSizeChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'horizontalStepSizeChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pageScrollModifiersChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filterMouseEventsChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyNavigationEnabledChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blockTargetWheelChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scrollFlickableTargetChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wheel'
        QtMocHelpers::SignalData<void(KirigamiWheelEvent *)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 12 },
        }}),
        // Slot '_k_rebindScrollBars'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'scrollUp'
        QtMocHelpers::MethodData<bool(qreal)>(15, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 16 },
        }}),
        // Method 'scrollUp'
        QtMocHelpers::MethodData<bool()>(15, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'scrollDown'
        QtMocHelpers::MethodData<bool(qreal)>(17, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 16 },
        }}),
        // Method 'scrollDown'
        QtMocHelpers::MethodData<bool()>(17, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'scrollLeft'
        QtMocHelpers::MethodData<bool(qreal)>(18, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 16 },
        }}),
        // Method 'scrollLeft'
        QtMocHelpers::MethodData<bool()>(18, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Method 'scrollRight'
        QtMocHelpers::MethodData<bool(qreal)>(19, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QReal, 16 },
        }}),
        // Method 'scrollRight'
        QtMocHelpers::MethodData<bool()>(19, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'target'
        QtMocHelpers::PropertyData<QQuickItem*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'verticalStepSize'
        QtMocHelpers::PropertyData<qreal>(22, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 1),
        // property 'horizontalStepSize'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 2),
        // property 'pageScrollModifiers'
        QtMocHelpers::PropertyData<Qt::KeyboardModifiers>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'filterMouseEvents'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'keyNavigationEnabled'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'blockTargetWheel'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Final, 6),
        // property 'scrollFlickableTarget'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Final, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WheelHandler, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Kirigami::WheelHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<DummyFlickableScrollHandler::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami12WheelHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami12WheelHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Kirigami12WheelHandlerE_t>.metaTypes,
    nullptr
} };

void Kirigami::WheelHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WheelHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->targetChanged(); break;
        case 1: _t->verticalStepSizeChanged(); break;
        case 2: _t->horizontalStepSizeChanged(); break;
        case 3: _t->pageScrollModifiersChanged(); break;
        case 4: _t->filterMouseEventsChanged(); break;
        case 5: _t->keyNavigationEnabledChanged(); break;
        case 6: _t->blockTargetWheelChanged(); break;
        case 7: _t->scrollFlickableTargetChanged(); break;
        case 8: _t->wheel((*reinterpret_cast<std::add_pointer_t<KirigamiWheelEvent*>>(_a[1]))); break;
        case 9: _t->_k_rebindScrollBars(); break;
        case 10: { bool _r = _t->scrollUp((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->scrollUp();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->scrollDown((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->scrollDown();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->scrollLeft((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->scrollLeft();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->scrollRight((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->scrollRight();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KirigamiWheelEvent* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::targetChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::verticalStepSizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::horizontalStepSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::pageScrollModifiersChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::filterMouseEventsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::keyNavigationEnabledChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::blockTargetWheelChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)()>(_a, &WheelHandler::scrollFlickableTargetChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WheelHandler::*)(KirigamiWheelEvent * )>(_a, &WheelHandler::wheel, 8))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuickItem**>(_v) = _t->target(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->verticalStepSize(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->horizontalStepSize(); break;
        case 3: *reinterpret_cast<Qt::KeyboardModifiers*>(_v) = _t->pageScrollModifiers(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->filterMouseEvents(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->keyNavigationEnabled(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->m_blockTargetWheel; break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->m_scrollFlickableTarget; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTarget(*reinterpret_cast<QQuickItem**>(_v)); break;
        case 1: _t->setVerticalStepSize(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setHorizontalStepSize(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setPageScrollModifiers(*reinterpret_cast<Qt::KeyboardModifiers*>(_v)); break;
        case 4: _t->setFilterMouseEvents(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setKeyNavigationEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 6:
            if (QtMocHelpers::setProperty(_t->m_blockTargetWheel, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->blockTargetWheelChanged();
            break;
        case 7:
            if (QtMocHelpers::setProperty(_t->m_scrollFlickableTarget, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->scrollFlickableTargetChanged();
            break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 1: _t->resetVerticalStepSize(); break;
        case 2: _t->resetHorizontalStepSize(); break;
        case 3: _t->resetPageScrollModifiers(); break;
        default: break;
        }
    }
}

const QMetaObject *Kirigami::WheelHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kirigami::WheelHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kirigami12WheelHandlerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return DummyFlickableScrollHandler::qt_metacast(_clname);
}

int Kirigami::WheelHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DummyFlickableScrollHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Kirigami::WheelHandler::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Kirigami::WheelHandler::verticalStepSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Kirigami::WheelHandler::horizontalStepSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Kirigami::WheelHandler::pageScrollModifiersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Kirigami::WheelHandler::filterMouseEventsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Kirigami::WheelHandler::keyNavigationEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Kirigami::WheelHandler::blockTargetWheelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Kirigami::WheelHandler::scrollFlickableTargetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Kirigami::WheelHandler::wheel(KirigamiWheelEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
