#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_VLC_Player_ControlBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PlaybackSpeed_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_LyricsFlickable_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_MiniPlayer_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PIPPlayer_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_Player_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PlayerControlLayout_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PlayerPlaylistVisibilityFSM_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PlayerToolbarVisibilityFSM_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_MinimalView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_ResumeDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_SliderBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TopBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksMenu_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksPageAudio_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksPageSpeed_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksPageSubtitle_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_TracksListPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_ControlLayout_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_ControlRepeater_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_NavigationBox_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_VLC_Player_PlayerControlbarControls_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/ControlBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_ControlBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PlaybackSpeed.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PlaybackSpeed_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/LyricsFlickable.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_LyricsFlickable_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/MiniPlayer.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_MiniPlayer_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PIPPlayer.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PIPPlayer_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/Player.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_Player_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PlayerControlLayout.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PlayerControlLayout_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PlayerPlaylistVisibilityFSM.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PlayerPlaylistVisibilityFSM_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PlayerToolbarVisibilityFSM.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PlayerToolbarVisibilityFSM_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/MinimalView.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_MinimalView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/ResumeDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_ResumeDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/SliderBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_SliderBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TopBar.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TopBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksMenu.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksMenu_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksPage.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksPageAudio.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksPageAudio_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksPageSpeed.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksPageSpeed_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksPageSubtitle.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksPageSubtitle_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/TracksListPage.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_TracksListPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/ControlLayout.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_ControlLayout_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/ControlRepeater.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_ControlRepeater_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/NavigationBox.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_NavigationBox_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/VLC/Player/PlayerControlbarControls.qml"), &QmlCacheGeneratedCode::_qt_qml_VLC_Player_PlayerControlbarControls_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_player_cachegen)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_player_cachegen))
int QT_MANGLE_NAMESPACE(qCleanupResources_player_cachegen)() {
    return 1;
}
