#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'firstrunwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef FIRSTRUNWIZARD_H
#define FIRSTRUNWIZARD_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizard>
#include <QtWidgets/QWizardPage>
#include "widgets/native/mlfolderseditor.hpp"

QT_BEGIN_NAMESPACE

class Ui_firstrun
{
public:
    QWizardPage *welcomePage;
    QGridLayout *gridLayout;
    QFrame *horizontalFrame_policy;
    QVBoxLayout *verticalLayout;
    QLabel *privacy;
    QTextEdit *policy;
    QCheckBox *privacyCheckbox;
    QFrame *horizontalFrame_prefs;
    QVBoxLayout *verticalLayout_3;
    QLabel *pref;
    QLabel *label;
    QCheckBox *enableMl;
    QWizardPage *folderPage;
    QGridLayout *gridLayout_2;
    QLabel *folderLabel;
    QPushButton *addButton;
    MLFoldersEditor *entryPoints;
    QWizardPage *colorSchemePage;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *lightButton;
    QSpacerItem *verticalSpacer_3;
    QPushButton *lightImage;
    QVBoxLayout *verticalLayout_5;
    QRadioButton *darkButton;
    QSpacerItem *verticalSpacer_4;
    QPushButton *darkImage;
    QLabel *colorLabel;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *systemButton;
    QSpacerItem *verticalSpacer_2;
    QPushButton *daynightImage;
    QSpacerItem *verticalSpacer_5;
    QLabel *explainerLabel;
    QSpacerItem *verticalSpacer_10;
    QWizardPage *layoutPage;
    QGridLayout *gridLayout_4;
    QSpacerItem *verticalSpacer_7;
    QSpacerItem *verticalSpacer_6;
    QLabel *layoutLabel;
    QVBoxLayout *verticalLayout_6;
    QRadioButton *modernButton;
    QSpacerItem *verticalSpacer_8;
    QPushButton *modernImage;
    QVBoxLayout *verticalLayout_7;
    QRadioButton *classicButton;
    QSpacerItem *verticalSpacer_9;
    QPushButton *classicImage;
    QLabel *layoutExplainer;
    QSpacerItem *verticalSpacer_11;
    QButtonGroup *layoutGroup;

    void setupUi(QWizard *firstrun)
    {
        if (firstrun->objectName().isEmpty())
            firstrun->setObjectName("firstrun");
        firstrun->resize(800, 500);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(firstrun->sizePolicy().hasHeightForWidth());
        firstrun->setSizePolicy(sizePolicy);
        firstrun->setMinimumSize(QSize(650, 400));
        welcomePage = new QWizardPage();
        welcomePage->setObjectName("welcomePage");
        welcomePage->setMinimumSize(QSize(320, 0));
        gridLayout = new QGridLayout(welcomePage);
        gridLayout->setObjectName("gridLayout");
        horizontalFrame_policy = new QFrame(welcomePage);
        horizontalFrame_policy->setObjectName("horizontalFrame_policy");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(horizontalFrame_policy->sizePolicy().hasHeightForWidth());
        horizontalFrame_policy->setSizePolicy(sizePolicy1);
        verticalLayout = new QVBoxLayout(horizontalFrame_policy);
        verticalLayout->setObjectName("verticalLayout");
        privacy = new QLabel(horizontalFrame_policy);
        privacy->setObjectName("privacy");

        verticalLayout->addWidget(privacy);

        policy = new QTextEdit(horizontalFrame_policy);
        policy->setObjectName("policy");
        policy->setEnabled(true);

        verticalLayout->addWidget(policy);

        privacyCheckbox = new QCheckBox(horizontalFrame_policy);
        privacyCheckbox->setObjectName("privacyCheckbox");
        privacyCheckbox->setChecked(true);

        verticalLayout->addWidget(privacyCheckbox);


        gridLayout->addWidget(horizontalFrame_policy, 0, 0, 1, 1);

        horizontalFrame_prefs = new QFrame(welcomePage);
        horizontalFrame_prefs->setObjectName("horizontalFrame_prefs");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(horizontalFrame_prefs->sizePolicy().hasHeightForWidth());
        horizontalFrame_prefs->setSizePolicy(sizePolicy2);
        verticalLayout_3 = new QVBoxLayout(horizontalFrame_prefs);
        verticalLayout_3->setObjectName("verticalLayout_3");
        pref = new QLabel(horizontalFrame_prefs);
        pref->setObjectName("pref");

        verticalLayout_3->addWidget(pref);

        label = new QLabel(horizontalFrame_prefs);
        label->setObjectName("label");

        verticalLayout_3->addWidget(label);

        enableMl = new QCheckBox(horizontalFrame_prefs);
        enableMl->setObjectName("enableMl");
        enableMl->setChecked(true);

        verticalLayout_3->addWidget(enableMl);


        gridLayout->addWidget(horizontalFrame_prefs, 1, 0, 1, 1);

        firstrun->addPage(welcomePage);
        folderPage = new QWizardPage();
        folderPage->setObjectName("folderPage");
        gridLayout_2 = new QGridLayout(folderPage);
        gridLayout_2->setObjectName("gridLayout_2");
        folderLabel = new QLabel(folderPage);
        folderLabel->setObjectName("folderLabel");

        gridLayout_2->addWidget(folderLabel, 0, 0, 1, 1);

        addButton = new QPushButton(folderPage);
        addButton->setObjectName("addButton");
        addButton->setMaximumSize(QSize(16, 16));

        gridLayout_2->addWidget(addButton, 0, 1, 1, 1);

        entryPoints = new MLFoldersEditor(folderPage);
        if (entryPoints->rowCount() < 1)
            entryPoints->setRowCount(1);
        entryPoints->setObjectName("entryPoints");
        entryPoints->setEditTriggers(QAbstractItemView::AllEditTriggers);
        entryPoints->setAlternatingRowColors(true);
        entryPoints->setShowGrid(false);
        entryPoints->setRowCount(1);
        entryPoints->horizontalHeader()->setProperty("showSortIndicator", QVariant(false));

        gridLayout_2->addWidget(entryPoints, 1, 0, 1, 2);

        firstrun->addPage(folderPage);
        colorSchemePage = new QWizardPage();
        colorSchemePage->setObjectName("colorSchemePage");
        gridLayout_3 = new QGridLayout(colorSchemePage);
        gridLayout_3->setObjectName("gridLayout_3");
        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout_3->addItem(verticalSpacer, 1, 0, 1, 1);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        lightButton = new QRadioButton(colorSchemePage);
        lightButton->setObjectName("lightButton");

        verticalLayout_4->addWidget(lightButton, 0, Qt::AlignHCenter);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        verticalLayout_4->addItem(verticalSpacer_3);

        lightImage = new QPushButton(colorSchemePage);
        lightImage->setObjectName("lightImage");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/theme_light.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        lightImage->setIcon(icon);
        lightImage->setIconSize(QSize(87, 154));
        lightImage->setFlat(true);

        verticalLayout_4->addWidget(lightImage);


        gridLayout_3->addLayout(verticalLayout_4, 2, 1, 1, 1);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        darkButton = new QRadioButton(colorSchemePage);
        darkButton->setObjectName("darkButton");

        verticalLayout_5->addWidget(darkButton, 0, Qt::AlignHCenter);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        verticalLayout_5->addItem(verticalSpacer_4);

        darkImage = new QPushButton(colorSchemePage);
        darkImage->setObjectName("darkImage");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/misc/theme_dark.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        darkImage->setIcon(icon1);
        darkImage->setIconSize(QSize(87, 154));
        darkImage->setFlat(true);

        verticalLayout_5->addWidget(darkImage);


        gridLayout_3->addLayout(verticalLayout_5, 2, 2, 1, 1);

        colorLabel = new QLabel(colorSchemePage);
        colorLabel->setObjectName("colorLabel");

        gridLayout_3->addWidget(colorLabel, 0, 0, 1, 1);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        systemButton = new QRadioButton(colorSchemePage);
        systemButton->setObjectName("systemButton");
        systemButton->setChecked(true);

        verticalLayout_2->addWidget(systemButton, 0, Qt::AlignHCenter);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        verticalLayout_2->addItem(verticalSpacer_2);

        daynightImage = new QPushButton(colorSchemePage);
        daynightImage->setObjectName("daynightImage");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/misc/theme_daynight.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        daynightImage->setIcon(icon2);
        daynightImage->setIconSize(QSize(87, 154));
        daynightImage->setFlat(true);

        verticalLayout_2->addWidget(daynightImage);


        gridLayout_3->addLayout(verticalLayout_2, 2, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        gridLayout_3->addItem(verticalSpacer_5, 3, 1, 1, 1);

        explainerLabel = new QLabel(colorSchemePage);
        explainerLabel->setObjectName("explainerLabel");
        explainerLabel->setScaledContents(false);
        explainerLabel->setAlignment(Qt::AlignCenter);
        explainerLabel->setWordWrap(true);

        gridLayout_3->addWidget(explainerLabel, 4, 0, 1, 3);

        verticalSpacer_10 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout_3->addItem(verticalSpacer_10, 5, 0, 1, 1);

        firstrun->addPage(colorSchemePage);
        layoutPage = new QWizardPage();
        layoutPage->setObjectName("layoutPage");
        gridLayout_4 = new QGridLayout(layoutPage);
        gridLayout_4->setObjectName("gridLayout_4");
        verticalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(verticalSpacer_7, 4, 0, 1, 2);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout_4->addItem(verticalSpacer_6, 1, 0, 1, 2);

        layoutLabel = new QLabel(layoutPage);
        layoutLabel->setObjectName("layoutLabel");

        gridLayout_4->addWidget(layoutLabel, 0, 0, 1, 1);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        modernButton = new QRadioButton(layoutPage);
        layoutGroup = new QButtonGroup(firstrun);
        layoutGroup->setObjectName("layoutGroup");
        layoutGroup->addButton(modernButton);
        modernButton->setObjectName("modernButton");
        modernButton->setChecked(true);

        verticalLayout_6->addWidget(modernButton, 0, Qt::AlignHCenter);

        verticalSpacer_8 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        verticalLayout_6->addItem(verticalSpacer_8);

        modernImage = new QPushButton(layoutPage);
        modernImage->setObjectName("modernImage");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/prefsmenu/sample_minimal.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        modernImage->setIcon(icon3);
        modernImage->setIconSize(QSize(244, 145));
        modernImage->setFlat(true);

        verticalLayout_6->addWidget(modernImage);


        gridLayout_4->addLayout(verticalLayout_6, 2, 0, 1, 1);

        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        classicButton = new QRadioButton(layoutPage);
        layoutGroup->addButton(classicButton);
        classicButton->setObjectName("classicButton");

        verticalLayout_7->addWidget(classicButton, 0, Qt::AlignHCenter);

        verticalSpacer_9 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        verticalLayout_7->addItem(verticalSpacer_9);

        classicImage = new QPushButton(layoutPage);
        classicImage->setObjectName("classicImage");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/prefsmenu/sample_complete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        classicImage->setIcon(icon4);
        classicImage->setIconSize(QSize(244, 145));
        classicImage->setFlat(true);

        verticalLayout_7->addWidget(classicImage);


        gridLayout_4->addLayout(verticalLayout_7, 2, 1, 1, 1);

        layoutExplainer = new QLabel(layoutPage);
        layoutExplainer->setObjectName("layoutExplainer");
        layoutExplainer->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(layoutExplainer, 5, 0, 1, 2);

        verticalSpacer_11 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout_4->addItem(verticalSpacer_11, 6, 0, 1, 1);

        firstrun->addPage(layoutPage);

        retranslateUi(firstrun);

        QMetaObject::connectSlotsByName(firstrun);
    } // setupUi

    void retranslateUi(QWizard *firstrun)
    {
        firstrun->setWindowTitle(Q_("Welcome", nullptr));
        privacy->setText(Q_("Privacy and Network Access Policy", nullptr));
        privacyCheckbox->setText(Q_("Allow metadata network access", nullptr));
        pref->setText(Q_("Preferences", nullptr));
        label->setText(Q_("<small>For a full list of settings, see the <b>preferences</b> tab</small>", nullptr));
        enableMl->setText(Q_("Enable the Media Library", nullptr));
        folderLabel->setText(Q_("Folders scanned by the Media Library", nullptr));
        addButton->setText(Q_("+", nullptr));
        lightButton->setText(Q_("Light", nullptr));
        lightImage->setText(QString());
        darkButton->setText(Q_("Dark", nullptr));
        darkImage->setText(QString());
        colorLabel->setText(Q_("Select a color scheme", nullptr));
        systemButton->setText(Q_("System ", nullptr));
        daynightImage->setText(QString());
        explainerLabel->setText(Q_("<i>VLC will automatically switch to dark mode accordingly with system settings</i>", nullptr));
        layoutLabel->setText(Q_("Select a layout", nullptr));
        modernButton->setText(Q_("Modern", nullptr));
        modernImage->setText(QString());
        classicButton->setText(Q_("Classic", nullptr));
        classicImage->setText(QString());
        layoutExplainer->setText(Q_("<i>VLC will use a modern layout with no menubar or pinned controls but with client-side decoration</i>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class firstrun: public Ui_firstrun {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FIRSTRUNWIZARD_H
