#define Q_(a,b) QString::fromUtf8(_(a))
/********************************************************************************
** Form generated from reading UI file 'open_net.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef OPEN_NET_H
#define OPEN_NET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OpenNetwork
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label_1;
    QLabel *examples;
    QHBoxLayout *hboxLayout;
    QComboBox *urlComboBox;
    QPushButton *clearHistoryButton;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *OpenNetwork)
    {
        if (OpenNetwork->objectName().isEmpty())
            OpenNetwork->setObjectName("OpenNetwork");
        OpenNetwork->resize(487, 273);
        vboxLayout = new QVBoxLayout(OpenNetwork);
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(OpenNetwork);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        label_1 = new QLabel(groupBox);
        label_1->setObjectName("label_1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_1->sizePolicy().hasHeightForWidth());
        label_1->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_1, 0, 0, 1, 1);

        examples = new QLabel(groupBox);
        examples->setObjectName("examples");
        examples->setStyleSheet(QString::fromUtf8("color: #838383; "));
        examples->setText(QString::fromUtf8("http://www.example.com/stream.avi\n"
"rtp://@:1234\n"
"mms://mms.examples.com/stream.asx\n"
"rtsp://server.example.org:8080/test.sdp\n"
"http://www.yourtube.com/watch?v=gg64x"));
        examples->setMargin(5);
        examples->setIndent(10);

        gridLayout->addWidget(examples, 2, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        urlComboBox = new QComboBox(groupBox);
        urlComboBox->setObjectName("urlComboBox");
        urlComboBox->setMaximumSize(QSize(600, 16777215));
        urlComboBox->setEditable(true);
        urlComboBox->setInsertPolicy(QComboBox::NoInsert);

        hboxLayout->addWidget(urlComboBox);

        clearHistoryButton = new QPushButton(groupBox);
        clearHistoryButton->setObjectName("clearHistoryButton");
        clearHistoryButton->setMaximumSize(QSize(100, 16777215));

        hboxLayout->addWidget(clearHistoryButton);


        gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);


        vboxLayout->addWidget(groupBox);

        spacerItem = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(OpenNetwork);

        QMetaObject::connectSlotsByName(OpenNetwork);
    } // setupUi

    void retranslateUi(QWidget *OpenNetwork)
    {
        OpenNetwork->setWindowTitle(Q_("Form", nullptr));
        groupBox->setTitle(Q_("Network Protocol", nullptr));
        label_1->setText(Q_("Please enter a network URL:", nullptr));
        clearHistoryButton->setText(Q_("Clear History", nullptr));
#if QT_CONFIG(tooltip)
        clearHistoryButton->setToolTip(Q_("Clear the network URL browsing history", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class OpenNetwork: public Ui_OpenNetwork {};
} // namespace Ui

QT_END_NAMESPACE

#endif // OPEN_NET_H
